C@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
C     *t2fm.f*
C*********************************************************************
C
C
C
      PROGRAM TOUGH2
C
C          @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
C          @                                                      @
C          @     TOUGH2, MODULE T2FM, VERSION 1.0, APRIL 2002     @
C          @                FRONT END FOR TMVOC                   @
C          @                                                      @
C          @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
C
C-----PROGRAM MAIN IS A HIGH-LEVEL EXECUTIVE ROUTINE.
C
C     MAIN CALLS SEVERAL LOWER-LEVEL EXECUTIVE ROUTINES, WHICH EXECUTE
C     THE ACTUAL CALCULATIONS.
C
C     ALL LARGE ARRAYS ARE DIMENSIONED IN PARAMETER STATEMENTS IN AN
C     INCLUDE FILE 'T2'. IF MODIFICATIONS IN ARRAY DIMENSIONS ARE
C     DESIRED, THESE NEED TO BE DONE ONLY IN FILE 'T2'.
C
C=======================================================================
C
C               GLOSSARY OF MAJOR ARRAYS, IN ORDER OF APPEARANCE BELOW
C
C=======================================================================
C
C     ELEMENTS:
C                  *ELEM* HOLDS ELEMENT CODE NAMES (IDENTIFIERS)
C                  *MATX* HOLDS MATERIAL DOMAIN IDENTIFIER
C                  *EVOL* HOLDS VOLUME
C                  *PHI* IS POROSITY
C                  *P* IS PRESSURE AT LAST CONVERGED TIME STEP
C                  *T* IS TEMPERATURE AT LAST CONVERGED TIME STEP
C                  *AI* IS FOR HEAT EXCHANGE WITH CONFINING LAYERS
C                  *AHT* ARE HEAT TRANSFER AREAS
C
C
C     PRIMARY VARIABLES:
C                  *X* LATEST CONVERGED VALUES OF PRIMARY VARIABLES
C                  *DX* LATEST INCREMENTS OF PRIMARY VARIABLES
C                  *DELX* SMALL INCREMENTS OF PRIMARY VARIABLES
C                         FOR NUMERICALLY COMPUTING DERIVATIVES
C                  *R* LATEST RESIDUALS
C                  *DOLD* ACCUMULATION TERMS AT BEGINNING OF TIME STEP
C                  *ROW*  SCALING FACTORS FOR MATRIX ROWS
C                  *COL*  SCALING FACTORS FOR MATRIX COLUMNS
C
C
C     CONNECTIONS (INTERFACES):
C                  *NEX1* INDICES OF FIRST ELEMENTS IN A CONNECTION
C                  *NEX2* INDICES OF SECOND ELEMENTS IN A CONNECTION
C                  *DEL1* FIRST ELEMENT NODAL DISTANCES
C                  *DEL2* SECOND ELEMENT NODAL DISTANCES
C                  *AREA* INTERFACE AREAS
C                  *BETA* COSINE OF ANGLE BETWEEN NODAL LINE AND
C                         THE VERTICAL
C                  *ISOX* ISOTROPY INDICES
C                  *GLO* HEAT FLOW RATES
C                  *ELEM1* CODE NAMES OF FIRST ELEMENT IN A CONNECTION
C                  *ELEM2* CODE NAMES OF SECOND ELEMENT
C
C
C     LINEAR EQUATION SOLVER:
C                  *IRN* ROW INDICES OF NON-ZERO MATRIX ELEMENTS
C                  *ICN* COLUMN INDICES OF NON-ZERO MATRIX ELEMENTS
C                  *WKAREA* WORKSPACE USED IN MA28
C                  *IKEEP* INTERNAL STORAGE SPACE FOR MA28
C                  *JVECT* COLUMN INDICES OF NON-ZERO MATRIX ELEMENTS
C                  *IW* INTERNAL STORAGE SPACE FOR MA28
C                  *W19A* WORKSPACE FOR SCALING ROUTINE (MC19A)
C
C
C     SINKS AND SOURCES:
C                  *F1* TABLE OF GENERATION TIMES
C                  *F2* TABLE OF GENERATION RATES
C                  *F3* TABLE OF INJECTION ENTHALPIES
C
C
C     OTHER LARGE ARRAYS:
C                  *PAR* SECONDARY (THERMOPHYSICAL) PARAMETERS FOR
C                        ALL ELEMENTS
C                  *FLO* RATES OF GAS AND LIQUID PHASE FLOWS ACROSS
C                        ALL INTERFACES
C                  *VEL* PORE VELOCITIES OF GAS AND LIQUID PHASE FLOW
C
C
C=======================================================================
C
C$$$$$$$$$ ASSIGN PARAMETERS FOR FLEXIBLE DIMENSIONING OF LARGE ARRAYS $
C
C***********************************************************************
C*                                                                     *
C*                    THE FILE 'T2' IS INCLUDED                        *
C*                                                                     *
C***********************************************************************
C
C
      INCLUDE 'T2'
C
      COMMON/MADIM/M1,M2,M3,M4,M5,M6,M7,M8
C
C=======================================================================
C
C
C$$$$$$$$$ COMMON BLOCKS FOR ELEMENTS $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
C
C     THESE BLOCKS HAVE A LENGTH OF NEL (= NUMBER OF ELEMENTS)
C
      COMMON/E1/ELEM(MNEL)
      COMMON/E2/MATX(MNEL)
      COMMON/E3/EVOL(MNEL)
      COMMON/E4/PHI(MNEL)
      COMMON/E5/P(MNEL)
      COMMON/E6/T(MNEL)
      COMMON/E7/PM(MNEL)
      COMMON/VINWES/AI(MNEL)
      COMMON/AHTRAN/AHT(MNEL)
C.....7-20-93: DEFINE COORDINATE ARRAYS
      COMMON/XYZ1/X1(MNEL)
      COMMON/XYZ2/X2(MNEL)
      COMMON/XYZ3/X3(MNEL)
C
C$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
C
C$$$$$$$$$ COMMON BLOCKS FOR PRIMARY VARIABLES $$$$$$$$$$$$$$$$$$$$$$$$$
C
C     DATA BLOCKS /P1/, /P2/, /P3/ HAVE A LENGTH OF (NK+1)*NEL.
C     NK IS THE NUMBER OF MASS BALANCE EQUATIONS PER GRID BLOCK.
C     DATA BLOCKS /P4/ THROUGH /P7/ HAVE A LENGTH OF NEQ*NEL.
C     NEQ IS THE NUMBER OF BALANCE EQUATIONS PER GRID BLOCK.
C
      COMMON/P1/X((MNK+1)*MNEL)
      COMMON/P2/DX((MNK+1)*MNEL)
      COMMON/P3/DELX((MNK+1)*MNEL)
      COMMON/P4/R(MNEQ*MNEL+1)
      COMMON/P5/DOLD(MNEQ*MNEL)
      COMMON/P6/ROW(MNEQ*MNEL)
      COMMON/P7/COL(MNEQ*MNEL)
C
C$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
C
C
C$$$$$$$$$ COMMON BLOCKS FOR CONNECTIONS $$$$$$$$$$$$$$$$$$$$$$$$$$$C
C
C     THESE BLOCKS HAVE A LENGTH OF NCON (=NUMBER OF CONNECTIONS).
C
      COMMON/C1/NEX1(MNCON)
      COMMON/C2/NEX2(MNCON)
      COMMON/C3/DEL1(MNCON)
      COMMON/C4/DEL2(MNCON)
      COMMON/C5/AREA(MNCON)
      COMMON/C6/BETA(MNCON)
      COMMON/C7/ISOX(MNCON)
      COMMON/C8/GLO(MNCON)
      COMMON/C9/ELEM1(MNCON)
      COMMON/C10/ELEM2(MNCON)
      COMMON/C11/FVD(MNCON)
C.....6-8-95: NEW ARRAY SIG(MNCON) TO HOLD COEFFICIENTS FOR
C             RADIATIVE HEAT TRANSFER.
      COMMON/C12/SIG(MNCON)
C.....6-4-93: APPEND TWO COMMON BLOCKS USED IN THE T2VOC VERSION
C             OF TOUGH2.
      COMMON/FLOVP1/FLOVG(MNPH*MNCON)
      COMMON/FLOVP2/FLOVW(MNPH*MNCON)
C.....12-25-00: APPEND COMMON BLOCKS USED FOR TMVOC
      COMMON /NEW/ INDICAT(MNEL)
      COMMON /NEW0/ INDCAT0(MNEL)
      COMMON /G32AB/ EGP(MNOGN)
C.....DIFFUSIVE FLUXES
      COMMON/FMOLDIF/FDIF(MNCON*MNPH*MNK)
      COMMON/VOLBC/VBC(MNEL)
C.....6-09-99: COMMON BLOCKS FOR T2DM
C***  BEGIN ADDITION FOR T2DM
C
      COMMON/NUMGBXYZ/NGB(3)
      COMMON/NDUPPAR/NZMULTI,NZDISF,NZDUP
      COMMON/DARVELM/DVELM((2*MNEQ+1)*MNPH*MNCON)
      COMMON/FMECDIS/FDIS(MNCON*MNPH*MNK)
C
      PARAMETER (NDIMM = 2)
      PARAMETER (NNZERO = (4*NDIMM+1)*MNEQ*NREDM)
C
      COMMON/SD1/N2Z(NNZERO)
      COMMON/SD2/IDLI(NNZERO)
      COMMON/SD3/IDLII(NNZERO)
      COMMON/SD4/IDLIII(NNZERO)
      COMMON/SD5/IFLAGG(NNZERO)
      COMMON/SD6/NFLAGG(NNZERO)
      COMMON/SD7/N3ZMRK,ID
C
C***  END ADDITION FOR T2DM
C
C$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
C
C$$$$$$$$$ COMMON BLOCKS FOR LINEAR EQUATIONS $$$$$$$$$$$$$$$$$$$$$$$$$
C
C     ARRAYS USED BY BOTH MA28 AND THE CONJUGATE GRADIENT PACKAGE.
      COMMON/L1/IRN(MNZ)
      COMMON/L2/ICN(MNZ)
      COMMON/L3/CO(MNZ+1)
      COMMON/L4/WKAREA(MNEQ*MNEL+10)
C     WKAREA HAS A LENGTH OF NEQ*NEL+10.
C
C     JVECT IS GIVEN A SLIGHTLY LARGER DIMENSION THAN NEEDED FOR
C     MA28, TO ACCOMMODATE THE CONJUGATE GRADIENTS.
      COMMON/L7/JVECT(NIWORK)
C     JVECT HAS A LENGTH OF NZ.
C
C     ARRAYS USED BY MA28 ONLY.
      COMMON/L5/IKEEP(5*MNEQ*MNEL)
C     IKEEP HAS A LENGTH OF 5*NEQ*NEL.
      COMMON/L6/IW(8*MNEQ*MNEL)
C     IW HAS A LENGTH OF 8*NEQ*NEL.
      COMMON/L8/W19A(5*MNEQ*MNEL)
C     W19A HAS A LENGTH OF 5*NEQ*NEL.
      COMMON/AMMIS/MA,IPIV,U,IAB,NZ
C
C     ARRAY USED BY CONJUGATE GRADIENT SOLVERS ONLY.
C     COMMON/CGARA6/RWORK(NRWORK)
C     COMMON/SOLVER/MATSLV,NMAXIT,ICLOSR,CLOSUR,ISYM,IUNIT,NVECTR,SEED
      COMMON/SOLL/LENW,LENIW
C
C     ARRAYS USED BY LUBAND ONLY.
      COMMON/LUB1/AB(NRWORK)
      COMMON/LUB3/NSUPDI,NSUBDI,MNZP1,MNETP1,MNELP1,NNNBIG
      COMMON/LUB4/MATORD,NSUBDG,NSUPDG,NTOTD
C
C********* END MODIFICATION.
C$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
C
C$$$$$ COMMON BLOCKS FOR SINKS/SOURCES $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
C
      COMMON/G1/F1(MGTAB)
      COMMON/G2/F2(MGTAB)
      COMMON/G3/F3(MGTAB)
      COMMON/G3A/PW(MGTAB)
C     ARRAYS F1, F2, AND F3 HOLD, RESPECTIVELY, TABLES OF TIMES, RATES,
C     AND ENTHALPIES OF TIME-DEPENDENT SINKS OR SOURCES. THEIR DIMENSIONS
C     MUST NOT BE SMALLER THAN THE TOTAL NUMBER OF SUCH DATA.
C
      COMMON/G4/ELEG(MNOGN)
      COMMON/G5/SOURCE(MNOGN)
      COMMON/G6/LTABG(MNOGN)
      COMMON/G7/G(MNOGN)
      COMMON/G8/EG(MNOGN)
      COMMON/G9/NEXG(MNOGN)
      COMMON/G10/TYPE(MNOGN)
      COMMON/G11/NGIND(MNOGN)
      COMMON/G12/LCOM(MNOGN)
      COMMON/G13/PI(MNOGN)
      COMMON/G14/PWB(MNOGN)
      COMMON/G15/HG(MNOGN)
      COMMON/G16/GPO(MNOGN)
      COMMON/G17/SDENS(MNOGN)
      COMMON/G18/SSAT(MNOGN)
      COMMON/G19/GVOL(MNOGN)
      COMMON/G20/HL(MNOGN)
      COMMON/G21/HS(MNOGN)
      COMMON/G22/QVGC(MNOGN)
      COMMON/G23/QVWC(MNOGN)
      COMMON/G24/QVOC(MNOGN)
      COMMON/G25/GRAD(MNOGN)
C     THE DIMENSION OF ALL ARRAYS IN COMMON G4 THROUGH G25 MUST BE
C     EQUAL TO OR LARGER THAN THE TOTAL NUMBER OF SINKS AND SOURCES.
C
      COMMON/G26/FF(MNPH*MNOGN)
C     THE DIMENSION OF ARRAY FF MUST BE EQUAL TO OR LARGER THAN THE
C     PRODUCT OF NUMBER OF PHASES AND TOTAL NUMBER OF SINKS AND SOURCES.
C
      COMMON/G27/FNAM(MNOGN)
      COMMON/G28/NFTAB(MNOGN)
      COMMON/G29/IFTIT(MNOGN)
      COMMON/G30/JFTIT(MNOGN)
      COMMON/G31/IJF(MNOGN)
C
C$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
C
      COMMON/SECPAR/PAR((MNEQ+1)*(MNPH*(MNB+MNK)+2)*MNEL)
C     ARRAY PAR HAS A LENGTH OF (NEQ+1)*NSEC*NEL.
C     NSEC = NPH*(NB+NK)+2 IS THE NUMBER OF SECONDARY PARAMETERS.
C     NPH IS THE NUMBER OF PHASES.
C     NB IS THE NUMBER OF THERMOPHYSICAL PARAMETERS (USUALLY 6).
C     NK IS THE NUMBER OF COMPONENTS (SPECIES).
C
      COMMON/COMPO/FLO(MNPH*MNCON)
      COMMON/PORVEL/VEL(MNPH*MNCON)
C
      COMMON/TITLE/TITLE
      CHARACTER*80 TITLE
      CHARACTER*5 ELEM,ELEM1,ELEM2,ELEG,SOURCE,VV,TYPE
CFi      LOGICAL EX
      COMMON/NN/NEL,NCON,NOGN,NK,NEQ,NPH,NB,NK1,NEQ1,NBK,NSEC,NFLUX
      COMMON/DMN/INUM,IPRINT,MCYC,MCYPR,MSEC,TZERO,TIMP1
      COMMON/SVZ/NOITE,MOP(24)
      COMMON/LDIM/LICN,LIRN
      COMMON/BC/NELA
      COMMON/V/IV,IS
      COMMON/RANT2/IRANT2
      COMMON/FF/H1
      CHARACTER*1 H1
      COMMON/SOLVR1/MATSLV,NMAXIT,NNVVCC,IIUUNN,IISSOO,NACTDI
      COMMON/SOLVR2/RITMAX,CLOSUR
      COMMON/SOLVR3/ORDRNG,OPROCS,ZPROCS,COORD
      CHARACTER*2 ORDRNG,OPROCS,ZPROCS
      CHARACTER*5 COORD
C
C$$$$$$$$$ COMMON BLOCKS FOR HYSTERESIS MODEL $$$$$$$$$$$$$$$$$$$$$$$$$$
C
ccd..feb 2008 Define integer array irun for sub-threshold DS

      COMMON/KHYST1/IHYSTER,KCYCX,IEHYS(16)
      COMMON/KHYST2/ICURV(MNEL)
      COMMON/KHYST3/KTMPO(MNEL)
      COMMON/KHYST4/irun(MNEL)
      COMMON/KHYST5/IFLG1(MNEL)
      COMMON/KHYST6/KFLAG(MNEL)
      COMMON/SAVHYS1/SOLDH(MNEL)
      COMMON/SAVHYS2/HOLD(MNEL)
      COMMON/TMPHYS1/STMPO(MNEL)
      COMMON/TMPHYS2/HTMPO(MNEL)
      COMMON/HYS11/STRN1(MNEL)
      COMMON/HYS12/STRN2(MNEL)
      COMMON/HYS13/STRN3(MNEL)
      COMMON/HYS21/HTRN1(MNEL)
      COMMON/HYS22/HTRN2(MNEL)
      COMMON/HYS23/HTRN3(MNEL)
      COMMON/HYS31/SCMP1(MNEL)
      COMMON/HYS32/SCMP2(MNEL)
      COMMON/HYS33/BB2(MNEL)
C
C     IEHYS()  : Hysteresis control parameters read through block HYSTE
C     IEHYS(1) : 
C     IEHYS(1) : flag to print information about hysteretic characteristic curves
C                =  0  no additional print out
C                >= 1  print a one-line message to the output file every time a 
C                      capillary-pressure branch switch occurs (recommended)  
C                >= 2  print a one-line message to the iTOUGH2 message (.msg) file 
C                      every time the third-order scanning imbibition curve crosses 
C                      either the first-order scanning imbibition curve or the 
C                      second-order scanning drainage curve
C
C     IEHYS(2) : flag indicating when to apply capillary-pressure branch switching
C                =  0 after convergence of time step (recommended)
C                >  0 after each Newton-Raphson iteration
C
C     IEHYS(3) : run parameter for sub-threshold saturation change
C                =  0  no branch switch unless | S| > CP(10)
C                >  0  allow branch switch after run of IEHYS(3) consecutive time 
C                      steps for which all | S| < CP(10) and all | S| are the same sign
C
CTSB
C$$$$$$$$$ COMMON BLOCKS FOR TORBENS HYSTERESIS MODEL $$$$$$$$$$$$$$$$$$
C
      COMMON/TORBENR1/SEOT(MNEL)
      COMMON/TORBENR2/SETRN1(MNEL)
      COMMON/TORBENR3/SOR(MNEL)
      COMMON/TORBENR4/SOROLD(MNEL)
      COMMON/TORBENR5/SORTMPO(MNEL)
CTSE
ccd..feb2008 more common blocks for hysteresis (save more intermediate results)
c
      common/hyssav1/strn1old(MNEL)
      common/hyssav2/strn1tmpo(MNEL)
      common/hyssav3/strn2old(MNEL)
      common/hyssav4/strn2tmpo(MNEL)
      common/hyssav5/strn3old(MNEL)
      common/hyssav6/strn3tmpo(MNEL)
      common/hyssav7/htrn1old(MNEL)
      common/hyssav8/htrn1tmpo(MNEL)
      common/hyssav9/htrn2old(MNEL)
      common/hyssav10/htrn2tmpo(MNEL)
      common/hyssav11/htrn3old(MNEL)
      common/hyssav12/htrn3tmpo(MNEL)
      common/hyssav13/scmp1old(MNEL)
      common/hyssav14/scmp1tmpo(MNEL)
      common/hyssav15/scmp2old(MNEL)
      common/hyssav16/scmp2tmpo(MNEL)
      common/hyssav17/bb2old(MNEL)
      common/hyssav18/bb2tmpo(MNEL)
      common/hyssa21/ hmin(MNEL)
      common/hyssa22/ hminold(MNEL)
      common/hyssa23/ hmintmpo(MNEL)
      common/hyssa31/ shmin(MNEL)
      common/hyssa32/ shminold(MNEL)
      common/hyssa33/ shmintmpo(MNEL)    
      common/hyssa41/ hminp(MNEL)
      common/hyssa42/ hminpold(MNEL)
      common/hyssa43/ hminptmpo(MNEL)
ccd..end

C
C
      DIMENSION VV(26)
C
CC      INQUIRE(FILE='T2INP',EXIST=EX)                                    PC
CC      IF(EX) GOTO 222                                                   PC
CC      PRINT  223                                                        PC
CC  223 FORMAT(' FILE *T2INP* DOES NOT EXIST --- IT MUST EXIST AS AN OLD',PC
CC     X' FILE: *********** STOP EXECUTION *********')                    PC
CC      STOP                                                              PC
CCC                                                                       PC
CC  222 PRINT  224                                                        PC
CC  224 FORMAT(' FILE *T2INP* EXISTS --- OPEN AS AN OLD FILE')            PC
CC      OPEN(5,FILE='T2INP',STATUS='OLD')                                 PC
C
      KCYCX=0
      IHYSTER=0
      MNELP1 = MNEL+1
      MNZP1  = MNZ+1
      MNETP1 = NREDM+1
      LENW=NRWORK
      LENIW=NIWORK
C
      H1=CHAR(12)
      PRINT 1,H1
    1 FORMAT(A1/7X,'@@@@@   @@   @  @   @@@  @  @   @@       @@@  @ ',
     A  ' @   @  ',
     1  '@  @  @      @@   @@@@@  @   @@   @   @     @@@   @  @  @   @'/
     2  7X,'  @    @  @  @  @  @     @  @  @  @     @     @  @@ @@',
     B  '  @  @',
     3  '  @     @  @    @    @  @  @  @@  @     @  @  @  @  @@  @'/
     4  7X,'  @    @  @  @  @  @ @@  @@@@    @       @@   @  @ @ @',
     C  '  @  @',
     5  '  @     @@@@    @    @  @  @  @ @ @     @@@   @  @  @ @ @'/
     6  7X,'  @    @  @  @  @  @  @  @  @   @          @  @  @   @',
     D  '  @  @',
     71X,  ' @     @  @    @    @  @  @  @  @@     @ @   @  @  @  @@'/
     8  7X,'  @     @@    @@    @@@  @  @  @@@@     @@@   @  @   @',
     E  '   @@ ',
     9  '  @@@@  @  @    @    @   @@   @   @     @  @   @@   @   @'//)
C
      PRINT 4
    4 FORMAT(20X,'TOUGH2 IS A PROGRAM FOR MULTIPHASE MULTICOMPONENT',
     X' FLOW IN PERMEABLE MEDIA, INCLUDING HEAT FLOW.'/17X,
     X'  IT IS A MEMBER OF THE MULKOM FAMILY OF CODES, DEVELOPED',
     X' AT LAWRENCE BERKELEY NATIONAL LABORATORY.')
      PRINT 5
    5 FORMAT(/26X,80('*')/26X,19('*'),41X,20('*')/
     X26X,19('*'),'     TOUGH2 - VERSION 2.1 (JULY 2011)    ',20('*')/
     X26X,19('*'),'          T2CG2 SOLVER PACKAGE           ',20('*')/
     X26X,19('*'),41X,20('*')/26X,80('*')/)
      PRINT 23
   23 FORMAT(8X,'COPYRIGHT 2011 BY THE REGENTS OF THE UNIVERSITY OF ',
     X'CALIFORNIA (SUBJECT TO APPROVAL BY THE U.S. DEPARTMENT OF ',
     X'ENERGY).')
      PRINT 24
   24 FORMAT(/
     X' NOTICE: THIS SOFTWARE WAS DEVELOPED UNDER FUNDING FROM THE ',
     X'U.S. DEPARTMENT OF ENERGY AND THE U.S. GOVERNMENT CONSEQUENTLY ',
     X'RETAINS'/' CERTAIN RIGHTS AS FOLLOWS: THE U.S. GOVERNMENT HAS ',
     X'BEEN GRANTED FOR ITSELF AND OTHERS ACTING ON ITS BEHALF A ',
     X'PAID-UP, NONEXCLUSIVE,'/' IRREVOCABLE, WORLDWIDE LICENSE IN ',
     X'THE SOFTWARE TO REPRODUCE, PREPARE DERIVATIVE WORKS, AND ',
     X'PERFORM PUBLICLY AND DISPLAY PUBLICLY.'/' BEGINNING FIVE (5) ',
     X'YEARS AFTER THE DATE PERMISSION TO ASSERT COPYRIGHT IS OBTAINED',
     X' FROM THE U.S. DEPARTMENT OF ENERGY, AND SUBJECT'/' TO ANY ',
     X'SUBSEQUENT FIVE (5) YEAR RENEWALS, THE U.S. GOVERNMENT IS ',
     X'GRANTED FOR ITSELF AND OTHERS ACTING ON ITS BEHALF A PAID-UP,'/
     X' NONEXCLUSIVE, IRREVOCABLE, WORLDWIDE LICENSE IN THE SOFTWARE ',
     X'TO REPRODUCE, PREPARE DERIVATIVE WORKS, DISTRIBUTE COPIES TO ',
     X'THE'/' PUBLIC, PERFORM PUBLICLY AND DISPLAY PUBLICLY, AND TO ',
     X'PERMIT OTHERS TO DO SO.')
C     PRINT 22
   22 FORMAT(
     X5X,'TO ENGAGE RADIATIVE HEAT TRANSFER, ENTER A COEFFICIENT "S',
     X'IGX" IN THE LAST FIELD (COLUMNS 71-80) OF A CONNECTION RECORD.'/
     X5X,'SIGX IS THE (RELATIVE) "RADIANT EMITTANCE".'/
     X5X,'ENTERING SIGX < 0 WILL CAUSE CONDUCTIVE HEAT TRANSFER TO BE ',
     X'SUPPRESSED; WILL USE ABS(SIGX) AS RADIATIVE TRANSFER ',
     X'COEFFICIENT.'/
     X5X,'INTERNALLY, SIGX WILL BE MULTIPLIED WITH THE STEFAN-',
     X'BOLTZMANN CONSTANT, WHICH IS 5.669E-8 J/(M^2 K^4 S).')
C
      M1=MNEL
      M2=MNCON
      M3=MNEQ
      M4=MNPH
      M5=MNB+MNK
      M6=MNOGN
      M7=MGTAB
      M8=MNK
C     REVISE LENGTH ASSIGNMENT OF MA28 ARRAYS
      LIRN=MNZ
      LICN=MNZ
C
      MPRIM=(MNK+1)*MNEL
      MSEC=(MNEQ+1)*(MNPH*(MNB+MNK)+2)*MNEL
      PRINT 6,MNEL,MNCON,MNEQ,MNK,MNPH,MNB,MNOGN,MGTAB
    6 FORMAT(//' PARAMETERS FOR FLEXIBLE DIMENSIONING OF MAJOR ARRAYS ',
     X'(MAIN PROGRAM) ARE AS FOLLOWS'//
     X' MNEL =',I7,'   MNCON =',I7,'   MNEQ =',I3,'   MNK =',I3,
     X'   MNPH =',I3,
     X'   MNB =',I3,'   MNOGN =',I5,'   MGTAB =',I6/' ',131('='))
C
      PRINT 7,MNEL,MNCON,MPRIM,MNOGN
    7 FORMAT(/' MAXIMUM NUMBER OF VOLUME ELEMENTS (GRID BLOCKS):',12X,
     X'MNEL  =',I8/
     X' MAXIMUM NUMBER OF CONNECTIONS (INTERFACES):',17X,'MNCON =',I8/
     X' MAXIMUM LENGTH OF PRIMARY VARIABLE ARRAYS:',18X,'MPRIM =',I8/
     X' MAXIMUM NUMBER OF GENERATION ITEMS (SINKS/SOURCES):',9X,
     X'MNOGN =',I8)
      PRINT 9,MGTAB,MSEC,MNZ,LIRN,LICN
    9 FORMAT(
     X' MAXIMUM NUMBER OF TABULAR (TIME-DEPENDENT) GENERATION DATA',
     X': MGTAB =',I8/
     X' LENGTH OF SECONDARY PARAMETER ARRAY:',24X,'MSEC  =',I8/
     X' MAXIMUM NUMBER OF JACOBIAN MATRIX ELEMENTS:',17X,'MNZ   =',I8/
     X/' LARGE LINEAR EQUATION ARRAYS: LENGTH OF IRN IS',14X,
     X'LIRN  =',I8/
     X'                               LENGTH OF ICN AND CO IS',7X,
     X'LICN  =',I8//' ',131('='))
C
      PRINT 21
   21 FORMAT(/' ARRAY DIMENSIONING IS MADE ACCORDING TO THE NEEDS OF',
     X' THE CONJUGATE GRADIENT SOLVERS'/
     X' WHEN USING LUBAND, ONLY A SMALLER-SIZE PROBLEM CAN BE',
     X' ACCOMMODATED'/' RESTRICTION WITH MA28 IS: ',
     X'{NUMBER OF ELEMENTS} + 2 * {NUMBER OF CONNECTIONS} < {MNEL',
     X' + 2* MNCON}/4'//' ',131('='))
C---*----1----*----2----*----3----*----4----*----5----*----6----*----7----*----8
C
      CALL IO
      CALL SEC(TZERO)
C
      WRITE(11,899)
C   8 FORMAT(/'      TOUGH2   1.11 CG  28 JANUARY   1994',6X,
C   8 FORMAT(/'      TOUGH2   1.12 CG  12 FEBRUARY  1997',6X,
C   8 FORMAT(/'      TOUGH2   1.20 CG  26 SEPTEMBER 1997',6X,
C 899 FORMAT(/'      TOUGH2   1.20 CG  12 FEBRUARY  1998',6X,
C 899 FORMAT(/'      TOUGH2   1.20 CG  11 MARCH     1998',6X,
C 899 FORMAT(/'      TOUGH2   2.00     19 MAY       1999',6X,
C 899 FORMAT(/'      TOUGH2   2.00      8 JUNE      1999',6X,
C 899 FORMAT(/'      TOUGH2   2.00     25 DECEMBER  2000',6X,
C 899 FORMAT(/'      TOUGH2   2.00     12 APRIL     2001',6X,
C 899 FORMAT(/'      TOUGH2   2.00     21 JUNE      2001',6X,
c 899 FORMAT(/'      TOUGH2   2.00      5 SEPTEMBER 2001',6X,
c 899 FORMAT(/'      TOUGH2   2.00     20 February  2005',6X,
c 899 FORMAT(/'      TOUGH2   2.00     20 August    2010',6X,
  899 FORMAT(/'      TOUGH2   2.1      30 JUNE      2011',6X,
     X'MAIN PROGRAM'/
     X47X,'SPECIAL VERSION FOR CONJUGATE GRADIENT PACKAGE T2CG2'/
     X47X,'INCLUDES DEFINITION OF COORDINATE ARRAYS',
     X' AND RADIATIVE HEAT TRANSFER CAPABILITY'/
     X47X,'HAS ADDITIONAL COMMON BLOCKS FOR TMVOC')
C
      READ(5, 2)TITLE
    2 FORMAT(A80)
      PRINT 3,TITLE
    3 FORMAT(/1X,131('=')//5X,'PROBLEM TITLE:  ',A80//)
C
C-----FETCH INPUT DATA.
C
      CALL INPUT
C     IF(IHYSTER.LT.0) OPEN(82,FILE='HYSTERESIS',STATUS='UNKNOWN')
CHyamB
      IF(IHYSTER.LT.0) THEN
        OPEN(82,FILE='HYSTERESIS',STATUS='UNKNOWN')
        write(82,*) 'VARIABLES = Time "Liq. Sat." "Cap. Pres."
     X             "curve" "krg" "krl"'
        write(82,*) 'ZONE t="HYSTERESIS"'
      ENDIF
CHyamE
C     EVALUATE FLOATING POINT PROCESSOR.
      CALL FLOP
C
      IF(IS.NE.0) GOTO 100
C
      CALL RFILE
      PRINT 10,NEL,NELA,NCON,NOGN
   10 FORMAT(/' MESH HAS',I7,' ELEMENTS (',I7,' ACTIVE) AND',I7,
     A' CONNECTIONS (INTERFACES)',
     A' BETWEEN THEM'/' GENER HAS',I6,' SINKS/SOURCES')
      IF(IRANT2.NE.0) PRINT 112
  112 FORMAT(' BLOCK-BY-BLOCK PERMEABILITY MODIFICATION IS IN EFFECT')
C
C
C.....SET SOLVER TYPE AND MAKE INFORMATIVE PRINTOUT
      CALL SINSUB
C
      IF(MOP(7).NE.0) CALL INDATA
C
      CALL SEC(ELT1)
      ELT1=ELT1-TZERO
      PRINT 11,ELT1
   11 FORMAT(//' END OF TOUGH2 INPUT JOB --- ELAPSED TIME = ',F8.4,' SEC
     AONDS'/)
C
      PRINT 79,H1
   79 FORMAT(A1/' ',131('*'))
      PRINT 80
   80 FORMAT(' *          ARRAY *MOP*',
     X' ALLOWS TO GENERATE MORE PRINTOUT IN VARIOUS SUBROUTINES, AND',
     X' TO MAKE SOME CALCULATIONAL CHOICES.           *')
      PRINT 78
   78 FORMAT(' ',131('*')/)
      PRINT 81,MOP(1)
   81 FORMAT(' ',
     X'  MOP(1)  =',I2,' *** ALLOWS TO GENERATE A SHORT PRINTOUT FOR'
     X,' EACH NEWTON-RAPHSON ITERATION'/
     X'           = 0, 1, OR 2: GENERATE 0, 1, OR 2 LINES OF PRINTOUT'//
     X12X,'MORE PRINTOUT IS GENERATED FOR MOP(I) > 0 IN THE FOLLOWING',
     X' SUBROUTINES (THE LARGER MOP IS, THE MORE WILL BE PRINTED).'/)
      PRINT 82,MOP(2),MOP(3),MOP(4),MOP(5),MOP(6),MOP(8)
   82 FORMAT(' ',
     X'  MOP(2)  =',I2,' *** CYCIT    ',
     X'   MOP(3) =',I2,' *** MULTI    ',
     X'   MOP(4) =',I2,' *** QU    ',
     X'   MOP(5) =',I2,' *** EOS    ',
     X'   MOP(6) =',I2,' *** LINEQ    '/
     X'   MOP(8)  =',I2,' *** DISF (T2DM ONLY)'/)
      PRINT 83,MOP(7)
   83 FORMAT(' ',
     X'  MOP(7)  =',I2,' *** IF UNEQUAL ZERO, WILL GENERATE A PRINTOUT',
     X' OF INPUT DATA'/)
C
      PRINT 84,MOP(9),MOP(10)
   84 FORMAT(' ',
     X8X,'   CALCULATIONAL CHOICES OFFERED BY MOP ARE AS FOLLOWS:'//
     X'   MOP(9)  =',I2,' *** CHOOSES FLUID COMPOSITION ON WITHDRAWAL',
     X' (PRODUCTION).'/
     X'           = 0: ACCORDING TO RELATIVE MOBILITIES.'/
     X'           = 1: ACCORDING TO COMPOSITION IN PRODUCING ELEMENT.'//
     X'   MOP(10) =',I2,' *** CHOOSES INTERPOLATION FORMULA FOR DEPEN',
     X'DENCE OF THERMAL CONDUCTIVITY ON LIQUID SATURATION (SL).'/
     X'           = 0: K = KDRY + SQRT(SL)*(KWET-KDRY)'/
     X'           = 1: K = KDRY + SL*(KWET-KDRY)'/)
      PRINT 85,MOP(11)
   85 FORMAT(' ',
     X'  MOP(11) =',I2,' *** CHOOSES EVALUATION OF MOBILITY',
     X' AND ABSOLUTE PERMEABILITY AT INTERFACES.'/
     X'           = 0: MOBILITIES ARE UPSTREAM WEIGHTED WITH WUP.',
     X' (DEFAULT IS WUP = 1.0). PERMEABILITY IS UPSTREAM WEIGHTED.'/
     X'           = 1: MOBILITIES ARE AVERAGED BETWEEN',
     X' ADJACENT ELEMENTS. PERMEABILITY IS UPSTREAM WEIGHTED.'/
     X'           = 2: MOBILITIES ARE UPSTREAM WEIGHTED WITH WUP.',
     X' (DEFAULT IS WUP = 1.0). PERMEABILITY IS HARMONIC WEIGHTED.'/
     X'           = 3: MOBILITIES ARE AVERAGED BETWEEN',
     X' ADJACENT ELEMENTS. PERMEABILITY IS HARMONIC WEIGHTED.'/
     X'           = 4: MOBILITY * PERMEABILITY PRODUCT IS HARMONIC',
     X' WEIGHTED.'/)
      PRINT 86,MOP(12),MOP(13),MOP(14),MOP(15)
   86 FORMAT(
     X'   MOP(12) =',I2,' *** CHOOSES PROCEDURE FOR INTERPOLATING ',
     X'GENERATION RATES FROM A TIME TABLE.'/
     X'           = 0: TRIPLE LINEAR INTERPOLATION.'/
     X'           = 1: "STEP FUNCTION" OPTION.'/
     X'           = 2: RIGOROUS "STEP RATE CAPABILITY" OPTION.'//
     &'   MOP(13) =',I2,' *** DEFINES CONTENT OF INCON AND SAVE FILE.',/
     &,'           = 0: STANDARD CONTENT.'/,
     &'           = 2: READS PARAMETERS OF HYSTERESIS MODEL FROM ',
     &'FILE INCON.',/,/,
     X'   MOP(14) =',I2,' *** T2DM ONLY.  SPECIFIES ASSIGNMENT OF',
     X' COMPONENTS OF BOUNDARY VELOCITY AND',
     X' CONCENTRATION GRADIENT VECTORS.'/
     X'                   AFFECTS ONLY THE INTERPOLATED ',
     X' COMPONENTS.  DIRECT COMPONENTS ARE USED WHERE AVAILABLE.'/
     X'           = 0: VELOCITY AND GRADIENT AT BOUNDARY ARE ZERO.'/
     X'           = 1: VELOCITY IS ZERO; GRADIENT AT BOUNDARY IS',
     X' NEAREST-NEIGHBOR.'/
     X'           = 2: VELOCITY IS NEAREST NEIGHBOR; GRADIENT AT',
     X' BOUNDARY IS ZERO.'/
     X'           = 3: VELOCITY AND GRADIENT AT BOUNDARY',
     X' ARE NEAREST-NEIGHBOR.'//
     X'   MOP(15) =',I2,' *** ALLOWS TO SELECT A SEMI-ANALYTICAL HEAT',
     X' EXCHANGE CALCULATION WITH CONFINING BEDS.'/
     X'           = 0: NO SEMI-ANALYTICAL HEAT EXCHANGE'/
     X'           > 0: SEMI-ANALYTICAL HEAT EXCHANGE ENGAGED (WHEN A',
     X' SPECIAL SUBROUTINE MODULE *QLOSS* IS PRESENT)'/)
      PRINT 87,MOP(16),MOP(17),MOP(18)
   87 FORMAT(' ',
     X'  MOP(16) =',I2,' *** PERMITS TO CHOOSE TIME STEP SELECTION ',
     X'OPTION'/
     X'           = 0: USE TIME STEPS EXPLICITLY PROVIDED AS INPUT.'/
     X'           > 0: INCREASE TIME STEP BY AT LEAST A FACTOR 2, IF',
     X' CONVERGENCE OCCURS IN .LE. MOP(16) ITERATIONS.'//
     X'   MOP(17) =',I2,' *** HANDLES TIME STEPPING AFTER',
     X                  ' LINEAR EQUATION SOLVER FAILURE.'/
     X'           = 0: NO TIME STEP REDUCTION DESPITE',
     X                  ' LINEAR EQUATION SOLUTION FAILURE.'/
     X'           = 9: REDUCE TIME STEP AFTER',
     X                  ' LINEAR EQUATION SOLUTION FAILURE.'//
     X'   MOP(18) =',I2,' *** ALLOWS TO SELECT HANDLING OF INTERFACE ',
     X'DENSITY.'/
     X'           = 0: PERFORM UPSTREAM WEIGHTING FOR INTERFACE DENSITY.
     X'/'           > 0: COMPUTE INTERFACE DENSITY AS AVERAGE OF',
     X' THE TWO GRID BLOCK DENSITIES.'/
     X'                HOWEVER, WHEN ONE OF THE TWO PHASE SATURATIONS',
     X' IS ZERO, DO UPSTREAM WEIGHTING.'/)
C
      PRINT 88,MOP(19),MOP(20),MOP(21)
   88 FORMAT('   MOP(19) =',I2,' *** IS USED IN SOME EOS-MODULES FOR',
     X' SELECTING DIFFERENT OPTIONS'//
     X'   MOP(20) =',I2,' *** IS USED IN SOME EOS-MODULES FOR',
     X' SELECTING DIFFERENT OPTIONS'//
     X'   MOP(21) =',I2,' *** PERMITS TO SELECT LINEAR',
     X' EQUATION SOLVER'/
     X'           = 0: DEFAULTS TO MOP(21) = 3'/
     X'           = 1: DIRECT SOLVER - SEE MOP(21)=6'/
     X'           = 2: SUBROUTINE DSLUBC: BI-CONJUGATE GRADIENT',
     X' SOLVER; PRECONDITIONER: INCOMPLETE LU FACTORIZATION'/
     X'           = 3: SUBROUTINE DSLUCS: BI-CONJUGATE GRADIENT',
     X' SOLVER - LANCZOS TYPE; PRECONDITIONER: INCOMPLETE LU',
     X' FACTORIZATION'/
     X'           = 4: SUBROUTINE DSLUGM: GENERALIZED MINIMUM',
     X' RESIDUAL CONJUGATE GRADIENTS; PRECONDITIONER: INCOMPLETE LU',
     X' FACTORIZATION'/
     X'           = 5: SUBROUTINE DLUSTB: STABILIZED BI-CONJUGATE',
     X' GRADIENT SOLVER; PRECONDITIONER: INCOMPLETE LU',
     X' FACTORIZATION'/
     X'           = 6: SUBROUTINE LUBAND: DIRECT SOLVER USING',
     X' LU DECOMPOSITION'/)
C
      PRINT 89,MOP(22),MOP(23)
  89  FORMAT('   MOP(22) =',I2,' *** T2DM ONLY.  SPECIFIES METHOD OF',
     X' SUMMATION OF DUPLICATE ELEMENTS IN THE JACOBIAN. ***',/,
     X'           = 0: USE SUMDUP2.',/,
     X'           = 1: USE SUMDUP.',/,
     X'           = 2: IF MATSLV = 1, USE MA28 INTERNAL SUMMATION.'//,
     X'   MOP(23) =',I2,' *** T2DM ONLY.  HANDLES EFFECTS OF',
     X' NON-CONNECTED NEIGHBOR GRID BLOCKS ON DISPERSIVE FLUX IN ',
     X' DISF. ***',/,
     X'           = 0: INCLUDE INFLUENCE OF NEIGHBOR GRID BLOCKS.',
     X'  (MORE ACCURATE JACOBIAN, SLOWER LINEAR EQUATION SOLUTION).'/,
     X'           = 1: NEGLECT INFLUENCE OF NEIGHBOR GRID BLOCKS.',
     X'  (LESS ACCURATE JACOBIAN, FASTER LINEAR EQUATION SOLUTION).'/)
C
      PRINT 90,MOP(24)
   90 FORMAT('   MOP(24) =',I2,' *** PERMITS TO SELECT HANDLING OF',
     X' MULTIPHASE DIFFUSIVE FLUXES AT INTERFACES'/
     X'           = 0: HARMONIC WEIGHTING OF FULLY-COUPLED EFFECTIVE ',
     X'MULTIPHASE DIFFUSIVITY'/
     X'           = 1: SEPARATE HARMONIC WEIGHTING OF GAS AND LIQUID ',
     X'PHASE DIFFUSIVITIES.'//
     X' ',131('*'))
C
C-----SOLVE MASS- AND ENERGY-TRANSPORT EQUATIONS.
      IF(IS.EQ.0) CALL CYCIT
C     IF IS.NE.0 (KEYWORD "ENDFI" IN INPUT FILE, OR PROBLEM IN RFILE),
C     NO FLOW SIMULATION WILL BE PERFORMED.
C
  100 CONTINUE
c     IF(IV.EQ.0) GOTO 15
      PRINT 20,H1
   20 FORMAT(A1/
     X' ',131('*')/' *',129X,'*'/' *',49X,'SUMMARY OF PROGRAM UNITS',
     X' USED',51X,'*'/' *',129X,'*'/' ',131('*')//
     X'       UNIT    VERSION     DATE                 COMMENTS'/
     X1X,131('_')/)
C
      ENDFILE 11
      REWIND 11
C
   16 CONTINUE
      DO17 I=1,26
   17 VV(I)='     '
C
      READ(11,14,END=15) (VV(I),I=1,26)
   14 FORMAT(26A5)
      PRINT 14,(VV(I),I=1,26)
      GOTO 16
C
   15 CONTINUE
      PRINT 19
   19 FORMAT(//1X,131('*'))
      CALL SEC(ELT)
      ELT=ELT-TZERO
      ELTC=ELT-ELT1
      PRINT 12,ELT,ELTC,ELT1,H1
   12 FORMAT(//' END OF TOUGH2 SIMULATION --- ELAPSED TIME =',F9.3,
     A' SEC --- CALCULATION TIME =',F9.3,' SEC --- DATA INPUT TIME ='
     B,F9.3,' SEC'/A1)
      STOP
      END
C
      SUBROUTINE INPUT
C          .....................................................
C          .                                                   .
C          .   TMVOC, MODULE T2FM, VERSION 1.0, APRIL 2002     .
C          .....................................................
C
C-----READ ALL DATA PROVIDED THROUGH THE INPUT FILE.
C
C
C$$$$$$$$$ COMMON BLOCKS FOR CONNECTIONS $$$$$$$$$$$$$$$$$$$$$$$$$$$C
C
      COMMON/C1/NEX1(1)
      COMMON/C2/NEX2(1)
      COMMON/C3/DEL1(1)
      COMMON/C4/DEL2(1)
      COMMON/C5/AREA(1)
      COMMON/C6/BETA(1)
      COMMON/C7/ISOX(1)
      COMMON/C8/GLO(1)
      COMMON/C9/ELEM1(1)
      COMMON/C10/ELEM2(1)
C
C$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
C
      COMMON/NN/NEL,NCON,NOGN,NK,NEQ,NPH,NB,NK1,NEQ1,NBK,NSEC,NFLUX
      COMMON/NNN/NKIN,NKIN1
      COMMON/CONTST/RE1,RE2,RERM,NER,KER,DFAC
C
C$$$$$ COMMON BLOCKS FOR SINKS/SOURCES $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
C
      COMMON/G1/F1(1)
      COMMON/G2/F2(1)
      COMMON/G3/F3(1)
      COMMON/G9/NEXG(1)
C
C$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
C
      COMMON/SVZ/NOITE,MOP(24)
      COMMON/SOLID/NM,DM(27),POR(27),PER(3,27),CWET(27),SH(27)
      COMMON/SOCH/MAT(27)
      COMMON/SOLI/COM(27),EXPAN(27),CDRY(27),TORT(27),GK(27)
      COMMON/SOLII/XKD3(27),XKD4(27),SII3(27)
      COMMON/SOIN/YIN(21,27)
      COMMON/AMMIS/MA,IPIV,U,IAB,NZ
      COMMON/STEP/ELST
      COMMON/STE1/NST
      COMMON/DEFINI/DEP(21)
      COMMON/DG/WUP,WNR
      COMMON/DFM/TIMAX,REDLT
      COMMON/DLT/NDLT,DLT(100)
      COMMON/DM/DELTEN,DELTEX,FOR,FORD
      COMMON/DMN/INUM,IPRINT,MCYC,MCYPR,MSEC,TZERO,TIMP1
      COMMON/DOP/ENTH,KDATA,QUAL
      COMMON/DX/K,NI,SCALE
      COMMON/POV6/TSTART
      COMMON/CYC/KCYC,ITER,ITERC,TIMIN,SUMTIM,GF,TIMOUT
      COMMON/PATCH/SING
      COMMON/TIM/ITI,DELAF,ITPR,TIS(100),ITCO,NOWTIM,DELTMX
C-----NOW FOR A COMMON BLOCK WITH EOS-SELECTION PARAMETERS.
      COMMON/EOSEL/IE(16),FE(512)
      COMMON/RPCAP/IRP(27),RP(14,27),ICP(27),CP(14,27),IRPD,RPD(14),
     &       ICPD,CPD(14)
      COMMON/BIND/DIFF0,TEXP,BE
      COMMON/V/IV,IS
C.....FOR T2VOC
      COMMON/ADSORP/OCK2,FOX2,FOC(27),ALAM2
      COMMON/CRITP/TCRIT2,PCRIT2,ZCRIT2,OMEGA2,DIPOLM2
      COMMON/VPOIL/TBOIL2,VPA2,VPB2,VPC2,VPD2
      COMMON/HCAPL/AMO,CPA2,CPB2,CPC2,CPDD2
      COMMON/DENOIL/RHOREF2,TDENRF,DIFV02,TDIFRF,TEXPO2
      COMMON/VISOIL/VLOA2,VLOB2,VLOC2,VLOD2,VOLCRT2
      COMMON/HCOIL/SOLA2,SOLB2,SOLC2,SOLD2
C.....FOR TMVOC
      COMMON/COMPOUNDS/ HCNAMES(20),IT2VOC
      COMMON/NUMBER/ NHC
      COMMON/DEFINI2/INDICE
      COMMON/INTYPE/INRE
      COMMON/INDINIT/INDXIN(27)
      COMMON /NCGN/NCG,NCGCODE(10)
      COMMON /NCGC/NCGINP(10)
      common/ncgl/alncg(10)
C---*----1----*----2----*----3----*----4----*----5----*----6----*----7----*----8
C**  COMMON BLOCKS FOR THE THERMODYNAMIC PROPERTIES OF THE HYDROCARBONS.
      COMMON/CRITPTM/TCRITM(20),PCRITM(20),VOLCRITM(20),ZCRITM(20),
     *    OMEGAM(20), DIPOLMM(20)
      COMMON/VPOILTM/TBOILM(20),VPAM(20),VPBM(20),VPCM(20),VPDM(20)
      COMMON/HCAP0/CPAM(20), CPBM(20), CPCM(20), CPDDM(20)
      COMMON/DENOILTM/RHOREFM(20),TDENREF(20),DIFV0M(20),TDIFREF(20),
     *    TEXPOM(20)
      COMMON/VISOILTM/VLOAM(20), VLOBM(20), VLOCM(20), VLODM(20)
      COMMON/EQUIL1/SOLAM(20), SOLBM(20), SOLCM(20), SOLDM(20)
      COMMON/MOLWT/AMWTM(20)
      COMMON/ADSORPTM/OCKM(20),FOXM,FOCM(27),ALAMM(20)
C.... END TMVOC
      COMMON/SOLVR1/MATSLV,NMAXIT,NNVVCC,IIUUNN,IISSOO,NACTDI
      COMMON/SOLVR2/RITMAX,CLOSUR
      COMMON/SOLVR3/ORDRNG,OPROCS,ZPROCS,COORD
C
      COMMON/FGT1/IOFT,IOFU,IGOFT,IGOFU,NOFT(100),NGOFT(100)
      COMMON/FGT2/EOFT(100),EGOFT(100),ECOFT(100)
      COMMON/FGT3/ICOFT,ICOFU,NCOFT(100)
      COMMON/FF/H1
      COMMON/DIPA/FDDIAG(3,20)
      COMMON/DIPA1/IDDIAG
C
C$$$$$$$$$ COMMON BLOCKS FOR HYSTERESIS MODEL $$$$$$$$$$$$$$$$$$$$$$$$$$
C
      COMMON/KHYST1/IHYSTER,KCYCX,IEHYS(16)
      COMMON/KHYST2/ICURV(1)
      COMMON/KHYST3/KTMPO(1)
      COMMON/KHYST4/irun(1)
      COMMON/KHYST5/IFLG1(1)
      COMMON/KHYST6/KFLAG(1)
      COMMON/SAVHYS1/SOLDH(1)
      COMMON/SAVHYS2/HOLD(1)
      COMMON/TMPHYS1/STMPO(1)
      COMMON/TMPHYS2/HTMPO(1)
      COMMON/HYS11/STRN1(1)
      COMMON/HYS12/STRN2(1)
      COMMON/HYS13/STRN3(1)
      COMMON/HYS21/HTRN1(1)
      COMMON/HYS22/HTRN2(1)
      COMMON/HYS23/HTRN3(1)
      COMMON/HYS31/SCMP1(1)
      COMMON/HYS32/SCMP2(1)
      COMMON/HYS33/BB2(1)
      COMMON/TORBENR1/SEOT(1)
      COMMON/TORBENR2/SETRN1(1)
      COMMON/TORBENR3/SOR(1)
      COMMON/TORBENR4/SOROLD(1)
      COMMON/TORBENR5/SORTMPO(1)
      common/hyssav1/strn1old(1)
      common/hyssav2/strn1tmpo(1)
      common/hyssav3/strn2old(1)
      common/hyssav4/strn2tmpo(1)
      common/hyssav5/strn3old(1)
      common/hyssav6/strn3tmpo(1)
      common/hyssav7/htrn1old(1)
      common/hyssav8/htrn1tmpo(1)
      common/hyssav9/htrn2old(1)
      common/hyssav10/htrn2tmpo(1)
      common/hyssav11/htrn3old(1)
      common/hyssav12/htrn3tmpo(1)
      common/hyssav13/scmp1old(1)
      common/hyssav14/scmp1tmpo(1)
      common/hyssav15/scmp2old(1)
      common/hyssav16/scmp2tmpo(1)
      common/hyssav17/bb2old(1)
      common/hyssav18/bb2tmpo(1)
      common/hyssa21/ hmin(1)
      common/hyssa22/ hminold(1)
      common/hyssa23/ hmintmpo(1)
      common/hyssa31/ shmin(1)
      common/hyssa32/ shminold(1)
      common/hyssa33/ shmintmpo(1)    
      common/hyssa41/ hminp(1)
      common/hyssa42/ hminpold(1)
      common/hyssa43/ hminptmpo(1)
C
      COMMON/ECH/EOSN(20)
      COMMON/MARE/AMRES
C
      LOGICAL LHYSTE
      CHARACTER*1 H1
      CHARACTER*5 EOFT,EGOFT
      CHARACTER*10 ECOFT,NCGINP
C
      CHARACTER*5 ELEM1,ELEM2,MAT,ELST,VER,WORD,MA12,NAM
      CHARACTER*3 EL,MA1,EL1,EL2,SL
      CHARACTER MA2*2,TYPE*5,ITAB*1
      CHARACTER*2 ORDRNG,OPROCS,ZPROCS
      CHARACTER*5 COORD
      CHARACTER W75*75,COMM*80
      CHARACTER*20 HCNAMES
      CHARACTER EOSN*10
C
      DIMENSION VER(26),XIN(21,27),NAM(27),COMM(50),XX(21),INDICEIN(27)
C
      SAVE ICALL,ICOM
      DATA ICALL,ICOM/0,0/
      DATA LHYSTE/.FALSE./
      DATA VER /'PARAM','ELEME','CONNE','GENER','INCON','START','RESTA',
     X'ENDCY','ENDFI','ROCKS','MULTI','TIMES','SELEC','RPCAP','MESHM',
     X'INDOM','NOVER','FOFT ','GOFT ','DIFFU','COFT ','CHEMP','SOLVR',
     X'NCGAS','INDEX','HYSTE'/
C
      ICALL=ICALL+1
      IF(ICALL.EQ.1) WRITE(11,899)
C 899 FORMAT(/6X,'INPUT    1.0      11 APRIL     1991',6X,
C 899 FORMAT(/6X,'INPUT    1.01     12 JANUARY   1995',6X,
C 899 FORMAT(/6X,'INPUT    1.01      3 JULY      1995',6X,
C 899 FORMAT(/6X,'INPUT    1.01     23 JULY      1997',6X,
C 899 FORMAT(/6X,'INPUT    1.01     11 SEPTEMBER 1997',6X,
C 899 FORMAT(/6X,'INPUT    1.01     22 OCTOBER   1997',6X,
C 899 FORMAT(/6X,'INPUT    1.01     11 MARCH     1998',6X,
C 899 FORMAT(/6X,'INPUT    1.01     13 MARCH     1998',6X,
C 899 FORMAT(/6X,'INPUT    1.01     16 MARCH     1998',6X,
C 899 FORMAT(/6X,'INPUT    1.01     19 MARCH     1998',6X,
C 899 FORMAT(/6X,'INPUT    1.01     28 MAY       1998',6X,
C 899 FORMAT(/6X,'INPUT    1.01     29 APRIL     1999',6X,
C 899 FORMAT(/6X,'INPUT    1.50     20 MAY       1999',6X,
C 899 FORMAT(/6X,'INPUT    1.50     21 JUNE      2001',6X,
C 899 FORMAT(/6X,'INPUT    2.0      24 AUGUST    2001',6X,
C 899 FORMAT(/6X,'INPUT    2.0       4 JANUARY   2002',6X,
c 899 FORMAT(/6X,'INPUT    2.0       2 APRIL     2002',6X,
C 899 FORMAT(/6X,'INPUT    2.1      17 AUGUST    2010',6X,
  899 FORMAT(/6X,'INPUT    2.1       1 JULY      2011',6X,
     X'READ ALL DATA PROVIDED THROUGH FILE *INPUT*')
C
C.....INITIALIZE PARAMETERS FOR TIME SLICES
      IOFT=0
      IGOFT=0
      ICOFT=0
C
      NI=0
C     NI = 1 FOR START; NI = 0 OTHERWISE.
C
      DELTMX=0.
      ITI=0
C-----ITI IS THE FLAG FOR THE *TIMES* DATA BLOCK. IF THIS BLOCK
C     IS PRESENT, ITI WILL BE SET EQUAL TO THE NUMBER OF TIME DATA
C     PROVIDED.
      KIN=0
C-----KIN IS THE FLAG FOR THE *INDOM* DATA BLOCK FOR DOMAIN-SPECIFIC
C     INITIALIZATION. IF THIS BLOCK IS PRESENT, KIN WILL BE SET EQUAL
C     TO THE NUMBER OF DOMAINS IN IT.
      IS=0
C     WHEN KEYWORD "ENDFI" IS ENCOUNTERED IN THE INPUT FILE,
C     RESET IS=IS+1; THIS WILL IMMEDIATELY RETURN CONTROL TO
C     MAIN PROGRAM TOUGH2, AND FLOW SIMULATION WILL BE BYPASSED.
C
C     ASSIGN DEFAULTS FOR RELATIVE PERMEABILITY AND CAPILLARY
C     PRESSURE.
C     DEFAULTS ARE KREL = 1, PCAP = 0, APPROPRIATE FOR SINGLE PHASE FLOW
      IRPD=5
      IF(EOSN(1).EQ.'TMVOC1    ') THEN
C+++++ SECTION FOR TMVOC
        ICPD=9
        IT2VOC=1
C.....DEFAULT INRE=1 TO READ INDICES
        INRE=1
C
C.....DEFAULT ASSIGNMENTS FOR NCGS
         NCG=1
         NCGINP(1)='AIR       '
         INCG=0
      ELSE
        INRE=0
        ICPD=1
      END IF
      CPD(1)=0.
      CPD(2)=0.
      CPD(3)=1.
C
      IISSOO = 0
C
C.....INITIALIZE ARRAYS FOR PRIMARY VARIABLES
      DO 1910 I=1,NK1 
         DEP(I)=0.
 1910    XX(I)=0.
C
C.....INITIALIZE COMPONENT AND EQUATION COUNTERS 
      NEQ1=NEQ+1
      NK1=NK+1
      NFLUX=2*NEQ+1
      NBK=NB+NK
      NSEC=NPH*NBK+2
C.....SET COMPONENT COUNTER FOR INTIALIZING THERMODYNAMIC
C     CONDITIONS; DEFAULT VALUES CORRESPOND TO WHAT IS
C     DESIRED FOR THE FLOW SIMULATION
      NKIN=NK
      NKIN1=NK1
C.....FLAG FOR 'DIFFU'-DATA (DIFFUSION COEFFICIENTS)
      IDDIAG=0
C
 5019 READ(5, 5020)WORD,W75
 5020 FORMAT(A5,A75)
C
      DO900 K=1,26
  900 IF(WORD.EQ.VER(K)) GOTO920
C.....COME HERE FOR UNKNOWN KEYWORD
      ICOM=ICOM+1
      IF(ICOM.LE.50) COMM(ICOM)=WORD//W75
C     PRINT 901,WORD
C 901 FORMAT(' HAVE READ UNKNOWN BLOCK LABEL "',A5,
C    X'" --- IGNORE THIS, AND CONTINUE READING INPUT DATA')
      GOTO 5019
C
  920 GOTO(1000,1100,1200,1300,1900,1450,5019,1500,1600,1700,1800,
     X2000,2100,2200,2300,2400,2500,80,180,190,280,2600,3000,4000,
     X4100,4400),K
C
C*****READ ROCK PROPERTIES.*********************************************
C                                                                      *
C     KEYWORD 'ROCKS'
 1700 NM=1
    2 READ(5, 1)MAT(NM),NAD,DM(NM),POR(NM),(PER(I,NM),I=1,3),
     ACWET(NM),SH(NM)
    1 FORMAT(A5,I5,7E10.4)
C
      IF(MAT(NM).EQ.'     ') GOTO 3
C
CAB      INDICEIN(1,NM)=0.
      YIN(1,NM)=0.
      COM(NM)=0.

      EXPAN(NM)=0.
      CDRY(NM)=0.
      TORT(NM)=0.
      GK(NM)=0.
      FOC(NM) = 0.E0
      FOCM(NM) = 0.E0
      IRP(NM)=0
      ICP(NM)=0
      IF(NAD.GE.1)READ(5, 7)COM(NM),EXPAN(NM),CDRY(NM),TORT(NM),GK(NM)
     X,XKD3(NM),XKD4(NM)
C.....5-25-00: ASSIGN FRACTION OF ORGANIC CARBON FOR T2VOC & TMVOC
      FOC(NM)=XKD3(NM)
      FOCM(NM)=XKD3(NM)
    7 FORMAT(8E10.4)
      IF(CDRY(NM).EQ.0.) CDRY(NM)=CWET(NM)
      IF(NAD.LE.1) GOTO 1701
C-----READ PARAMETERS FOR RELATIVE PERMEABILITY AND CAPILLARY PRESSURE.
      READ(5, 1702)IRP(NM),(RP(I,NM),I=1,7)
      IF (IRP(NM).EQ.12.AND.EOSN(1).NE.'ECO2M     ')
     &    READ(5,'(8E10.4)') (RP(I,NM),I=8,14)
      READ(5, 1702)ICP(NM),(CP(I,NM),I=1,7)
      IF (ICP(NM).EQ.12.AND.EOSN(1).NE.'ECO2M     ')
     &    READ(5,'(8E10.4)') (CP(I,NM),I=8,14)
 1702 FORMAT(I5,5X,7E10.4)
      IF ((IRP(NM).EQ.12.OR.ICP(NM).EQ.12).AND.EOSN(1).NE.'ECO2M     ') 
     &   IHYSTER=-1
 1701 CONTINUE
C
      NAD=0
C
      PRINT 5,NM,MAT(NM)
    5 FORMAT(/' DOMAIN NO.',I3,'     MATERIAL NAME -- ',A5)
      NM=NM+1
      GOTO2
    3 CONTINUE
      NM=NM-1
      GOTO5019
C
C-----END OF ROCK PROPERTIES.-------------------------------------------
C
C     KEYWORD 'RPCAP'
 2200 CONTINUE
C-----COME HERE TO READ DEFAULT ASSIGNMENTS FOR RELATIVE PERMEABILITY
C     AND CAPILLARY PRESSURE PARAMETERS.
      READ(5, 1702)IRPD,(RPD(I),I=1,7)
      IF (IRPD.EQ.12.AND.EOSN(1).NE.'ECO2M     ')
     &    READ(5,'(8E10.4)') (RPD(I),I=8,14)
      READ(5, 1702)ICPD,(CPD(I),I=1,7)
      IF (ICPD.EQ.12.AND.EOSN(1).NE.'ECO2M     ')
     &    READ(5,'(8E10.4)') (CPD(I),I=8,14)
      IF ((IRPD.EQ.12.OR.ICPD.EQ.12).AND.EOSN(1).NE.'ECO2M     ') 
     &   IHYSTER=-1

      GOTO 5019
C
C     KEYWORD 'MESHM'
 2300 CONTINUE
C-----COME HERE TO CALL INTERNAL MESH GENERATION MODULE.
      CALL MESHM
      GOTO 5019
C
C     KEYWORD 'START'
 1450 NI=1
      GOTO5019
C*****READ COMPUTATION PARAMETERS.**************************************
C
C     KEYWORD 'PARAM'
 1000 READ(5, 200)NOITE,KDATA,MCYC,MSEC,MCYPR,(MOP(I),I=1,24),
     XDIFF0,TEXP,BE
  200 FORMAT(2I2,3I4,24I1,4E10.4)
C
      if (mcyc.eq.9999) mcyc=1000000
      READ(5, 8)TSTART,TIMAX,DELTEN,DELTMX,ELST,GF,REDLT,SCALE
    8 FORMAT(4E10.4,A5,5X,3E10.4)
      NDLT=0
      IF(DELTEN.GE.0.) GOTO4
      NDLT=-INT(DELTEN)
      DO6 N=1,NDLT
    6 READ(5, 7)(DLT(J+8*(N-1)),J=1,8)
      DELTEN=DLT(1)
    4 CONTINUE
C
      READ(5, 7)RE1,RE2,U,WUP,WNR,DFAC,FOR,AMRES
C
C-----ASSIGN DEFAULT VALUES.
C
      IF(NOITE.EQ.0) NOITE=8
      IF(KDATA.EQ.0) KDATA=1
      IF(MCYPR.EQ.0) MCYPR=1
C.....3-11-98: NEXT STATEMENT COMMENTED OUT (RE/EOS7R)
C     IF(TEXP.EQ.0.) TEXP=1.80
      IF(REDLT.EQ.0.) REDLT=4.
      IF(SCALE.EQ.0.) SCALE=1.
      IF(RE1.EQ.0.) RE1=1.E-5
      IF(RE2.EQ.0.) RE2=1.
      IF(FOR.EQ.0.) FOR=1.
      IF(WUP.LE.0.) WUP=1.
      IF(WNR.EQ.0.) WNR=1.
C     IF(DFAC.EQ.0.) DFAC=1.E-8
      IF(U.EQ.0.) U=0.1
C
C-----INITIALIZE SOME PARAMETERS.
C
      KCYC=0
      ITER=0
      ITERC=0
      TIMIN=TSTART
C
      IF(INRE.EQ.1) THEN
C+++++ READ PHASE INDEX
       READ(5, 6400) INDICE
 6400  FORMAT (I5)
      END IF
      READ(5, 1908)(DEP(I),I=1,NKIN1)
C
C-----END OF COMPUTATION PARAMETERS.------------------------------------
C
      GOTO5019
C
C*****READ ELEMENT DATA.************************************************
C
C     KEYWORD 'ELEME'
 1100 CONTINUE
      PRINT 41
      REWIND 4
      PRINT 1103
 1103 FORMAT(' WRITE FILE *MESH* FROM INPUT DATA')
      WRITE(4,1101)
 1101 FORMAT(5HELEME)
 1102 READ(5, 10)EL,NE,NSEQ,NADD,MA1,MA2,VOLX,AHTX,PMX,X,Y,Z
   10 FORMAT(A3,I2,2I5,A3,A2,2E10.4,4E10.4)
      IF(EL.EQ.'   '.AND.NE.EQ.0) GOTO40
      NSEQ1=NSEQ+1
      IF(MA1.EQ.'   ') GOTO15
C
C-----FIND MATERIAL INDEX.
      MA12=MA1//MA2
      DO11 M=1,NM
      IF(MA12.EQ.MAT(M)) GOTO 12
   11 CONTINUE
      PRINT 9,MA1,MA2,EL,NE
    9 FORMAT(' REFERENCE TO UNKNOWN MATERIAL ',A3,A2,' AT ELEMENT ',A2
     &,I3,' --- WILL IGNORE ELEMENT')
      GOTO1102
C
   12 CONTINUE
C
C-----COME HERE FOR ELEMENTS WITH MATERIAL NAME.
C
      DO13 I=1,NSEQ1
      N=NE+(I-1)*NADD
   13 WRITE(4,14) EL,N,M,VOLX,AHTX,PMX,X,Y,Z
   14 FORMAT(A3,I2,10X,I5,2E10.4,4E10.4)
      GOTO1102
C
C-----COME HERE FOR ELEMENTS WITH MATERIAL NUMBER.
C
   15 CONTINUE
      DO113 I=1,NSEQ1
      N=NE+(I-1)*NADD
  113 WRITE(4,114) EL,N,MA2,VOLX,AHTX,PMX,X,Y,Z
  114 FORMAT(A3,I2,13X,A2,2E10.4,4E10.4)
      GOTO1102
C
C-----END OF ELEMENT DATA.----------------------------------------------
C
   40 WRITE(4,41)
   41 FORMAT(5H     )
      GOTO5019
C
C*****READ CONNECTION DATA.*********************************************
C
C     KEYWORD 'CONNE'
 1200 WRITE(4,1201)
 1201 FORMAT('CONNE')
      NCON=0
 1202 READ(5, 20)EL1,NE1,EL2,NE2,NSEQ,NAD1,NAD2,ISOT,
     &     D1,D2,AREAX,BETAX,SIGX
   20 FORMAT(A3,I2,A3,I2,4I5,5E10.4)
      IF(EL1.EQ.'   '.AND.NE1.EQ.0) GOTO50
      IF(EL1.EQ.'+++') GOTO51
C
      NSEQ1=NSEQ+1
C
      DO23 I=1,NSEQ1
      NCON=NCON+1
      N1=NE1+(I-1)*NAD1
      N2=NE2+(I-1)*NAD2
   23 WRITE(4,24) EL1,N1,EL2,N2,ISOT,D1,D2,AREAX,BETAX,SIGX
   24 FORMAT(A3,I2,A3,I2,15X,I5,5E10.4)
      GOTO1202
C
C-----END OF CONNECTION DATA.-------------------------------------------
C
   50 WRITE(4,41)
      GOTO5019
   51 WRITE(4,42)
   42 FORMAT('+++  ')
      READ(5, 1505)(NEX1(N),NEX2(N),N=1,NCON)
      WRITE(4,1505) (NEX1(N),NEX2(N),N=1,NCON)
 1505 FORMAT(16I5)
      GOTO5019
C
C
C*****READ SINK/SOURCE DATA.*******************************************
C
C     KEYWORD 'GENER'
 1300 CONTINUE
      PRINT 41
      REWIND 3
      PRINT 1302
 1302 FORMAT(' WRITE FILE *GENER* FROM INPUT DATA')
      WRITE(3,1301)
 1301 FORMAT('GENER')
      NOGN=0
   35 READ(5, 30)EL,NE,SL,NS,NSEQ,NADD,
     &     NADS,LTAB,TYPE,ITAB,GX,EX,HX
   30 FORMAT(A3,I2,A3,I2,4I5,5X,A4,A1,3E10.4)
      IF(EL.EQ.'   '.AND.NE.EQ.0) GOTO60
      IF(EL.EQ.'+++') GOTO61
      NSEQ1=NSEQ+1
      IF(LTAB.EQ.0) LTAB=1
C
      DO 33 I=1,NSEQ1
        NOGN=NOGN+1
        N1=NE+(I-1)*NADD
        N2=NS+(I-1)*NADS
        WRITE(3,34) EL,N1,SL,N2,LTAB,TYPE,ITAB,GX,EX,HX
   34   FORMAT(A3,I2,A3,I2,15X,I5,5X,A4,A1,3G10.4)
        LTABA=IABS(LTAB)
        IF (LTABA.LE.1.OR.TYPE.EQ.'DELV') GO TO 33
        IF (I.GE.2) GO TO 133
C-----COME HERE TO READ TABLES OF TIMES AND RATES.
        READ(5, 36)(F1(L),L=1,LTABA)
        READ(5, 36)(F2(L),L=1,LTABA)
        IF (ITAB.NE.' ') READ(5, 36)(F3(L),L=1,LTABA)
   36   FORMAT(4E14.7)
  133   CONTINUE
        WRITE(3,36) (F1(L),L=1,LTABA)
        WRITE(3,36) (F2(L),L=1,LTABA)
        IF (ITAB.NE.' ') WRITE(3,36) (F3(L),L=1,LTABA)
   33   CONTINUE
      GO TO 35
C
C-----END OF SINK/SOURCE DATA.-----------------------------------------
C
   60 WRITE(3,41)
      GOTO5019
   61 WRITE(3,42)
      READ(5, 1505)(NEXG(N),N=1,NOGN)
      WRITE(3,1505) (NEXG(N),N=1,NOGN)
      GOTO5019
C
C*****READ MULTICOMPONENT PARAMETERS************************************
C
C
C     KEYWORD 'MULTI'
 1800 CONTINUE
      READ(5, 1801)NK,NEQ,NPH,NB,NKIN
      NEQ1=NEQ+1
      NK1=NK+1
      NFLUX=2*NEQ+1
      NBK=NB+NK
      NSEC=NPH*NBK+2
      IF(NKIN.EQ.0) NKIN=NK
      IF(NKIN.LT.0) THEN
C.....  T2VOC STYLE INITIAL CONDITIONS !
        NKIN=-NKIN
        IT2VOC=-1
C.....  DO NOT READ INDICES
        INRE=0
      ENDIF
      NKIN1=NKIN+1
 1801 FORMAT(16I5)
C
      GOTO 5019
C
C-----END OF MULTICOMPONENT PARAMETERS----------------------------------
C
C     KEYWORD 'INDEX'
 4100 CONTINUE
      INRE=1
      GOTO 5019

C***** READ MULTI-COMPONENT INITIAL CONDITIONS *************************
C
C     KEYWORD 'INCON'
 1900 CONTINUE
      PRINT 41
      REWIND 1
      PRINT 1909
 1909 FORMAT(' WRITE FILE *INCON* FROM INPUT DATA')
      WRITE(1,1901)
 1901 FORMAT('INCON')
C
 1906 CONTINUE
      IF(INRE.EQ.1) THEN
C+++++ READ INDICES
       READ(5, 8500) EL, NE, NSEQ, NADD, PORX, INDICEE
      ELSE
       READ(5, 1902)EL,NE,NSEQ,NADD,PORX
      END IF
 8500 FORMAT(A3,I2,2I5,E15.8,I2)
 1902 FORMAT(A3,I2,2I5,E15.8,5G10.4)
C
      IF(EL.EQ.'   '.AND.NE.EQ.0) GOTO 1903
      IF(EL.EQ.'+++') GOTO 1904
C
      READ(5, 1908)(XX(I),I=1,NKIN1)
C     READ 1908 X1,X2,X3,X4
 1908 FORMAT(4E20.13)
C
      NSEQ1=NSEQ+1
      DO 1905 I=1,NSEQ1
      N1=NE+(I-1)*NADD
      IF(INRE.EQ.1) THEN
C+++++  WRITE INDICES
        WRITE (1,8700) EL, N1, PORX, INDICEE
      ELSE
        WRITE(1,1907) EL,N1,PORX
      END IF
 8700 FORMAT (A3,I2,10X,E15.8,I2)
 1907 FORMAT(A3,I2,10X,E15.8,5E10.4)
C
      WRITE(1,1908) (XX(J),J=1,NKIN1)
C     WRITE(1,1908) X1,X2,X3,X4
C
CFiHysteresisB
C --- Write history for hysteresis model
         IF (IHYSTER.LT.0.AND.MOP(13).GT.0) THEN
            READ(5,7003) SLD,HLD,ICRV,SORE,STR1,STR2,STR3,
     &                HTR1,HTR2,HTR3,SCM1,SCM2,BBB2,     
     &                HMIN2,SHMIN2,HMINP2               
            WRITE(1,7003) SLD,HLD,ICRV,SORE,STR1,STR2,STR3,
     &            HTR1,HTR2,HTR3,SCM1,SCM2,BBB2,
     &            HMIN2,SHMIN2,HMINP2
 7003 FORMAT(2E20.14,I5,15x,e20.14,4(/,3E20.14))
         ENDIF
CFiHysteresisE

 1905 CONTINUE
C
      GOTO 1906
 1904 WRITE(1,42)
C
      READ(5, 74)KCYCX,ITERCX,NMX,TSTX,TIMINX
   74 FORMAT(3I5,2E15.8)
      WRITE(1,74) KCYCX,ITERCX,NMX,TSTX,TIMINX
C
      GOTO 5019
C
 1903 WRITE(1,1403)
 1403 FORMAT('     ')
      GOTO 5019
C
C-----END OF MULTICOMPONENT INITIAL CONDITIONS--------------------------
C
C***** READ DOMAIN-SPECIFIC INITIAL CONDITIONS ***********************
C     KEYWORD 'INDOM'
 2400 IF(INRE.EQ.1) THEN
C+++++  READ INDICES
        READ(5, 1) NAM(KIN+1),INDICEIN(KIN+1)
      ELSE
        READ(5, 1) NAM(KIN+1)
      ENDIF
C
      IF(NAM(KIN+1).EQ.'     ') GOTO 5019
      KIN=KIN+1
C
      READ(5, 1908)(XIN(I,KIN),I=1,NKIN1)
C     READ 1908)(XIN(I,KIN),I=1,4)
      GOTO 2400
C
C-----END OF DOMAIN-SPECIFIC INITIAL CONDITIONS------------------------
C
C     KEYWORD 'NOVER'
 2500 CONTINUE
C-----SET FLAG FOR SUPPRESSING VERSION-PRINTOUT AT END OF RUN.
      IV=0
      GOTO 5019
C
C
C***********************************************************************
C*                                                                     *
C*            READ SOLVER TYPE AND CORRESPONDING PARAMETERS            *
C*                                                                     *
C***********************************************************************
C     KEYWORD 'SOLVR'
 3000 IISSOO = 1
      READ(5, 3050) MATSLV,ZPROCS,OPROCS,RITMAX,CLOSUR
C
 3050 FORMAT(I1,2X,A2,3X,A2,2(E10.4))

      GOTO 5019
C
C-----END OF SOLVER DATA.-------------------------------------------
 4400 CONTINUE
C --- keyword 'HYSTE'
      LHYSTE=.TRUE.
      READ(5,1801) (IEHYS(I),I=1,16)
      GOTO 5019
C
C --- End of hysteresis control block
C
C     KEYWORD 'CHEMP'
 2600 CONTINUE
C-----ASSIGN VOC CHEMICAL PARAMETERS ('CHEMP')
      IF(EOSN(1).EQ.'TMVOC1    ') THEN
C+++++ SECTION FOR TMVOC1
       READ(5, 5300) NHC
       DO 501 L = 1, NHC
         READ(5, 5550) HCNAMES(L)
         READ(5, 5500)TCRITM(L),PCRITM(L),ZCRITM(L),OMEGAM(L),
     *      DIPOLMM(L)
         READ(5, 5500) TBOILM(L),VPAM(L),VPBM(L),VPCM(L),VPDM(L)
         READ(5, 5500) AMWTM(L),CPAM(L),CPBM(L),CPCM(L),CPDDM(L)
         READ(5, 5500) RHOREFM(L),TDENREF(L),DIFV0M(L),
     *      TDIFREF(L),TEXPOM(L)
         READ(5, 5500) VLOAM(L),VLOBM(L),VLOCM(L),VLODM(L),VOLCRITM(L)
         READ(5, 5500) SOLAM(L), SOLBM(L), SOLCM(L), SOLDM(L)
         READ(5, 5500) OCKM(L),FOXM,ALAMM(L)
  501  CONTINUE
 5300 FORMAT (2I5)
 5500 FORMAT (8E10.4)
 5550 FORMAT (A20)
      ELSE
C+++++ SECTION FOR T2VOC
       READ(5, 2002)TCRIT2,PCRIT2,ZCRIT2,OMEGA2,DIPOLM2
       READ(5, 2002)TBOIL2,VPA2,VPB2,VPC2,VPD2
       READ(5, 2002)AMO,CPA2,CPB2,CPC2,CPDD2
       READ(5, 2002)RHOREF2,TDENRF,DIFV02,TDIFRF,TEXPO2
       READ(5, 2002)VLOA2,VLOB2,VLOC2,VLOD2,VOLCRT2
       READ(5, 2002)SOLA2,SOLB2,SOLC2,SOLD2
       READ(5, 2002)OCK2,FOX2,ALAM2
      END IF
      GOTO5019
C
C     KEYWORD 'NCGAS'
 4000 CONTINUE
      INCG=1
C-----SPECIFY NON-CONDENSIBLE GASES FOR TMVOC V.2.0
       READ(5, 4005) NCG
       DO 4010 L = 1, NCG
         READ(5, 4015) NCGINP(L),alncg(l)
 4010  CONTINUE
 4005 FORMAT (I5)
 4015 FORMAT (A10,E10.4)
C
C-----END OF NCG SPECIFICATIONS -------------------
      GOTO5019
C
C*****READ DATA BLOCK WITH TIMES.
C     KEYWORD 'TIMES'
 2000 CONTINUE
      READ(5, 2001)ITI,ITE,DELAF,TINTER
C
C     ITI IS THE NUMBER OF TIMES PROVIDED.
C     ITE IS THE TOTAL NUMBER OF TIMES DESIRED.
C     DELAF IS THE MAXIMUM TIME STEP TO BE TAKEN AFTER ANY OF THE
C           PRESCRIBED TIMES HAVE BEEN REACHED.
C     TINTER IS THE TIME INCREMENT TO BE APPLIED FOR THE TIME VALUES
C            WITH INDEX ITI+1,ITI+2, ... ,ITE
C
C
 2001 FORMAT(2I5,3E10.4)
      READ(5, 2002)(TIS(I),I=1,ITI)
 2002 FORMAT(8E10.4)
      IF(ITE.LE.ITI.OR.TINTER.LE.0.) GOTO 5019
C
C-----COME HERE TO ASSIGN INCREMENTED TIMES.
      ITI1=ITI+1
      DO2003 I=ITI1,ITE
 2003 TIS(I)=TIS(I-1)+TINTER
C
      ITI=ITE
C
      GOTO 5019
C
C-----END OF TIMES BLOCK------------------------------------------------
C
C*****READ DATA BLOCK WITH SELECTION PARAMETERS*************************
C     KEYWORD 'SELEC'
 2100 CONTINUE
      READ(5, 1801)(IE(I),I=1,16)
      IE1=1
      IF(IE(1).GT.1) IE1=IE(1)
      IF(IE1.GT.64) PRINT 2101,IE1
 2101 FORMAT(' WWWWW WARNING WWWWW:',
     X' IE(1) =',I5,' IN BLOCK *SELEC* IS TOO LARGE; CAN READ AT MOST',
     X' 64 RECORDS'/)
      IE18=MIN(IE1*8,512)
      READ(5, 2002)(FE(I),I=1,IE18)
      GOTO 5019
C
C-----END OF SELECTION PARAMETERS---------------------------------------
C
C*****COME HERE FOR DATA BLOCK *FOFT* TO READ ELEMENT NAMES FOR WHICH
C     TIME SLICES ARE DESIRED.
C
C     KEYWORD 'FOFT '
   80 CONTINUE
      DO81 I=1,100
      READ(5, 5020)EOFT(I)
      IF(EOFT(I).EQ.'     ') GOTO82
   81 CONTINUE
   82 IOFT=I-1
      IOFU=IOFT
      IF(IOFU.GT.0) OPEN(12,FILE='FOFT',STATUS='UNKNOWN')
      REWIND 12
      GOTO5019
C
C*****COME HERE TO OPEN FILE FOR TIME-DEPENDENT GENERATION DATA.
C     KEYWORD 'GOFT '
  180 CONTINUE
      OPEN(13,FILE='GOFT',STATUS='UNKNOWN')
      REWIND 13
      DO181 I=1,100
      READ(5, 5020)EGOFT(I)
      IF(EGOFT(I).EQ.'     ') GOTO 182
  181 CONTINUE
  182 IGOFT=I-1
C.....WHEN NO ELEMENT DATA ARE GIVEN IN BLOCK GOFT, SET IGOFT
C     TO A FLAG THAT WILL CAUSE ALL ELEMENTS WITH GENERATION
C     DATA TO BE TABULATED
      IF(I.EQ.1) IGOFT=-1
      IGOFU=IGOFT
C     PRINT 183,IGOFT,(EGOFT(I),I=1,IGOFT)
  183 FORMAT(' INPUT !!!  IGOFT =',I4,' EGOFT ='/(20(1X,A5)))
      GOTO5019
C
C*****COME HERE FOR DATA BLOCK *COFT* TO READ CONNECTIONS FOR WHICH
C     TIME SLICES ARE DESIRED.
C
C     KEYWORD 'COFT '
  280 CONTINUE
      DO281 I=1,100
      READ(5, 5021)ECOFT(I)
 5021 FORMAT(A10)
      IF(ECOFT(I).EQ.'          ') GOTO282
  281 CONTINUE
  282 ICOFT=I-1
      ICOFU=ICOFT
      IF(ICOFU.GT.0) OPEN(14,FILE='COFT',STATUS='UNKNOWN')
      REWIND 14
      GOTO5019
C
C     KEYWORD 'DIFFU'
  190 CONTINUE
C.....COME HERE TO READ DIFFUSION COEFFICIENTS
      IDDIAG=IDDIAG+1
      DO191 K=1,NK
  191 READ(5, 7)(FDDIAG(I,K),I=1,NPH)
      GOTO5019
C
C     KEYWORD 'ENDFI'
 1600 PRINT 1601
 1601 FORMAT(/' HAVE READ KEYWORD *ENDFI* IN FILE INPUT ---',
     X' BYPASS FLOW SIMULATION   ',62('*')/
     X' OPTIONAL  PRINTOUT OF INPUT DATA (FOR MOP(7).NE.0) IS',
     X' AVAILABLE')
      IS=IS+1
C
C     KEYWORD 'ENDCY'
 1500 CONTINUE
      IF (IHYSTER.LT.0) THEN
         IF (.NOT.LHYSTE) THEN
            IEHYS(1)=IE(5)
            IEHYS(2)=IE(6)
            IEHYS(3)=IE(7)
         ENDIF
      ENDIF
C
C.....
      IF(EOSN(1).EQ.'TMVOC1    '.AND.INCG.EQ.0) PRINT 1501
 1501 FORMAT(/1X,131('=')//' NO NCGAS DATA BLOCK PROVIDED IN INPUT',
     X' FILE ... USE A SINGLE NCG, AIR, AS DEFAULT'//1X,131('='))
C
      IF(ICOM.NE.0) THEN
C.....COME HERE WHEN UNKNOWN KEYWORDS WERE ENCOUNTERED
C     CONSIDER SUCH RECORDS AS "COMMENTS," AND PRINT
C     THE FIRST 50 OF THEM
      ICO=MIN(50,ICOM)
      PRINT 901,H1,ICOM
  901 FORMAT(A1/23X,86('*')/23X,'*',84X,'*'/
     X23X,'*',5X,'COMMENTS IN INPUT DATA (HAVE ',
     X'ENCOUNTERED',I4,' RECORDS WITH UNKNOWN KEYWORDS)',
     X4X,'*'/23X,'*',30X,'(PRINT UP TO 50 OF THEM)',
     X30X,'*'/
     X23X,'*',84X,'*'/23X,86('*')/23X,'*',84X,'*')
      DO 902,I=1,ICO
  902 PRINT 903,COMM(I)
  903 FORMAT(23X,'*  ',A80,'  *')
      PRINT 904
  904 FORMAT(23X,'*',84X,'*'/23X,86('*'))
      ENDIF
C
      IF(KIN.EQ.0) GOTO 2410
C***** COME HERE TO PROCESS DOMAIN-SPECIFIC INITIAL CONDITIONS *******
      DO 2401 J=1,KIN
      DO 2402 I=1,NM
      IF(MAT(I).EQ.NAM(J)) GOTO 2403
 2402 CONTINUE
      GOTO 2401
 2403 IF(INRE.EQ.1) INDXIN(I)=INDICEIN(J)
      DO 2404 L=1,NKIN1
 2404 YIN(L,I)=XIN(L,J)
 2401 CONTINUE
C
 2410 CONTINUE
C-----END OF DOMAIN-SPECIFIC INITIAL CONDITIONS ----------------------
C
      RETURN
C
      END
C
      SUBROUTINE RFILE
C
C-----READ INPUT DATA FROM DISK FILES, WHICH ARE EITHER PROVIDED
C     BY THE USER, OR ARE INTERNALLY GENERATED IN SUBROUTINE
C     INPUT FROM DATA GIVEN IN THE JOB DECK.
C
C.....2-10-98: MODIFIED TO READ MULTIPLE DATA SETS WITH FLOWING
C              WELLBORE PRESSURE CORRECTIONS.
C.OCT.24,2000: MODIFIED TO SUPPORT TMVOC V. 1.0
C
C
C$$$$$$$$$ COMMON BLOCKS FOR CONNECTIONS $$$$$$$$$$$$$$$$$$$$$$$$$$$C
C
      COMMON/C1/NEX1(1)
      COMMON/C2/NEX2(1)
      COMMON/C3/DEL1(1)
      COMMON/C4/DEL2(1)
      COMMON/C5/AREA(1)
      COMMON/C6/BETA(1)
      COMMON/C7/ISOX(1)
      COMMON/C8/GLO(1)
      COMMON/C9/ELEM1(1)
      COMMON/C10/ELEM2(1)
      COMMON/C11/FVD(1)
      COMMON/C12/SIG(1)
C
C$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
C
C$$$$$ COMMON BLOCKS FOR SINKS/SOURCES $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
C
      COMMON/G1/F1(1)
      COMMON/G2/F2(1)
      COMMON/G3/F3(1)
      COMMON/G3A/PW(1)
      COMMON/G4/ELEG(1)
      COMMON/G5/SOURCE(1)
      COMMON/G6/LTABG(1)
      COMMON/G7/G(1)
      COMMON/G8/EG(1)
      COMMON/G9/NEXG(1)
      COMMON/G10/TYPE(1)
      COMMON/G11/NGIND(1)
      COMMON/G12/LCOM(1)
      COMMON/G13/PI(1)
      COMMON/G14/PWB(1)
      COMMON/G15/HG(1)
      COMMON/G16/GPO(1)
      COMMON/G17/SDENS(1)
      COMMON/G18/SSAT(1)
      COMMON/G19/GVOL(1)
      COMMON/G20/HL(1)
      COMMON/G21/HS(1)
      COMMON/G22/QVGC(1)
      COMMON/G23/QVWC(1)
      COMMON/G24/QVOC(1)
      COMMON/G25/GRAD(1)
      COMMON/G26/FF(1)
      COMMON/G27/FNAM(1)
      COMMON/G28/NFTAB(1)
      COMMON/G29/IFTIT(1)
      COMMON/G30/JFTIT(1)
      COMMON/G31/IJF(1)
      COMMON /NEW/ INDICAT(1)
      COMMON /NEW0/ INDCAT0(1)
      COMMON /NUMBER/NHC
      COMMON /DEFINI2/ INDICE
      COMMON/INTYPE/INRE
      COMMON/INDINIT/INDXIN(27)
CFi   COMMON/COMPOUNDS/ HCNAMES(20),IT2VOC
C
C$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
C
C
      COMMON/SVZ/NOITE,MOP(24)
      COMMON/P1/X(1)
      COMMON/E1/ELEM(1)
      COMMON/E2/MATX(1)
      COMMON/E3/EVOL(1)
      COMMON/E4/PHI(1)
      COMMON/E5/P(1)
      COMMON/E6/T(1)
      COMMON/AHTRAN/AHT(1)
C.....7-3-95: INCLUDE ARRAY TO BE USED FOR PERMEABILITY MODIFIERS
      COMMON/E7/PM(1)
C.....7-20-93: DEFINE COORDINATE ARRAYS
      COMMON/XYZ1/X1(1)
      COMMON/XYZ2/X2(1)
      COMMON/XYZ3/X3(1)
      COMMON/SOLID/NM,DM(27),POR(27),PER(3,27),CWET(27),SH(27)
      COMMON/SOCH/MAT(27)
      COMMON/SOLI/COM(27),EXPAN(27),CDRY(27),TORT(27),GK(27)
      COMMON/SOIN/YIN(21,27)
      COMMON/STEP/ELST
      COMMON/STE1/NST
      COMMON/DEFINI/DEP(21)
      COMMON/DFM/TIMAX,REDLT
      COMMON/DLT/NDLT,DLT(100)
      COMMON/DM/DELTEN,DELTEX,FOR,FORD
      COMMON/DMN/INUM,IPRINT,MCYC,MCYPR,MSEC,TZERO,TIMP1
      COMMON/DOP/ENTH,KDATA,QUAL
      COMMON/BALA/TFM0(27),TVM0(27),TFE0(27),TVE0(27),TSE0(27)
      COMMON/DX/K,NI,SCALE
      COMMON/POV6/TSTART
      COMMON/CYC/KCYC,ITER,ITERC,TIMIN,SUMTIM,GF,TIMOUT
      COMMON/NN/NEL,NCON,NOGN,NK,NEQ,NPH,NB,NK1,NEQ1,NBK,NSEC,NFLUX
      COMMON/NNN/NKIN,NKIN1
      COMMON/KONIT/KON,DELT,IGOOD
      COMMON/BC/NELA
      COMMON/MMC/MM,MC
      COMMON/MADIM/MNEL,MNCON,MNEQ,MNPH,MNBK,MNOGN,MGTAB,MNK
      COMMON/V/IV,IS
      COMMON/SOLVR1/MATSLV,NMAXIT,NNVVCC,IIUUNN,IISSOO,NACTDI
      COMMON/FGT1/IOFT,IOFU,IGOFT,IGOFU,NOFT(100),NGOFT(100)
      COMMON/FGT2/EOFT(100),EGOFT(100),ECOFT(100)
      COMMON/FGT3/ICOFT,ICOFU,NCOFT(100)
      COMMON/RANT2/IRANT2
C
C$$$$$$$$$ COMMON BLOCKS FOR HYSTERESIS MODEL $$$$$$$$$$$$$$$$$$$$$$$$$$
C
      COMMON/KHYST1/IHYSTER,KCYCX,IEHYS(16)
      COMMON/KHYST2/ICURV(1)
      COMMON/KHYST3/KTMPO(1)
      COMMON/KHYST4/irun(1)
      COMMON/KHYST5/IFLG1(1)
      COMMON/KHYST6/KFLAG(1)
      COMMON/SAVHYS1/SOLDH(1)
      COMMON/SAVHYS2/HOLD(1)
      COMMON/TMPHYS1/STMPO(1)
      COMMON/TMPHYS2/HTMPO(1)
      COMMON/HYS11/STRN1(1)
      COMMON/HYS12/STRN2(1)
      COMMON/HYS13/STRN3(1)
      COMMON/HYS21/HTRN1(1)
      COMMON/HYS22/HTRN2(1)
      COMMON/HYS23/HTRN3(1)
      COMMON/HYS31/SCMP1(1)
      COMMON/HYS32/SCMP2(1)
      COMMON/HYS33/BB2(1)
      COMMON/TORBENR1/SEOT(1)
      COMMON/TORBENR2/SETRN1(1)
      COMMON/TORBENR3/SOR(1)
      COMMON/TORBENR4/SOROLD(1)
      COMMON/TORBENR5/SORTMPO(1)
      common/hyssav1/strn1old(1)
      common/hyssav2/strn1tmpo(1)
      common/hyssav3/strn2old(1)
      common/hyssav4/strn2tmpo(1)
      common/hyssav5/strn3old(1)
      common/hyssav6/strn3tmpo(1)
      common/hyssav7/htrn1old(1)
      common/hyssav8/htrn1tmpo(1)
      common/hyssav9/htrn2old(1)
      common/hyssav10/htrn2tmpo(1)
      common/hyssav11/htrn3old(1)
      common/hyssav12/htrn3tmpo(1)
      common/hyssav13/scmp1old(1)
      common/hyssav14/scmp1tmpo(1)
      common/hyssav15/scmp2old(1)
      common/hyssav16/scmp2tmpo(1)
      common/hyssav17/bb2old(1)
      common/hyssav18/bb2tmpo(1)
      common/hyssa21/ hmin(1)
      common/hyssa22/ hminold(1)
      common/hyssa23/ hmintmpo(1)
      common/hyssa31/ shmin(1)
      common/hyssa32/ shminold(1)
      common/hyssa33/ shmintmpo(1)    
      common/hyssa41/ hminp(1)
      common/hyssa42/ hminpold(1)
      common/hyssa43/ hminptmpo(1)

C
      COMMON/ECH/EOSN(20)
      CHARACTER EOSN*10
C
      CHARACTER*5 EOFT,EGOFT,FFI,TYPE,TYPEN,MA12,FNAM
      CHARACTER*10 ECOFT
C
      CHARACTER*5 ELEM1,ELEM2,ELEG,SOURCE,ELEM,MAT,ELST,DENT,EL1,EL2,EL
      CHARACTER ITABG*1,TYP*4,MA1*3,MA2*2,TY1*1,TITL*80
C
      DIMENSION DEPU(21)
      SAVE ICALL
      DATA ICALL/0/
C
      ICALL=ICALL+1
      IF(ICALL.EQ.1) WRITE(11,899)
C 899 FORMAT(6X,'RFILE    1.01      6 JANUARY   1993',6X,
C 899 FORMAT(6X,'RFILE    1.02      4 FEBRUARY  1993',6X,
C 899 FORMAT(6X,'RFILE    1.02     26 OCTOBER   1995',6X,
C 899 FORMAT(6X,'RFILE    1.03     12 FEBRUARY  1997',6X,
C 899 FORMAT(6X,'RFILE    1.03      3 MARCH     1997',6X,
C 899 FORMAT(6X,'RFILE    1.03     23 JULY      1997',6X,
C 899 FORMAT(6X,'RFILE    1.03     26 SEPTEMBER 1997',6X,
C 899 FORMAT(6X,'RFILE    1.03     24 OCTOBER   1997',6X,
C 899 FORMAT(6X,'RFILE    1.1      23 JANUARY   1998',6X,
C 899 FORMAT(/6X,'RFILE    1.1      13 FEBRUARY  1998',6X,
C 899 FORMAT(/6X,'RFILE    1.1      11 MARCH     1998',6X,
C 899 FORMAT(/6X,'RFILE    1.1      16 MARCH     1998',6X,
C 899 FORMAT(/6X,'RFILE    1.1      28 MAY       1998',6X,
C 899 FORMAT(/6X,'RFILE    1.5      21 MAY       1999',6X,
C 899 FORMAT(/6X,'RFILE    1.5      18 JUNE      1999',6X,
C 899 FORMAT(/6X,'RFILE    1.5      24 OCTOBER   2000',6X,
C 899 FORMAT(/6X,'RFILE    1.5       6 NOVEMBER  2000',6X,
C 899 FORMAT(/6X,'RFILE    1.5      21 DECEMBER  2000',6X,
C 899 FORMAT(/6X,'RFILE    1.5      28 DECEMBER  2000',6X,
C 899 FORMAT(/6X,'RFILE    1.5      29 DECEMBER  2000',6X,
C 899 FORMAT(/6X,'RFILE    1.5       5 APRIL     2001',6X,
C 899 FORMAT(/6X,'RFILE    1.5      11 APRIL     2001',6X,
C 899 FORMAT(/6X,'RFILE    1.5      24 APRIL     2001',6X,
C 899 FORMAT(/6X,'RFILE    1.5       2 APRIL     2002',6X,
  899 FORMAT(/6X,'RFILE    2.1       1 JULY      2011',6X,
     X'INITIALIZE DATA FROM FILES *MESH* OR *MINC*, *GENER*,',
     X' AND *INCON*'/
     X47X,'ALSO INITIALIZES PERMEABILITY MODIFIERS AND COORDINATE ',
     X'ARRAYS'/
     X47X,'AND OPTIONALLY READS TABLES WITH FLOWING WELLBORE',
     X' PRESSURES')
C
      SCALEA=SCALE*SCALE
      SCALEV=SCALEA*SCALE
C
      PRINT 901
  901 FORMAT(' ')
C
C*****READ GEOMETRY DATA FROM FILE MESH OR MINC.***********************
C
C
C.....2-4-93: INITIALIZE FLAG FOR "DUMMY" CONNECTIONS W/ UNKNOWN BLOCKS
      NDUM=0
      IF(MC.EQ.0) THEN
      IM=4
      ELSE
      IM=10
      ENDIF
C
      REWIND IM
      READ(IM,5020) DENT
      N=0
      N0=0
 1492 CONTINUE
      N=N+1
C
      READ(IM,1499) DENT,MA1,MA2,EVOLX,AHTX,PMX,XX,YY,ZZ
 1499 FORMAT(A5,10X,A3,A2,2E10.4,4E10.4)
      IF(DENT.EQ.'     ') GOTO1502
      IF(N.LE.MNEL) GOTO 40
      PRINT 41,MNEL
   41 FORMAT(' NUMBER OF ELEMENTS SPECIFIED IN DATA BLOCK "ELEME"',
     X' EXCEEDS ALLOWABLE MAXIMUM OF ',I7/
     X' INCREASE PARAMETER *MNEL* IN MAIN PROGRAM, AND RECOMPILE'//
     X' ----------------- SKIP FLOW SIMULATION -----------------')
      IS=IS+1
      RETURN
   40 CONTINUE
      ELEM(N)=DENT
      EVOL(N)=EVOLX
      AHT(N)=AHTX
      PM(N)=PMX
      X1(N)=XX
      X2(N)=YY
      X3(N)=ZZ
      IF(MA1.EQ.'   ') GOTO15
C-----FIND MATERIAL INDEX
      MA12=MA1//MA2
      DO16 M=1,NM
      IF(MA12.EQ.MAT(M)) GOTO 17
   16 CONTINUE
      IF(MOP(7).NE.0) PRINT 18,MA12,ELEM(N)
   18 FORMAT(' REFERENCE TO UNKNOWN MATERIAL ',A5,' AT ELEMENT *',A5,
     X'* --- WILL IGNORE ELEMENT')
      N=N-1
      GOTO 1492
C
   17 MATX(N)=M
      GOTO 19
   15 CONTINUE
C-----COME HERE FOR ELEMENTS WITH DOMAIN NUMBER
      READ(MA2,'(I2)') MATX(N)
   19 CONTINUE
      IF(MATX(N).NE.0) GOTO 1498
C     COME HERE FOR ELEMENTS WITHOUT DOMAIN ASSIGNMENT, AND ASSIGN
C     THEM TO DOMAIN # 1.
      MATX(N)=1
      PRINT 1497,ELEM(N),MAT(1)
 1497 FORMAT(' WWWWW WARNING WWWWW:',
     X' ELEMENT *',A5,'* HAS NO DOMAIN ASSIGNMENT; ASSIGN TO',
     X' DOMAIN # 1, *',A5,'*')
 1498 CONTINUE
      IF(EVOL(N).LE.0..AND.N0.EQ.0) N0=N-1
C     N0 IS THE INDEX PRECEDING THE FIRST ELEMENT WITH V = 0.
      IF(SCALE.EQ.1.) GOTO1490
      EVOL(N)=EVOL(N)*SCALEV
C.....10-26-95: ADD PROPER SCALING OF HEAT TRANSFER AREAS
      AHT(N)=AHT(N)*SCALEA
 1490 CONTINUE
      GOTO 1492
 1502 NEL=N-1
      NELA=NEL
      IF(N0.NE.0) NELA=N0
C     NELA IS THE NUMBER OF "ACTIVE" ELEMENTS; IT IS ASSUMED THAT
C     ALL ELEMENTS WITH INDEX N > NELA ARE BOUNDARY ELEMENTS.
C
      NST=0
      IF(ELST.EQ.'     ') GOTO80
C
      DO81 N=1,NEL
      IF(ELEM(N).EQ.ELST) GOTO82
   81 CONTINUE
      GOTO 80
C
   82 NST=N
   80 CONTINUE
C
C.....FOFT-HANDLING
      IF(IOFT.GT.0) THEN
C     COME HERE TO ASSIGN INDEX NUMBERS OF ELEMENTS FOR WHICH TIME
C     PLOTS ARE DESIRED
      DO50 I=1,IOFT
      DO51 N=1,NEL
      IF(ELEM(N).NE.EOFT(I)) GOTO51
      NOFT(I)=N
      GOTO50
   51 CONTINUE
C
      NOFT(I)=0
      IOFU=IOFU-1
      PRINT 52,EOFT(I)
   52 FORMAT(' HAVE ENCOUNTERED UNKNOWN ELEMENT *',A5,'* IN DATA BLOCK',
     X' "FOFT" - WILL IGNORE UNKNOWN ELEMENT')
C
   50 CONTINUE
      ENDIF
C
      READ(IM,5020) DENT
      NCON=0
C! ---GJM - BEGIN: DETERMINE ACTIVE DIMENSIONS
      ISUMD1 = 0
      ISUMD2 = 0
      ISUMD3 = 0
C! ---GJM - END: DETERMINE ACTIVE DIMENSIONS
 1493 CONTINUE
      IF(NCON.LT.MNCON) GOTO 42
      PRINT 43,MNCON
   43 FORMAT('NUMBER OF CONNECTIONS SPECIFIED IN DATA BLOCK "CONNE"',
     X' EXCEEDS ALLOWABLE MAXIMUM OF ',I7/
     X' INCREASE PARAMETER *MNCON* IN MAIN PROGRAM, AND RECOMPILE'//
     X' ----------------- SKIP FLOW SIMULATION -----------------')
      IS=IS+1
      RETURN
   42 CONTINUE
C
      N=NCON+1
      READ(IM,1503) ELEM1(N),ELEM2(N),ISOX(N),DEL1(N),
     BDEL2(N),AREA(N),BETA(N),SIG(N)
 1503 FORMAT(2A5,15X,I5,5E10.4)
      IF(ELEM1(N).EQ.'+++  ') GOTO1504
      IF(ELEM1(N).EQ.'     ') GOTO 1507
      NCON=NCON+1
C
C ----------------
C! ..... GJM - BEGIN: DETERMINING DIMENSIONS
C ----------------
C
         IF(ISOX(N).EQ.1) ISUMD1 = ISUMD1+1
         IF(ISOX(N).EQ.2) ISUMD2 = ISUMD2+1
         IF(ISOX(N).EQ.3) ISUMD3 = ISUMD3+1
C
C ----------------
C! ..... GJM - END: DETERMINING DIMENSIONS
C ----------------
C
      IF(SCALE.EQ.1.) GOTO1491
      DEL1(N)=DEL1(N)*SCALE
      DEL2(N)=DEL2(N)*SCALE
      AREA(N)=AREA(N)*SCALEA
 1491 CONTINUE
      GOTO 1493
C
 1504 READ(IM,1505) (NEX1(N),NEX2(N),N=1,NCON)
      DO33 J=1,NCON
      IF(NEX1(J).NE.0.AND.NEX2(J).NE.0) GOTO 33
      NDUM=NDUM+1
   33 CONTINUE
 1505 FORMAT(16I5)
      GOTO1479
C
 1507 BACKSPACE IM
      WRITE(IM,1508)
 1508 FORMAT('+++  ')
C
C ----------------
C! ..... GJM - BEGIN: DETERMINING DIMENSIONS
C ----------------
C
      IF(ISUMD1.NE.0.AND.ISUMD2.NE.0.AND.ISUMD3.NE.0) NACTDI = 3
      IF(ISUMD2.EQ.0.OR.ISUMD3.EQ.0)  NACTDI = 2
      IF(ISUMD2.EQ.0.AND.ISUMD3.EQ.0) NACTDI = 1
C
      PRINT 6081, ISUMD1,ISUMD2,ISUMD3
 6081 FORMAT('*',T10,'THE NUMBERS OF CONNECTIONS IN THE X-, Y- AND Z-',
     &              'DIRECTIONS ARE ',I5,', ',I5,' AND ',I5,
     &              ' RESPECTIVELY',T130,'*',
     &            /,'*',T130,'*',/,130('*'),//)
C
C ----------------
C! ..... GJM - END: DETERMINING DIMENSIONS
C ----------------
C
      DO26 J=1,NCON
      EL1=ELEM1(J)
      EL2=ELEM2(J)
      DO27 N=1,NEL
      IF(ELEM(N).NE.EL1) GOTO27
      NEX1(J)=N
      GOTO28
   27 CONTINUE
      NDUM=NDUM+1
      NEX1(J)=0
      IF(MOP(7).NE.0)
     XPRINT 127,EL1,J
  127 FORMAT(' REFERENCE TO UNKNOWN ELEMENT ',A5,15H AT CONNECTION ,I7,
     A' --- WILL IGNORE CONNECTION')
      GOTO 26
C
   28 DO29 N=1,NEL
      IF(ELEM(N).NE.EL2) GOTO29
      NEX2(J)=N
      GOTO300
   29 CONTINUE
      NDUM=NDUM+1
      NEX2(J)=0
      IF(MOP(7).NE.0)
     XPRINT 127,EL2,J
  300 CONTINUE
   26 CONTINUE
C
   22 WRITE(IM,1505) (NEX1(N),NEX2(N),N=1,NCON)
      ENDFILE IM
 1479 CONTINUE
C
C.....HANDLING OF "DUMMY" CONNECTIONS
      IF(NDUM.NE.0) THEN
C        COME HERE WHEN DUMMY CONNECTIONS ARE PRESENT
         N=0
         DO30 J=1,NCON
         IF(NEX1(J).NE.0.AND.NEX2(J).NE.0) THEN
         N=N+1
         ELEM1(N)=ELEM1(J)
         ELEM2(N)=ELEM2(J)
         NEX1(N)=NEX1(J)
         NEX2(N)=NEX2(J)
         ISOX(N)=ISOX(J)
         DEL1(N)=DEL1(J)
         DEL2(N)=DEL2(J)
         AREA(N)=AREA(J)
         BETA(N)=BETA(J)
         SIG(N)=SIG(J)
         ELSE
         ENDIF
   30    CONTINUE
         PRINT 31,NCON,NDUM,N
   31    FORMAT(' OF',I7,' CONNECTIONS READ FROM FILE *MESH*,',
     X' NDUM =',I7,' CONNECTIONS MAKE REFERENCE TO UNKNOWN ELEMENTS',
     X' AND WILL BE IGNORED'/
     X' INITIALIZE A TOTAL OF NCON =',I7,' CONNECTIONS TO THE DATA',
     X' ARRAYS'/)
         NCON=N
      ELSE
      PRINT 32,NCON
   32 FORMAT(' ALL NCON =',I7,' CONNECTIONS READ FROM FILE *MESH*',
     X' REFERENCE KNOWN ELEMENTS, AND HAVE BEEN INITIALIZED TO THE',
     X' DATA ARRAYS'/)
      ENDIF
C
C-----END OF FILE MESH.------------------------------------------------
C
C.....COFT-HANDLING: TABULATE TIME-DEPENDENT CONNECTION DATA
      IF(ICOFT.GT.0) THEN
C     COME HERE TO ASSIGN INDEX NUMBERS OF CONNECTIONS FOR WHICH TIME
C     PLOTS ARE DESIRED
      DO250 I=1,ICOFT
      DO251 N=1,NCON
      IF(ELEM1(N)//ELEM2(N).NE.ECOFT(I)) GOTO251
      NCOFT(I)=N
      GOTO250
  251 CONTINUE
C
      NCOFT(I)=0
      ICOFU=ICOFU-1
      PRINT 252,ECOFT(I)(1:5),ECOFT(I)(6:10)
  252 FORMAT(' HAVE ENCOUNTERED UNKNOWN CONNECTION (',A5,')-(',A5,')',
     X' IN DATA BLOCK "COFT" - WILL IGNORE UNKNOWN CONNECTION')
C
  250 CONTINUE
      ENDIF
C
C.....9-26-97: OPTIONAL BLOCK-BY-BLOCK PERMEABILITY MODIFICATION
      CALL PMIN
C
C     INITIALIZE DIFFUSIVE VAPOR FLUX AS ZERO.
      DO25 J=1,NCON
      FVD(J)=0.
   25 CONTINUE
C
C*****READ SINKS/SOURCES FROM FILE GENER.******************************
C
      REWIND 3
      NOGN=0
      READ(3,5020,END=1415) DENT
      NGL=0
      KF=0
      KIJ=0
C
 1483 CONTINUE
      IF(NOGN.LT.MNOGN) GOTO 44
      PRINT 45,MNOGN
   45 FORMAT(' NUMBER OF SINKS/SOURCES SPECIFIED IN DATA BLOCK "GENER"',
     X' EXCEEDS ALLOWABLE MAXIMUM OF ',I6/
     X' INCREASE PARAMETER *MNOGN* IN MAIN PROGRAM, AND RECOMPILE'//
     X' ----------------- SKIP FLOW SIMULATION -----------------')
      IS=IS+1
      RETURN
   44 CONTINUE
C
      N=NOGN+1
      READ(3,1481) ELEG(N),SOURCE(N),LTABG(N),TYPE(N)
     A,G(N),EG(N),HG(N)
 1481 FORMAT(2A5,15X,I5,5X,A5,3G10.4)
      IF(ELEG(N).EQ.'+++  ') GOTO1482
      IF(ELEG(N).EQ.'     ') GOTO 1406
      NOGN=NOGN+1
C
      NGIND(N)=NGL
      LCOM(N)=0
      TYPEN=TYPE(N)
      TYP=TYPEN(1:4)
      TY1=TYPEN(1:1)
      ITABG=TYPEN(5:5)
C     PRINT 99,N,ELEG(N),SOURCE(N),TYPEN,TYP,ITABG
   99 FORMAT(' #',I3,' ELEM = *',A5,'*   SOURCE = *',A5,
     X'*   TYPEN = *',A5,'*   TYP = *',A5,'*   ITABG = *',A5,'*')
C
      IF(TY1.EQ.CHAR(70).OR.TY1.EQ.CHAR(102)) THEN
C.....COME HERE WHEN FIRST CHARACTER OF A TYPE DESIGNATION
C     IS AN UPPER OR LOWER-CASE F; SIGNIFYING A WELL ON 
C     DELIVERABILITY WITH FLOWING WELLBORE PRESSURE CORRECTION
C
      DO 140 I=1,5
         IF(TYPEN(I:I).EQ.' ') THEN
            FFI=TYPEN(1:I-1)
            GOTO 141
         ENDIF
  140 CONTINUE
         FFI=TYPEN
  141 CONTINUE
C
      OPEN(9,FILE=FFI,STATUS='OLD',ERR=142)
      GOTO 149
C.....IF FILE SPECIFIED IN THE F-TYPE GENER ITEM IS NOT
C     AVAILABLE; IGNORE THE ITEM
  142 CONTINUE
      PRINT 138,FFI
  138 FORMAT(' INVALID F-TYPE GENER ITEM;  FILE *',A5,'* IS',
     X' NOT AVAILABLE')
      GOTO 139
C
  149 CONTINUE
C.....COME HERE FOR VALID F-TYPE ITEM
         LCOM(N)=NK1+2
         PI(N)=G(N)
C
C.....CHECK WHETHER THE CURRENT WELLBORE PRESSURE FILE HAS
C     ALREADY BEEN READ.
         IF(KF.GE.1) THEN
            DO 143 K=1,KF
            IF(FFI.EQ.FNAM(K)) GOTO 144
  143       CONTINUE
            GOTO 145
  144       CONTINUE
C.....ASSIGN INDEX OF PREVIOUSLY READ WELLBORE PRESSURE
C     FILE TO CURRENT GENER ITEM
            NFTAB(N)=K
            GOTO 1483
C
         ENDIF
C
  145 CONTINUE
C.....READ NEW WELLBORE PRESSURE FILE
         KF=KF+1
         FNAM(KF)=FFI
         NFTAB(N)=KF
         READ(9,146) TITL
  146    FORMAT(A80)
         READ(9,147) NG,NH
         IFTIT(KF)=NG
         JFTIT(KF)=NH
  147    FORMAT(2I5)
         READ(9,148) (PW(KIJ+I),I=1,NG)
  148    FORMAT(8E10.4)
         READ(9,148) (PW(KIJ+NG+J),J=1,NH)
      DO136 I=1,NG
  136 READ(9,148) (PW(KIJ+NG+NH+(I-1)*NH+J),J=1,NH)
         IJF(KF)=KIJ
         KIJ=KIJ+NG+NH+NG*NH
      ENDIF
C
C.....END OF INITIALIZING F-TYPE GENER ITEMS
C
      IF(TYP.EQ.'WATR') TYP='COM2'
      IF(TYP.EQ.'NACL') TYP='COM1'
      IF(TYP.EQ.'HEAT') LCOM(N)=NK1
      IF(TYP.EQ.'MASS') LCOM(N)=1
      IF(TYP.EQ.'WATE') LCOM(N)=1
      IF(TYP.EQ.'AIR ') LCOM(N)=2
      IF(TYP.EQ.'COM1') LCOM(N)=1
      IF(TYP.EQ.'COM2') LCOM(N)=2
      IF(TYP.EQ.'COM3') LCOM(N)=3
      IF(TYP.EQ.'COM4') LCOM(N)=4
      IF(TYP.EQ.'COM5') LCOM(N)=5
      IF(TYP.EQ.'COM6') LCOM(N)=6
      IF(TYP.EQ.'COM7') LCOM(N)=7
      IF(TYP.EQ.'COM8') LCOM(N)=8
C
      IF(TYP.EQ.'COM9') LCOM(N)=9
      IF(TYP.EQ.'CO10') LCOM(N)=10
      IF(TYP.EQ.'CO11') LCOM(N)=11
      IF(TYP.EQ.'CO12') LCOM(N)=12
      IF(TYP.EQ.'CO13') LCOM(N)=13
      IF(TYP.EQ.'CO14') LCOM(N)=14
      IF(TYP.EQ.'CO15') LCOM(N)=15
      IF(TYP.EQ.'CO16') LCOM(N)=16
      IF(TYP.EQ.'CO17') LCOM(N)=17
      IF(TYP.EQ.'CO18') LCOM(N)=18
      IF(TYP.EQ.'CO19') LCOM(N)=19
      IF(TYP.EQ.'CO20') LCOM(N)=20
C
      IF(TYP.EQ.'DELV') THEN 
C-----COME HERE FOR TYPE = DELV, IN WHICH CASE A PRODUCTIVITY INDEX
C     (PI) AND A FLOWING WELLBORE PRESSURE (PWB) ARE SPECIFIED.
         LCOM(N)=NK1+1
         PI(N)=G(N)
         PWB(N)=EG(N)
      ENDIF
C
      IF(TYP.EQ.'VOL.') THEN
C-----COME HERE IF VOLUMETRIC PRODUCTION RATE IS SPECIFIED.
         LCOM(N)=NK1+3
         GVOL(N)=G(N)
      ENDIF
C
      IF (LCOM(N).NE.0) GO TO 13
  139 PRINT 14,TYPEN,ELEG(N),SOURCE(N)
   14 FORMAT(' IGNORE UNKNOWN GENERATION OPTION *',A5,'* AT ELEMENT *',
     1  A5,'* SOURCE *',A5,'*')
      NOGN=NOGN-1
      GO TO 1483
   13 CONTINUE
C
C
      LTABA=IABS(LTABG(N))
      IF(LTABA.LE.1.OR.TYP.EQ.'DELV') GOTO1483
      IF(NGL+LTABA.LE.MGTAB) GOTO 46
      PRINT 47,MGTAB
   47 FORMAT(' NUMBER OF TABULAR GENERATION DATA SPECIFIED IN DATA',
     X' BLOCK "GENER" EXCEEDS ALLOWABLE MAXIMUM OF ',I7/
     X' INCREASE PARAMETER *MGTAB* IN MAIN PROGRAM, AND RECOMPILE'//
     X' ----------------- SKIP FLOW SIMULATION -----------------')
      IS=IS+1
      RETURN
   46 CONTINUE
C
      READ(3,36) (F1(NGL+L),L=1,LTABA)
      READ(3,36) (F2(NGL+L),L=1,LTABA)
      IF(ITABG.NE.' ') READ(3,36) (F3(NGL+L),L=1,LTABA)
      NGL=NGL+LTABA
      GOTO 1483
 1482 CONTINUE
      IF(NOGN.EQ.0) GOTO 1415
C
      READ(3,1505) (NEXG(N),N=1,NOGN)
      GOTO1415
C
 1406 IF(NOGN.EQ.0) GOTO 1415
      BACKSPACE 3
      WRITE(3,1508)
      DO37 N=1,NOGN
      EL=ELEG(N)
      DO38 J=1,NEL
      IF(ELEM(J).NE.EL) GOTO38
      NEXG(N)=J
      GOTO37
   38 CONTINUE
      NEXG(N)=0
      PRINT 128,EL,N
  128 FORMAT(' REFERENCE TO UNKNOWN ELEMENT *',A5,'* AT SOURCE #',I5,
     A' --- WILL IGNORE SOURCE')
   37 CONTINUE
C
   39 WRITE(3,1505) (NEXG(N),N=1,NOGN)
      ENDFILE 3
 1415 CONTINUE
C
C.....GOFT-HANDLING (TABULATING GENERATION DATA)
      IF(IGOFT.GT.0) THEN
         DO 150 I=1,IGOFT
         DO 151 N=1,NOGN
C     PRINT 183,I,N,EGOFT(I),ELEG(N)
  183 FORMAT(' RFILE !!!  I =',I4,'  N =',I4,'  ELEG = ',A5,
     X'   EGOFT = ',A5)
         IF(ELEG(N).NE.EGOFT(I).OR.NEXG(N).EQ.0) GOTO 151
         NGOFT(I)=N
         GOTO 150
  151    CONTINUE
         NGOFT(I)=0
         IGOFU=IGOFU-1
         PRINT 152,EGOFT(I)
  152 FORMAT(' ELEMENT *',A5,'* IN DATA BLOCK "GOFT" DOES NOT',
     X' HAVE A VALID "GENER" ITEM - IGNORE THIS AND PROCEED')
  150    CONTINUE
      ENDIF
C
C-----END OF FILE GENER.-----------------------------------------------
C     PRINT 137,(PW(IJ),IJ=1,KIJ)
  137 FORMAT(' PW-ARRAY ======================================='/
     X(10(3X,E10.4)))
C
C*****READ INITIAL CONDITIONS FROM FILE INCON.*************************
C
 1300 REWIND 1
      IF(NI.EQ.0) GOTO1412
C
C-----COME HERE FOR START.
      DO1409 N=1,NEL
      PHI(N)=POR(MATX(N))
      NLOC=(N-1)*NKIN1
ccd..feb2008  Assign default hysteresis properties in case elem not in
c    incon or if incon does not yet contain hyster parameters (mop(13)=1).
c    Assumes liquid saturated, on icurv=0
      IF (IHYSTER.LT.0) THEN
        SOLDH(n)=1.
        HOLD(n)=0.
        ICURV(n)=0
        SOROLD(n)=0.
        STRN1old(n)=0.
        STRN2old(n)=0.
        STRN3old(n)=0.
        HTRN1old(n)=0.
        HTRN2old(n)=0.
        HTRN3old(n)=0.
        SCMP1old(n)=0.
        SCMP2old(n)=0.
        BB2old(n)=0.
        hminold(n)=0.
        shminold(n)=1.
        hminpold(n)=0.
      ENDIF
ccd..end
C
      IF(YIN(1,MATX(N)).NE.0.) GOTO 1410
      IF(INRE.EQ.1) INDICAT(N) = INDICE
      DO 2000 I=1,NKIN1
 2000 X(NLOC+I)=DEP(I)
      GOTO 1409
C
 1410 CONTINUE
      DO 1411 I=1,NKIN1
      IF(INRE.EQ.1) INDICAT(N)=INDXIN(MATX(N))      
 1411 X(NLOC+I)=YIN(I,MATX(N))
C
C
 1409 CONTINUE
      READ(1,5020,END=1703) DENT
C
C-----COME HERE FOR ASSIGNING INITIAL CONDITIONS.
C
      IUN=0
 2002 CONTINUE
      INDICEE=1
      IF(INRE.EQ.1) THEN
C+++++ READ INDICES
        READ (1,6500,END=1700) EL, PORX, INDICEE
      ELSE
        READ(1,1403,END=1700) EL,PORX
      END IF
 1403 FORMAT(A5,10X,E15.8,5E10.4)
 6500 FORMAT (A5,10X,E15.8,I2)
C
      IF(EL.EQ.'     ') THEN
         IF(IUN.NE.0) PRINT 1407,IUN
 1407    FORMAT(/' WWWWW WARNING WWWWW: INCON DATA AT ',I5,' UNKNOWN ',
     X   'ELEMENTS HAVE BEEN IGNORED')
         RETURN
      ENDIF
      IF(EL.EQ.'+++  ') GOTO 2004
C
      READ(1,1402,END=1700) (DEPU(I),I=1,NKIN1)
 1402 FORMAT(4E20.13)
C
CFiHysteresisB
C --- Read history for hysteresis model
      IF (IHYSTER.LT.0.AND.MOP(13).GT.0) THEN
         READ(1,7004) SLD,HLD,ICRV,SORE,STR1,STR2,STR3,
     &                HTR1,HTR2,HTR3,SCM1,SCM2,BBB2,
     &                HMIN2,SHMIN2,HMINP2
 7004 FORMAT(2E20.14,I5,15X,E20.14,4(/,3E20.14))
      ENDIF
CFiHysteresisE

      DO 2005 J=1,NEL
      IF(ELEM(J).NE.EL) GOTO 2005
      JLOC=(J-1)*NKIN1
      IF(INRE.EQ.1) INDICAT(J) = INDICEE
      IF(PORX.NE.0.) PHI(J)=PORX
C
      DO 2006 I=1,NKIN1
 2006 X(JLOC+I)=DEPU(I)
CFiHysteresisB
      IF (IHYSTER.LT.0) THEN
         SOLDH(J)=SLD
         HOLD(J)=HLD
         ICURV(J)=ICRV
         SOROLD(J)=SORE
         STRN1OLD(J)=STR1
         STRN2OLD(J)=STR2
         STRN3OLD(J)=STR3
         HTRN1OLD(J)=HTR1
         HTRN2OLD(J)=HTR2
         HTRN3OLD(J)=HTR3
         SCMP1OLD(J)=SCM1
         SCMP2OLD(J)=SCM2
         BB2OLD(J)=BBB2
         HMINOLD(J)=HMIN2
         SHMINOLD(J)=SHMIN2
         HMINPOLD(J)=HMINP2
      ENDIF
CFiHysteresisE
      GOTO 2002
 2005 CONTINUE
C
      IUN=IUN+1
      IF(IUN.LE.50) PRINT 1408,EL
 1408 FORMAT(' WILL IGNORE INITIAL CONDITION AT UNKNOWN ELEMENT ',A7)
C
      GOTO 2002
C
 2004 CONTINUE
         IF(IUN.NE.0) PRINT 1407,IUN
      READ(1,74) KCYC,ITERC,NMX,TSTART,TIMIN
C
C.....1-16-92: INTERROGATE NUMBER OF DOMAINS FED THROUGH *INCON*
C              ALWAYS USE NM FROM INPUT DATA
      IF(NMX.NE.NM) THEN
      PRINT 2008,NMX,NM,NM
 2008 FORMAT(' WWWWW WARNING WWWWW:',
     X' NUMBER OF DOMAINS READ FROM FILE *INCON* IS',I3/
     X'                      THIS IS DIFFERENT FROM NM =',I3,
     X' PROVIDED THROUGH INPUT DATA'/
     X' THE LATTER VALUE NM =',I3,' WILL BE USED'/)
      ENDIF
      IF(TIMAX.NE.0..AND.TIMIN.GE.TIMAX) TIMIN=0.
      IF(MOP(19).EQ.0) RETURN
      PRINT 2007,MOP(19)
 2007 FORMAT(//49H SUBROUTINE RFILE: HAVE ENCOUNTERED '+++' ON FILE,
     X' *INCON* --- RESET MOP(19) =',I2,' TO DEFAULT VALUE ZERO.'/)
      MOP(19)=0
      RETURN
C
 1412 CONTINUE
C-----COME HERE TO READ RESTART INFORMATION.
      READ(1,5020,END=1700) DENT
      IF(DENT.NE.'INCON') GOTO 1700
      DO 1702 N=1,NEL
      NLOC=(N-1)*NKIN1
      IF(INRE.EQ.1) THEN
C+++++ READ INDICES
        READ (1,6500,END=1700) EL, PHI(N),INDICEE 
      ELSE
        READ(1,1403,END=1700) EL,PHI(N)
      END IF
CAB
      IF(INRE.EQ.1) INDICAT(N) = INDICEE
      IF(PHI(N).EQ.0.) PHI(N)=POR(MATX(N))
      READ(1,1402,END=1700) (X(NLOC+I),I=1,NKIN1)
CFiHysteresisB
C --- Read history for hysteresis model
         IF (IHYSTER.LT.0.AND.MOP(13).GT.0) THEN
           READ(1,7004)
     &          SOLDH(N),HOLD(N),ICURV(N),sorold(n),STRN1old(N),
     &          STRN2old(N),STRN3old(N),HTRN1old(N),HTRN2old(N),
     &          HTRN3old(N),SCMP1old(N),SCMP2old(N),BB2old(N),
     &          hminold(n),shminold(n),hminpold(n)
         ENDIF
CFiHysteresisE
 1702 CONTINUE
C
      READ(1,5020) DENT
      IF(DENT.EQ.'     ') RETURN
      READ(1,74) KCYC,ITERC,NMX,TSTART,TIMIN
      IF(NMX.NE.NM) THEN
      PRINT 2008,NMX,NM,NM
      ENDIF
   74 FORMAT(3I5,2E15.8)
      IF(TIMAX.NE.0..AND.TIMIN.GE.TIMAX) TIMIN=0.
      IF(MOP(19).EQ.0) RETURN
      PRINT 2007,MOP(19)
      MOP(19)=0
   75 FORMAT(5E15.8)
C
C-----END OF FILE INCON.-----------------------------------------------
C
 1703 RETURN
 5020 FORMAT(A5)
   36 FORMAT(4E14.7)
C
 1700 PRINT 1701,DENT
 1701 FORMAT(' IMPROPER FORMAT ON FILE INCON --- IDENTIFIER "',A5,'"',//
     X' ----------------- SKIP FLOW SIMULATION -----------------')
      IS=IS+1
      RETURN
      END
C
      SUBROUTINE INDATA
C
C-----THIS ROUTINE PROVIDES A PRINTOUT OF MOST OF THE INPUT DATA.
C     IT IS CALLED ONLY IF MOP(7).NE.0.
C
C
C
C$$$$$$$$$ COMMON BLOCKS FOR ELEMENTS $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
C
C     THESE BLOCKS HAVE A LENGTH OF NEL (= NUMBER OF ELEMENTS)
C
      COMMON/E1/ELEM(1)
      COMMON/E2/MATX(1)
      COMMON/E3/EVOL(1)
      COMMON/E4/PHI(1)
      COMMON/E5/P(1)
      COMMON/E6/T(1)
C
C$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
C
C
C$$$$$$$$$ COMMON BLOCKS FOR PRIMARY VARIABLES $$$$$$$$$$$$$$$$$$$$$$$$$
C
C     THESE BLOCKS HAVE A LENGTH OF NEL*NEQ
C
      COMMON/P1/X(1)
      COMMON/P2/DX(1)
      COMMON/P3/DELX(1)
      COMMON/P4/R(1)
      COMMON/P5/DOLD(1)
C
C$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
C
C$$$$$$$$$ COMMON BLOCKS FOR CONNECTIONS $$$$$$$$$$$$$$$$$$$$$$$$$$$C
C
      COMMON/C1/NEX1(1)
      COMMON/C2/NEX2(1)
      COMMON/C3/DEL1(1)
      COMMON/C4/DEL2(1)
      COMMON/C5/AREA(1)
      COMMON/C6/BETA(1)
      COMMON/C7/ISOX(1)
      COMMON/C8/GLO(1)
      COMMON/C9/ELEM1(1)
      COMMON/C10/ELEM2(1)
C
C$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
C
      COMMON/TITLE/TITLE
      CHARACTER*80 TITLE
C
      COMMON/G10/TYPE(1)
      COMMON/ECH/EOSN(20)
      COMMON /NEW/ INDICAT(1)
      CHARACTER EOSN*10
      COMMON/NNN/NKIN,NKIN1
C
      COMMON/CONTST/RE1,RE2,RERM,NER,KER,DFAC
C
C$$$$$ COMMON BLOCKS FOR SINKS/SOURCES $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
C
      COMMON/G1/F1(1)
      COMMON/G2/F2(1)
      COMMON/G3/F3(1)
      COMMON/G4/ELEG(1)
      COMMON/G5/SOURCE(1)
      COMMON/G6/LTABG(1)
      COMMON/G7/G(1)
      COMMON/G8/EG(1)
      COMMON/G9/NEXG(1)
      COMMON/G11/NGIND(1)
      COMMON/G12/LCOM(1)
      COMMON/G13/PI(1)
      COMMON/G14/PWB(1)
      COMMON/G15/HG(1)
      COMMON/G16/GPO(1)
      COMMON/G17/SDENS(1)
      COMMON/G18/SSAT(1)
      COMMON/G19/GVOL(1)
      COMMON/G20/HL(1)
      COMMON/G21/HS(1)
      COMMON/G22/QVGC(1)
      COMMON/G23/QVWC(1)
      COMMON/G24/QVOC(1)
      COMMON/G25/GRAD(1)
      COMMON/G26/FF(1)
C
C$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
C
      COMMON/SVZ/NOITE,MOP(24)
      COMMON/SOLID/NM,DM(27),POR(27),PER(3,27),CWET(27),SH(27)
      COMMON/SOCH/MAT(27)
      COMMON /SOLI/ COM(27), EXPAN(27), CDRY(27), TORT(27), GK(27)
      COMMON/AMMIS/MA,IPIV,U,IAB,NZ
      COMMON/STEP/ELST
      COMMON/STE1/NST
      COMMON/DEFINI/DEP(21)
      COMMON/DG/WUP,WNR
      COMMON/DFM/TIMAX,REDLT
      COMMON/DLT/NDLT,DLT(100)
      COMMON/DM/DELTEN,DELTEX,FOR,FORD
      COMMON/DMN/INUM,IPRINT,MCYC,MCYPR,MSEC,TZERO,TIMP1
      COMMON/DOP/ENTH,KDATA,QUAL
      COMMON/DX/K,NI,SCALE
      COMMON/POV6/TSTART
      COMMON/CYC/KCYC,ITER,ITERC,TIMIN,SUMTIM,GF,TIMOUT
      COMMON/PATCH/SING
      COMMON/NN/NEL,NCON,NOGN,NK,NEQ,NPH,NB,NK1,NEQ1,NBK,NSEC,NFLUX
      COMMON/BIND/DIFF0,TEXP,BE
      COMMON/TIM/ITI,DELAF,ITPR,TIS(100),ITCO,NOWTIM,DELTMX
      COMMON/FF/H1
C
C$$$$$$$$$ COMMON BLOCKS FOR HYSTERESIS MODEL $$$$$$$$$$$$$$$$$$$$$$$$$$
C
      COMMON/KHYST1/IHYSTER,KCYCX,IEHYS(16)
      COMMON/KHYST2/ICURV(1)
      COMMON/KHYST3/KTMPO(1)
      COMMON/KHYST4/irun(1)
      COMMON/KHYST5/IFLG1(1)
      COMMON/KHYST6/KFLAG(1)
      COMMON/SAVHYS1/SOLDH(1)
      COMMON/SAVHYS2/HOLD(1)
      COMMON/TMPHYS1/STMPO(1)
      COMMON/TMPHYS2/HTMPO(1)
      COMMON/HYS11/STRN1(1)
      COMMON/HYS12/STRN2(1)
      COMMON/HYS13/STRN3(1)
      COMMON/HYS21/HTRN1(1)
      COMMON/HYS22/HTRN2(1)
      COMMON/HYS23/HTRN3(1)
      COMMON/HYS31/SCMP1(1)
      COMMON/HYS32/SCMP2(1)
      COMMON/HYS33/BB2(1)
      COMMON/TORBENR1/SEOT(1)
      COMMON/TORBENR2/SETRN1(1)
      COMMON/TORBENR3/SOR(1)
      COMMON/TORBENR4/SOROLD(1)
      COMMON/TORBENR5/SORTMPO(1)
      common/hyssav1/strn1old(1)
      common/hyssav2/strn1tmpo(1)
      common/hyssav3/strn2old(1)
      common/hyssav4/strn2tmpo(1)
      common/hyssav5/strn3old(1)
      common/hyssav6/strn3tmpo(1)
      common/hyssav7/htrn1old(1)
      common/hyssav8/htrn1tmpo(1)
      common/hyssav9/htrn2old(1)
      common/hyssav10/htrn2tmpo(1)
      common/hyssav11/htrn3old(1)
      common/hyssav12/htrn3tmpo(1)
      common/hyssav13/scmp1old(1)
      common/hyssav14/scmp1tmpo(1)
      common/hyssav15/scmp2old(1)
      common/hyssav16/scmp2tmpo(1)
      common/hyssav17/bb2old(1)
      common/hyssav18/bb2tmpo(1)
      common/hyssa21/ hmin(1)
      common/hyssa22/ hminold(1)
      common/hyssa23/ hmintmpo(1)
      common/hyssa31/ shmin(1)
      common/hyssa32/ shminold(1)
      common/hyssa33/ shmintmpo(1)    
      common/hyssa41/ hminp(1)
      common/hyssa42/ hminpold(1)
      common/hyssa43/ hminptmpo(1)

      common/intype/inre
      CHARACTER*1 H1
      CHARACTER*5 ELEM1,ELEM2,ELEM,ELEG,SOURCE,MAT,ELST,TYPEN,TYPE
      CHARACTER*4 TYP
      SAVE ICALL
      DATA ICALL/0/
      ICALL=ICALL+1
      IF(ICALL.EQ.1) WRITE(11,899)
C 899 FORMAT(6X,'INDATA   1.0       5 MARCH     1991',6X,
C 899 FORMAT(6X,'INDATA   1.0      24 OCTOBER   2000',6X,
C 899 FORMAT(6X,'INDATA   1.0       3 NOVEMBER  2000',6X,
C 899 FORMAT(6X,'INDATA   1.0      21 DECEMBER  2000',6X,
C 899 FORMAT(6X,'INDATA   1.0       5 APRIL     2001',6X,
C 899 FORMAT(6X,'INDATA   1.0      19 APRIL     2001',6X,
c 899 FORMAT(6X,'INDATA   1.0       5 SETTEMBRE 2001',6X,
C 899 FORMAT(6X,'INDATA   1.0       2 MAY       2002',6X,
  899 FORMAT(6X,'INDATA   2.1       1 JULY      2011',6X,
     X'PROVIDE PRINTOUT OF MOST DATA PROVIDED THROUGH FILE *INPUT*')
C
      H1=CHAR(12)
      PRINT 100,H1
      PRINT 150,TITLE
      PRINT 1000
      PRINT 1000
      PRINT 200
      PRINT 250
      PRINT 300,NOITE,KDATA,MCYC,MSEC,MCYPR,(MOP(I),I=1,24),
     XDIFF0,TEXP,BE
      PRINT 350
      PRINT 401,TSTART,TIMAX,DELTEN,DELTMX,ELST,GF,REDLT,SCALE
  401 FORMAT(4(5X,E10.4),10X,A5,3(5X,E10.4))
      IF(NDLT.EQ.0) GOTO 10
      PRINT 450
      PRINT 451,((DLT(J+8*(N-1)),J=1,8),N=1,NDLT)
  451 FORMAT(5X,8(5X,E10.4))
      GO TO 20
   10 PRINT 500
   20 PRINT 550
      PRINT 400,RE1,RE2,U,WUP,WNR,DFAC,FOR
      PRINT 600
C.... PRINT INDICEE
      PRINT 650,(DEP(I),I=1,NKIN1)
      PRINT 1000
      PRINT 700
      DO 1 I=1,NM
      PRINT 750
      PRINT 800,
     AI,MAT(I),DM(I),POR(I),CWET(I),SH(I),COM(I),EXPAN(I)
      PRINT 810
      PRINT 820,PER(1,I),PER(2,I),PER(3,I)
    1 CONTINUE
      PRINT 1000
      PRINT 830
      PRINT 840
      DO 2 I=1,NEL
      PRINT 850,ELEM(I),MATX(I),EVOL(I)
    2 CONTINUE
      PRINT 1000
      PRINT 860
      PRINT 870
      DO 3 I=1,NCON
      PRINT 880,ELEM1(I),ELEM2(I),ISOX(I),DEL1(I),DEL2(I),AREA(I),
     ABETA(I)
    3 CONTINUE
      IF(NOGN .LE. 0 )GO TO 30
      PRINT 1000
      PRINT 890
      IDELV=0
      DO 4 I=1,NOGN
      IF(LCOM(I).EQ.NK1+1) GOTO 6
      PRINT 895
      PRINT 900,ELEG(I),SOURCE(I), TYPE(I),G(I),EG(I)
      IF(LTABG(I) .GT. 1)GO TO 40
      PRINT 896
      GO TO 50
   40 PRINT 897
   50 CONTINUE
      GOTO 4
    6 CONTINUE
      IDELV=IDELV+1
      IF(IDELV.EQ.1) LAY=LTABG(I)
      IF(IDELV.EQ.1) PRINT 885,LAY
  885 FORMAT(/5X,'WELL ON DELIVERABILITY   $$$$$$$$   OPEN IN',I3,
     A'  LAYERS   $$$$$$$$'//5X,' ELEMENT    SOURCE   TYPE',
     &'              PI             PWB          DEL(Z)'/)
      TYPEN=TYPE(I)
      TYP=TYPEN(1:4)   
      PRINT 900,ELEG(I),SOURCE(I),TYP,PI(I),PWB(I),HG(I)
    4 CONTINUE
   30 PRINT 1000
      PRINT 910
      PRINT 920
      DO 5 I=1,NEL
c     IF(EOSN(1).EQ.'TMVOC1    ') THEN
      if(inre.ne.0) then
C+++++ SECTION FOR TMVOC
       PRINT 950,ELEM(I),INDICAT(I),PHI(I),(X((I-1)*NKIN1+J),J=1,NKIN1)
      ELSE
       PRINT 930,ELEM(I),PHI(I),(X((I-1)*NKIN1+J),J=1,NKIN1)
      END IF
CFiHysteresisB
      IF (IHYSTER.LT.0.AND.MOP(13).GT.0)
     &   PRINT 7006, SOLDH(N),HOLD(N),ICURV(N),sorold(n),STRN1old(N),
     &   STRN2old(N),STRN3old(N),HTRN1old(N),HTRN2old(N),HTRN3old(N),
     &   SCMP1old(N),SCMP2old(N),BB2old(N),hminold(n),shminold(n),
     &   hminpold(n)
 7006 FORMAT(2E20.14,I5,4(/,4E20.10))
CFiHysteresisE

    5 CONTINUE
      PRINT 1000
      PRINT 940
      PRINT 1000
      PRINT 1000
      RETURN
  100 FORMAT(A1/'     TOUGH2 INPUT DATA'/)
  150 FORMAT(5X,'PROBLEM TITLE: ',A80,/)
  200 FORMAT(/,5X,'PROBLEM SPECIFICATIONS'/)
  250 FORMAT('     NOITE     KDATA      MCYC      MSEC     MCYPR
     A      MOP                   DIFF0          TEXP            BE')
  300 FORMAT(5(4X,I5,1X),6X,24I1,3(5X,E10.4),/)
  350 FORMAT(117H       TSTART         TIMAX          DELTEN         DEL
     ATMX            ELST        GF             REDLT         SCALE )
  400 FORMAT(5X,8(5X,E10.4))
  450 FORMAT(/,5X,' VARIABLE TIME STEPS ARE PRESCRIBED')
  500 FORMAT(/,5X,' A CONSTANT TIME STEP OF DELTEN IS PRESCRIBED'/)
  550 FORMAT('            RE1            RE2            U           ',
     A'   WUP            WNR            DFAC           FOR')
  600 FORMAT(/'     DEP(1)       DEP(2)       DEP(3)',
     A'       DEP(4)       DEP(5)       DEP(6)',
     &'       DEP(7)       DEP(8)       DEP(9)',
     &'      DEP(10)')
  650 FORMAT(10(1X,E12.6))
  700 FORMAT(/,5X,'ROCK PROPERTIES'/)
  750 FORMAT(5X,'DOMAIN     MAT        DENSITY        POROSITY     ',
     A'CONDUCTIVITY     HEAT CAP       COMPR          EXPAN')
  800 FORMAT(5X,I4,6X,A5,6(5X,E10.4),/)
  810 FORMAT(2X,'          PERM1          PERM2          PERM3')
  820 FORMAT(5X,3(5X,E10.4),/)
  830 FORMAT(/,5X,'ELEMENTS'/)
  840 FORMAT(5X,'        ELEMENT       MATERIAL         VOLUME')
  850 FORMAT(14X,A5,8X,I5,10X,E10.4)
  860 FORMAT(/,5X,'CONNECTIONS'/)
  870 FORMAT(5X,'          ELEM1          ELEM2           ISOT    ',
     A'        DEL1           DEL2           AREA           BETA'/)
  880 FORMAT(5X,2(10X,A5),9X,I5,6X,4(5X,E10.4))
  890 FORMAT(/,5X,'GENERATION DATA'/)
  895 FORMAT(/'      ELEMENT    SOURCE   TYPE            RATE',
     A'          ENTHALPY'/)
  896 FORMAT(5X,'CONSTANT RATE IS SPECIFIED',/)
  897 FORMAT(5X,'VARIABLE RATE IS SPECIFIED',/)
  900 FORMAT(5X,2X,A5,5X,A5,4X,A4,5X,4(5X,E10.4))
  910 FORMAT(/,5X,'INITIAL CONDITIONS',/)
  920 FORMAT(1X,'ELEME INDC   POROSITY       X1         ',    
     A'  X2           X3           X4           X5    ',
     &'       X6           X7           X8',/)
  930 FORMAT(1X,A5,4X,9(E13.6))
  940 FORMAT(/,5X,'END OF INPUT DATA'/)
  950 FORMAT(1X,A5,2X,I1,1X,9(E13.6))
 1000 FORMAT(1X,131('*'))
      END
C
      SUBROUTINE CYCIT
C
C-----THIS ROUTINE IS THE EXECUTIVE ROUTINE FOR MARCHING IN TIME.
C
C
C
      COMMON/NN/NEL,NCON,NOGN,NK,NEQ,NPH,NB,NK1,NEQ1,NBK,NSEC,NFLUX
      COMMON/CONTST/RE1,RE2,RERM,NER,KER,DFAC
      COMMON/P1/X(1)
      COMMON/P2/DX(1)
      COMMON/E1/ELEM(1)
      COMMON/VINWES/AI(1)
      COMMON/SECPAR/PAR(1)
      COMMON/STEP/ELST
      COMMON/STE1/NST
      COMMON/AMMIS/MA,IPIV,U,IAB,NZ
      COMMON/KC/KC
      COMMON/DFM/TIMAX,REDLT
      COMMON/DOP/ENTH,KDATA,QUAL
      COMMON/SVZ/NOITE,MOP(24)
      COMMON/DM/DELTEN,DELTEX,FOR,FORD
      COMMON/DMN/INUM,IPRINT,MCYC,MCYPR,MSEC,TZERO,TIMP1
      COMMON/KONIT/KON,DELT,IGOOD
      COMMON/CYC/KCYC,ITER,ITERC,TIMIN,SUMTIM,GF,TIMOUT
      COMMON/TIM/ITI,DELAF,ITPR,TIS(100),ITCO,NOWTIM,DELTMX
      COMMON/FAIL/IHLV
      COMMON/BC/NELA
      COMMON/FGT1/IOFT,IOFU,IGOFT,IGOFU,NOFT(100),NGOFT(100)
      COMMON/FGT3/ICOFT,ICOFU,NCOFT(100)
      COMMON/MARE/AMRES
      COMMON/KHYST1/IHYSTER,KCYCX,IEHYS(16)
      COMMON/KHYST2/ICURV(1)
      COMMON/KHYST3/KTMPO(1)
      COMMON/KHYST4/irun(1)
      COMMON/KHYST5/IFLG1(1)
      COMMON/KHYST6/KFLAG(1)
      CHARACTER*5 ELEM,ELST
C
      SAVE ICALL
      DATA ICALL/0/
      ICALL=ICALL+1
      IF(ICALL.EQ.1) WRITE(11,899)
C 899 FORMAT(/6X,'CYCIT    1.0       5 MARCH     1991',6X,
C 899 FORMAT(/6X,'CYCIT    1.01      4 JANUARY   1995',6X,
C 899 FORMAT(/6X,'CYCIT    1.01     24 JULY      1997',6X,
C 899 FORMAT(/6X,'CYCIT    1.01     24 OCTOBER   1997',6X,
C 899 FORMAT(/6X,'CYCIT    1.01     28 MAY       1998',6X,
C 899 FORMAT(/6X,'CYCIT    1.01     21 JUNE      2001',6X,
c 899 FORMAT(/6X,'CYCIT    1.01     20 SEPTEMBER 2001',6X,
  899 FORMAT(/6X,'CYCIT    1.01     26 AUGUST    2010',6X,
     X'EXECUTIVE ROUTINE FOR MARCHING IN TIME')
C
      KC=0
      SUMTIM=TIMIN
      TIMOUT=SUMTIM
      KON=1
      DELT=DELTEN
      MA=0
      IAB=0
      IPIV=0
      IGOOD=0
      NSTL=(NST-1)*NK1
      NSTL2=(NST-1)*NEQ1*NSEC
C
      ITER=0
      IHLV=0
      KIT=0
      IQIT=0
      CALL EOS
      CALL BALLA
C
   30 KCYC=KCYC+1
      if(mod(kcyc-1,10).eq.0.and.kc.gt.0) then
         call wrifi
         kcyc=kcyc+1
c
         IF(MOP(15).ne.0) then
            WRITE(6,13)
            REWIND 8
            WRITE(8,14) (AI(N),N=1,NELA)
            endfile 8
         endif   
C
      endif   
C
C-----COME HERE FOR NEW TIME STEP.
C
      KC=KC+1
      IF(MOP(2).NE.0) PRINT 29,KC,KCYC,DELT,SUMTIM
   29 FORMAT(/23H CYCIT ..........  KC =,I4,8H  KCYC =,I4,9H  DELT = ,E1
     A2.6,11H  SUMTIM = ,E12.6/)
      IF(IQIT.NE.0) GOTO 10
      IF(TIMAX.NE.0..AND.SUMTIM.EQ.TIMAX) GOTO10
      IF(KC.GT.MCYC) GOTO10
      CALL SEC(TCUR)
      IF(MSEC.NE.0.AND.TCUR-TZERO.GT.MSEC) GOTO10
C
      IHALVE=0
C
   52 ITER=0
      NOW=0
C-----PRINTOUT OCCURS FOR NOW=1
C
      NOWTIM=0
      IF(ITI.NE.0) CALL TSTEP
      DELTEX=DELT
      KON=1
C
    3 CONTINUE
C
C-----COME HERE FOR NEXT ITERATION.
C
      IF(ITER.NE.0) GOTO19
      DO20 N=1,NEL
      NLOC=(N-1)*NK1
      DO20 K=1,NK1
      DX(NLOC+K)=0.
   20 CONTINUE
C
   19 CONTINUE
C
      ITER=ITER+1
      ITERC=ITERC+1
C
      FORD=FOR*DELTEX
C
C-----COMPUTE ACCUMULATION-, SOURCE-, AND FLOW-TERMS.
      CALL MULTI
      IF(IGOOD.EQ.3) GOTO 50
C
C-----CHECK FOR CONVERGENCE.
      IF(RERM.LE.RE1) GOTO51
C
      IF (MOP(1).NE.0) THEN
         IF (IHYSTER.EQ.0) THEN
            PRINT 22,KCYC,ITER,DELTEX,RERM,ELEM(NER),KER
         ELSE
            PRINT 222,KCYC,ITER,DELTEX,RERM,ELEM(NER),KER,
     &                 PAR((NER-1)*NEQ1*NSEC+1),KTMPO(NER)
   22 FORMAT(' ...ITERATING...  AT [',I4,',',I3,'] --- DELTEX = ',E12.6,
     A'   MAX. RES. = ',E12.6,'  AT ELEMENT ',A5,'  EQUATION ',I3)
  222 FORMAT(' ...ITERATING...  AT [',I4,',',I3,'] --- DELTEX = ',E12.6,
     A'   MAX. RES. = ',E12.6,'  AT ELEMENT ',A5,'  EQUATION ',I3,
     &       ' Sg=',E11.5,' ktmpo=',i3)
         ENDIF
      ENDIF
      IF(MOP(1).LT.2) GOTO 23
      NIT=NER
      IF(NST.NE.0) NIT=NST
      NIT1=(NIT-1)*NK1
      NIT2=(NIT-1)*NEQ1*NSEC
      PNIT=X(NIT1+1)+DX(NIT1+1)
      PRINT 24,ELEM(NIT),DX(NIT1+1),DX(NIT1+2),
     XPAR(NIT2+NSEC-1),PNIT,PAR(NIT2+1)
   24 FORMAT(31X,'     AT *',A5,'* ...   DX1= ',
     XE12.6,6H DX2= ,E12.6,5H T = ,F7.3,5H P = ,F9.0,5H S = ,E12.6)
   23 CONTINUE
C
      IF(AMRES.NE.0..AND.RERM.GT.AMRES) THEN
         PRINT 150,RERM
  150 FORMAT(' +++++++++ EXCESSIVE RESIDUAL = ',E12.6)
         GOTO50
      ENDIF
      IF(ITER.GT.NOITE) GOTO50
C
      CALL LINEQ
      IF(IGOOD.NE.0) GOTO50
      CALL EOS
      IF(IGOOD.EQ.0) GOTO54
   50 DELT=DELTEX/REDLT
C-----REDUCE TIME STEP.
      IGOOD=0
C     COME HERE TO CHECK ON NUMBER OF PREVIOUS TIME STEPS THAT
C     CONVERGED ON ITER=1.
      IF(KIT.LT.2.OR.MOP(16).EQ.0) GOTO 25
      PRINT 26,KCYC,DELTEX
   26 FORMAT(/' +++++++++ CONVERGENCE FAILURE ON TIME STEP #',I4,
     X' WITH DT = ',E12.6,
     X' SECONDS, FOLLOWING TWO STEPS THAT CONVERGED ON ITER = 1'/
     X'           STOP EXECUTION AFTER NEXT TIME STEP')
      IQIT=1
C
   25 CONTINUE
      PRINT 53,KCYC,ITER,DELT
   53 FORMAT(32H +++++++++ REDUCE TIME STEP AT (,I4,1H,,I3,32H) ++++++++
     A++++++++   NEW DELT = ,E12.6)
C
      IHALVE=IHALVE+1
      IF (IHALVE.LE.25) GO TO 11
C-----STOP AFTER 25 ATTEMPTS TO REDUCE TIME STEP.
      PRINT 12,DELTEX
   12 FORMAT(//' NO CONVERGENCE AFTER 25 REDUCTIONS OF TIME STEP SIZE'/
     1  ' LAST TIME STEP WAS ',E12.6,' SECONDS '//
     2  ' !!!!!!!!!! STOP EXECUTION !!!!!!!!!!')
C.....1-4-95: WRITE OUTPUT, FILES, BEFORE STOPPING.
      CALL OUT
C-----WRITE RESULTS ON FILE SAVE.
      CALL WRIFI
      IF(MOP(15).EQ.0) THEN  
      REWIND 8
C      WRITE(6,13)
      PRINT 13
      WRITE(8,14) (AI(N),N=1,NELA)
      ENDIF
      STOP
C
   11 ITER=0
      IHLV=1
      CALL EOS
      IHLV=0
      GOTO52
C
C-----COME HERE FOR CONVERGENCE.
   51 KON=2
      IF (IHYSTER.LT.0) CALL SAVEHS
ccd..feb2008  if hyster error occurred in SAVEHS, cut DELT and repeat step
      if(igood.ne.0)goto 50
C
C
      IF(MOP(15).GE.1) CALL QLOSS
      CALL CONVER
C
C     COUNT NUMBER OF CONSECUTIVE TIME STEPS THAT CONVERGE ON
C     ITER = 1.
      IF(ITER.GT.1) KIT=0
      IF(ITER.EQ.1) KIT=KIT+1
      IF(KIT.LT.10) GOTO 55
      PRINT 56
   56 FORMAT(' +++++++++ FOR 10 CONSECUTIVE TIME STEPS HAVE',
     X' CONVERGENCE ON ITER = 1'/
     X'           WRITE OUT CURRENT DATA, THEN STOP EXECUTION')
      IQIT=1
   55 CONTINUE
C
      IF(KON.EQ.2.AND.NST.NE.0) PRINT 49,ELEM(NST),KCYC,ITER,SUMTIM,
     ADELTEX,DX(NSTL+1),DX(NSTL+2),
     BPAR(NSTL2+NSEC-1),X(NSTL+1),PAR(NSTL2+1),ICURV(NER)
      IF(KON.EQ.2.AND.NST.EQ.0) PRINT 49,ELEM(NER),KCYC,ITER,SUMTIM,
     ADELTEX,DX((NER-1)*NK1+1),DX((NER-1)*NK1+2),
     XPAR((NER-1)*NEQ1*NSEC+NSEC-1),
     BX((NER-1)*NK1+1),PAR((NER-1)*NEQ1*NSEC+1),ICURV(NER)
   49 FORMAT(1X,A5,1H(,I4,1H,,I3,1H),6H ST = ,E12.6,6H DT = ,E12.6,
     A' DX1= ',E12.6,6H DX2= ,E12.6,5H T = ,F7.3,5H P = ,F9.0,
     B' S = ',E12.6,' ICURV=',I1)
C
      IF(KON.EQ.2.AND.(IOFU.GT.0.OR.IGOFU.NE.0.OR.ICOFU.NE.0)) 
     XCALL FGTAB
C-----DETERMINE WHETHER PRINTOUT IS REQUIRED.
      IF(IQIT.NE.0) NOW=1
      IF(NOWTIM.EQ.1.AND.KON.EQ.2) NOW=1
      IF(KON.EQ.2.AND.MOD(KCYC,MCYPR).EQ.0) NOW=1
   54 IF(KDATA.GE.10.AND.MOD(KCYC,MCYPR).EQ.0) NOW=1
      IF(NOW.EQ.0) GOTO33
      CALL OUT
      IF(KON.EQ.2) CALL BALLA
   33 CONTINUE
C
C
      GOTO(3,30),KON
C
C-----WRITE RESULTS ON FILE SAVE.
   10 CALL WRIFI
      IF(MOP(15).EQ.0) RETURN
      REWIND 8
C      WRITE(6,13)
      PRINT 13
   13 FORMAT(' WRITE COEFFICIENTS FOR SEMI-ANALYTICAL HEAT LOSS CALCULAT
     XION ONTO FILE *TABLE*')
      WRITE(8,14) (AI(N),N=1,NELA)
   14 FORMAT(4E20.13)
C
      RETURN
      END
C
      SUBROUTINE PHAS(N,J2LOC,GN,FAC)
C
C-----THIS SUBROUTINE CALCULATES PHASE COMPOSITION AND FLOWING
C     ENTHALPY OF MULTIPHASE SOURCE FLUID.
C
      COMMON/SECPAR/PAR(1)
      COMMON/SVZ/NOITE,MOP(24)
      COMMON/KONIT/KON,DELT,IGOOD
      COMMON/G8/EG(1)
      COMMON/G26/FF(1)
      COMMON/NN/NEL,NCON,NOGN,NK,NEQ,NPH,NB,NK1,NEQ1,NBK,NSEC,NFLUX
      COMMON/DKM/D(21,22)
C.... FOR TMVOC
      COMMON/ECH/EOSN(20)
      COMMON/G16/GPO(1)
      COMMON/G32AB/EGP(1)
      CHARACTER EOSN*10
      DIMENSION AMMIX(3)
C
      SAVE ICALL
      DATA ICALL/0/
      ICALL=ICALL+1
      IF(ICALL.EQ.1) WRITE(11,899)
C 899 FORMAT(6X,'PHAS     1.0       3 NOVEMBER  1997',6X,
C 899 FORMAT(6X,'PHAS     1.0       5 NOVEMBER  1999',6X,
C 899 FORMAT(6X,'PHAS     1.0      26 OCTOBER   2000',6X,
C 899 FORMAT(6X,'PHAS     1.0      10 NOVEMBER  2000',6X,
c 899 FORMAT(6X,'PHAS     1.0      10 OCTOBER   2001',6X,
  899 FORMAT(6X,'PHAS     1.0      25 OCTOBER   2002',6X,
     X'CALCULATE COMPOSITION AND ENTHALPY OF SOURCE FLUID')
C
      IC=MOP(9)+1
C-----LOOP OVER NEQ1 SETS OF PARAMETERS.
      DO 101 M=1,NEQ1
      JLM2=J2LOC+(M-1)*NSEC
C
C-----ZERO OUT BLOCK (J,J).
      DO 1011 K=1,NK1
 1011 D(K,M)=0.
C
      FFS=0.
C-----FFS IS THE SUM OVER MOBILITY*DENSITY IN ALL PHASES.
C-----NOW COMPUTE FRACTIONAL FLOWS IN THE VARIOUS PHASES.
C
      IF(EOSN(1).EQ.'TMVOC1    ') THEN
C+++++ SECTION FOR TMVOC: COMPUTE AVERAGE MOLECULAR WEIGHT OF PHASES 
         CALL AMOLW(JLM2,AMMIX(1),AMMIX(2),AMMIX(3))
         AMMIX(1)=AMMIX(1)*1.E-3
         AMMIX(2)=AMMIX(2)*1.E-3
         AMMIX(3)=AMMIX(3)*1.E-3
      END IF
C
      DO 30 NP=1,NPH
      J2LNP=JLM2+(NP-1)*NBK
C
      FF((N-1)*NPH+NP)=0.
C
      IF(EOSN(1).EQ.'TMVOC1    ') THEN
C+++++SECTION FOR TMVOC: FRACTIONAL FLOWS ARE CONVERTED TO A MASS BASIS
        IF(IC.EQ.1.AND.PAR(J2LNP+3).NE.0.)
     &  FF((N-1)*NPH+NP)=PAR(J2LNP+2)*PAR(J2LNP+4)*AMMIX(NP)
     &    /PAR(J2LNP+3)
        IF(IC.EQ.2) FF((N-1)*NPH+NP)=PAR(J2LNP+1)*PAR(J2LNP+4)
     &    *AMMIX(NP) 
      ELSE
C     ..HERE FRACTIONAL FLOWS ARE COMPUTED ON A MASS BASIS
        IF(IC.EQ.1.AND.PAR(J2LNP+3).NE.0.)
     &  FF((N-1)*NPH+NP)=PAR(J2LNP+2)*PAR(J2LNP+4)/PAR(J2LNP+3)
        IF(IC.EQ.2) FF((N-1)*NPH+NP)=PAR(J2LNP+1)*PAR(J2LNP+4)
      ENDIF
C      
      FFS=FFS+FF((N-1)*NPH+NP)
C
      IF(MOP(4).GE.5) PRINT 3,M,NP,PAR(J2LNP+2),PAR(J2LNP+3),
     APAR(J2LNP+4),FF((N-1)*NPH+NP),FFS
    3 FORMAT(' M=',I2,' NP=',I1,' K=',E12.6,' VIS=',E12.6,' D=',E12.6,
     &' FF=',E12.6,' FFS=',E12.6)
C
   30 CONTINUE
C
      EG(N)=0.
      IF(EOSN(1).EQ.'TMVOC1    ') THEN
C+++++ SECTION FOR TMVOC: COMPUTE AVERAGE MOLECULAR WEIGHT OF PHASES 
       GPO(N)=0.
       EGP(N)=0.
      END IF
C
C-----RENORMALIZE FRACTIONAL FLOWS TO 1, AND COMPUTE CONTRIBUTIONS
C     OF COMPONENTS IN PHASES.
C
      DO 31 NP=1,NPH
        J2LNP=JLM2+(NP-1)*NBK
        IF(FFS.NE.0.) THEN
C  ..      RENORMALIZED FRACT. FLOWS ON A MASS BASIS
           FF((N-1)*NPH+NP)=FF((N-1)*NPH+NP)/FFS
        ENDIF
        EG(N)=EG(N)+FF((N-1)*NPH+NP)*PAR(J2LNP+5)
C
        IF(EOSN(1).EQ.'TMVOC1    '.AND.FF((N-1)*NPH+NP).NE.0.) THEN
C+++++     SECTION FOR TMVOC
           GPO(N)=GPO(N)+ GN*FF((N-1)*NPH+NP)/AMMIX(NP)
           EGP(N)=EGP(N)+ GN*FF((N-1)*NPH+NP)/AMMIX(NP)*PAR(J2LNP+5)
        ENDIF
C
        DO 32 K=1,NK
        IF(EOSN(1).EQ.'TMVOC1    '.AND.FF((N-1)*NPH+NP).NE.0.) THEN
C+++++    SECTION FOR TMVOC
          D(K,M)=D(K,M)-FAC*GN*FF((N-1)*NPH+NP)/AMMIX(NP)*
     &           PAR(J2LNP+NB+K)
        ELSE      
          D(K,M)=D(K,M)-FAC*GN*FF((N-1)*NPH+NP)*PAR(J2LNP+NB+K)
        ENDIF
   32   CONTINUE
   31 CONTINUE
C
c     IF(EOSN(1).EQ.'TMVOC1    ') THEN
      IF(EOSN(1).EQ.'TMVOC1    '.AND.GPO(N).NE.0.) THEN
C+++++   SECTION FOR TMVOC
         EGP(N)=EGP(N)/GPO(N)
         D(NK1,M)=D(NK1,M)-FAC*GPO(N)*EGP(N)
      ELSE
         D(NK1,M)=D(NK1,M)-FAC*GN*EG(N)
      ENDIF
C
  101 CONTINUE
C
      RETURN
      END
C
      SUBROUTINE PHASD(N,JLOCP,J2LOC,FAC)
C
C-----THIS SUBROUTINE CALCULATES PHASE COMPOSITION, FLOW RATE,
C     AND FLOWING ENTHALPY FOR WELLS ON DELIVERABILITY
C
      COMMON/P1/X(1)
      COMMON/P2/DX(1)
      COMMON/P3/DELX(1)
      COMMON/SECPAR/PAR(1)
      COMMON/G7/G(1)
      COMMON/G8/EG(1)
      COMMON/G13/PI(1)
      COMMON/G14/PWB(1)
      COMMON/G26/FF(1)
      COMMON/SVZ/NOITE,MOP(24)
      COMMON/KONIT/KON,DELT,IGOOD
      COMMON/NN/NEL,NCON,NOGN,NK,NEQ,NPH,NB,NK1,NEQ1,NBK,NSEC,NFLUX
      COMMON/DKM/D(21,22)
C.... FOR TMVOC
      COMMON/ECH/EOSN(20)
      COMMON/G16/GPO(1)
      COMMON/G32AB/EGP(1)
      CHARACTER EOSN*10
      DIMENSION AMMIX(3)
C
      SAVE ICALL
      DATA ICALL/0/
      ICALL=ICALL+1
      IF(ICALL.EQ.1) WRITE(11,899)
C 899 FORMAT(6X,'PHASD    1.0      16 JANUARY   1998',6X,
C 899 FORMAT(6X,'PHASD    1.0      27 MARCH     1998',6X,
C 899 FORMAT(6X,'PHASD    1.0      10 NOVEMBER  2000',6X,
C 899 FORMAT(6X,'PHASD    1.0      12 APRIL     2001',6X,
  899 FORMAT(6X,'PHASD    1.0      14 NOVEMBER  2001',6X,
     X'CALCULATE COMPOSITION, RATE AND ENTHALPY FOR WELLS',
     X' ON DELIVERABILITY')
C
      IC=MOP(9)+1
      PIN=PI(N)
C-----LOOP OVER NEQ1 SETS OF PARAMETERS.
      DO 101 M=1,NEQ1
      JLM2=J2LOC+(M-1)*NSEC
      IF(EOSN(1).EQ.'EWASG     ') THEN
C.....   RENORMALIZE PRODUCTIVITY INDEX FOR PERMEABILITY REDUCTION
C        PIN=PI(N)*PAR(JLM2+2*NBK+6)
         PIN=PI(N)*PAR(JLM2+2*NBK+3)
      ENDIF
C
C-----ZERO OUT BLOCK (J,J).
      DO 1011 K=1,NK1
 1011 D(K,M)=0.
C
      FFS=0.
C-----FFS IS THE SUM OVER MOBILITY*DENSITY IN ALL PHASES.
C-----NOW COMPUTE FRACTIONAL FLOWS IN THE VARIOUS PHASES.
      G(N)=0.
C
       IF(EOSN(1).EQ.'TMVOC1    ') THEN
C+++++   SECTION FOR TMVOC
         CALL AMOLW(JLM2,AMMIX(1),AMMIX(2),AMMIX(3))
         AMMIX(1)=AMMIX(1)*1.E-3
         AMMIX(2)=AMMIX(2)*1.E-3
         AMMIX(3)=AMMIX(3)*1.E-3
       END IF
C
      DO 30 NP=1,NPH
      J2LNP=JLM2+(NP-1)*NBK
C
      FF((N-1)*NPH+NP)=0.
C
      IF(EOSN(1).EQ.'TMVOC1    ') THEN
C+++++SECTION FOR TMVOC: FRACTIONAL FLOWS ARE CONVERTED TO A MASS BASIS
        IF(IC.EQ.1.AND.PAR(J2LNP+3).NE.0.)
     &  FF((N-1)*NPH+NP)=PAR(J2LNP+2)*PAR(J2LNP+4)*AMMIX(NP)
     &    /PAR(J2LNP+3)
        IF(IC.EQ.2) FF((N-1)*NPH+NP)=PAR(J2LNP+1)*PAR(J2LNP+4)
     &    *AMMIX(NP) 
      ELSE
C     ..HERE FRACTIONAL FLOWS ARE COMPUTED ON A MASS BASIS
        IF(IC.EQ.1.AND.PAR(J2LNP+3).NE.0.)
     A  FF((N-1)*NPH+NP)=PAR(J2LNP+2)*PAR(J2LNP+4)/PAR(J2LNP+3)
C
        IF(IC.EQ.2) FF((N-1)*NPH+NP)=PAR(J2LNP+1)*PAR(J2LNP+4)
C
CC      PRINT 120,NP,FF((N-1)*NPH+NP),PAR(J2LNP+1),PAR(J2LNP+4)
CC 120  FORMAT('NP, FF, PAR1, PAR4=', I5,3(1X,E10.4) )
C
      ENDIF
C
C
C=====COMPUTE FLOW RATES FOR EACH PHASE AND TOTAL RATE FOR WELL ON
C     DELIVERABILITY.===================================================
C
      DPRES=0.
      IF(M.EQ.2) DPRES=DELX(JLOCP+1)
      DELP=X(JLOCP+1)+DX(JLOCP+1)+DPRES+PAR(J2LNP+6)-PWB(N)
      IF(DELP.LE.0.) DELP=0.
C-----COMPUTE MASS FLOW RATE.
      FF((N-1)*NPH+NP)=FF((N-1)*NPH+NP)*PIN*DELP
      G(N)=G(N)-FF((N-1)*NPH+NP)
      FFS=FFS+FF((N-1)*NPH+NP)
C
C=====END OF DELIVERABILITY SECTION.====================================
C
      IF(MOP(4).GE.5) PRINT 3,M,NP,PAR(J2LNP+2),PAR(J2LNP+3),
     APAR(J2LNP+4),DELP,FF((N-1)*NPH+NP),G(N)
    3 FORMAT(3H M=,I2,4H NP=,I1,3H K=,E12.6,5H VIS=,E12.6,3H D=,E12.6,
     A' DELP=',E12.6,' FF=',E12.6,' GN=',E12.6)
C
   30 CONTINUE
C
      GN=G(N)
      EG(N)=0.
      IF(EOSN(1).EQ.'TMVOC1    ') THEN
C+++++ SECTION FOR TMVOC: COMPUTE AVERAGE MOLECULAR WEIGHT OF PHASES 
       GPO(N)=0.
       EGP(N)=0.
      END IF
C
C-----RENORMALIZE FRACTIONAL FLOWS TO 1, AND COMPUTE CONTRIBUTIONS
C     OF COMPONENTS IN PHASES.
C
      DO 31 NP=1,NPH
      J2LNP=JLM2+(NP-1)*NBK
C  .. RENORMALIZED FRACT. FLOWS ON A MASS BASIS
      IF(FFS.NE.0.)
     &    FF((N-1)*NPH+NP)=FF((N-1)*NPH+NP)/FFS
      EG(N)=EG(N)+FF((N-1)*NPH+NP)*PAR(J2LNP+5)
C
      IF(EOSN(1).EQ.'TMVOC1    '.AND.FF((N-1)*NPH+NP).NE.0.) THEN
C+++++   SECTION FOR TMVOC
         GPO(N)=GPO(N)+ GN*FF((N-1)*NPH+NP)/AMMIX(NP)
         EGP(N)=EGP(N)+ GN*FF((N-1)*NPH+NP)/AMMIX(NP)*PAR(J2LNP+5)
      ENDIF
C
      DO 32 K=1,NK
        IF(EOSN(1).EQ.'TMVOC1    '.AND.FF((N-1)*NPH+NP).NE.0.) THEN
C+++++    SECTION FOR TMVOC
          D(K,M)=D(K,M)-FAC*GN*FF((N-1)*NPH+NP)/AMMIX(NP)*
     &           PAR(J2LNP+NB+K)
        ELSE      
          D(K,M)=D(K,M)-FAC*GN*FF((N-1)*NPH+NP)*PAR(J2LNP+NB+K)
        ENDIF
   32 CONTINUE
C
   31 CONTINUE
C
      IF(EOSN(1).EQ.'TMVOC1    '.AND.GPO(N).NE.0.) THEN
C+++++   SECTION FOR TMVOC
         EGP(N)=EGP(N)/GPO(N)
         D(NK1,M)=D(NK1,M)-FAC*GPO(N)*EGP(N)
      ELSE
         D(NK1,M)=D(NK1,M)-FAC*GN*EG(N)
      ENDIF
C
  101 CONTINUE
C
      RETURN
      END
C
      SUBROUTINE GCOR(N,LTABA)
C
C-----THIS SUBROUTINE CALCULATES A VERY SIMPLE GRAVITY CORRECTION
C     TO FLOWING BOTTOMHOLE PRESSURE OF MULTI-FEEDZONE WELLS
C     ON DELIVERABILITY.
C
C=====COME HERE FOR WELL ON DELIVERABILITY, AND COMPUTE GRAVITY
C     CORRECTION TO FLOWING BOTTOMHOLE PRESSURE.========================
C     CODING CAN HANDLE UP TO THREE PHASES FLOWING SIMULTANEOUSLY
C
C     COMPUTE THIS CORRECTION ONCE AND FOR ALL, AT BEGINNING OF EACH
C     TIME STEP, USING NON-INCREMENTED VARIABLES.
C
C     THE GRAVITY-CORRECTIONS FOR FLOWING WELLBORE PRESSURE ARE
C     COMPUTED FOR ALL LAYERS IN WHICH A WELL ON DELIVERABILITY
C     IS OPEN. THE COMPUTATION IS MADE WHEN THE SOURCE CORRESPONDING
C     TO THE BOTTOM LAYER IS ENCOUNTERED. THIS IS SINGLED OUT BY
C     HAVING "TYPE = DELV" AND "LTAB = NUMBER OF LAYERS".
C
      COMMON/P1/X(1)
      COMMON/SECPAR/PAR(1)
      COMMON/G9/NEXG(1)
      COMMON/G13/PI(1)
      COMMON/G14/PWB(1)
      COMMON/G15/HG(1)
      COMMON/G22/QVGC(1)
      COMMON/G23/QVWC(1)
      COMMON/G24/QVOC(1)
      COMMON/G25/GRAD(1)
      COMMON/NN/NEL,NCON,NOGN,NK,NEQ,NPH,NB,NK1,NEQ1,NBK,NSEC,NFLUX
      COMMON/ECH/EOSN(20)
      CHARACTER EOSN*10
C.... FOR TMVOC
      DIMENSION AMMIX(3)
C
      SAVE ICALL
      DATA ICALL/0/
      ICALL=ICALL+1
      IF(ICALL.EQ.1) WRITE(11,899)
C 899 FORMAT(6X,'GCOR     1.0      15 JANUARY   1998',6X,
C 899 FORMAT(6X,'GCOR     1.0      27 MARCH     1998',6X,
C 899 FORMAT(6X,'GCOR     1.0      27 OCTOBER   2000',6X,
  899 FORMAT(6X,'GCOR     1.0      14 NOVEMBER  2001',6X,
     X'PERFORM SIMPLE GRAVITY CORRECTION FOR FLOWING',
     X' BOTTOMHOLE PRESSURE')
C
      NLTAB1=N+LTABA-1
C
C
C-----LOOP OVER ALL LAYERS FOR THIS WELL.
      DO 50 ND=N,NLTAB1
C     ND IS THE GENERATION INDEX.
      JD=NEXG(ND)
      JDLOC=(JD-1)*NEQ
      JDLOCP=(JD-1)*NK1
      JDLOC2=(JD-1)*NSEC*NEQ1
CFiB --- JLM2 not set; set JLM2 to zero to reproduce results in manual
      JLM2=JDLOC2
CFiE
      PIND=PI(ND)
      IF(EOSN(1).EQ.'EWASG     ') THEN
C.....   RENORMALIZE PRODUCTIVITY INDEX FOR PERMEABILITY REDUCTION
C         PIND=PI(ND)*PAR(JDLOC2+2*NBK+6)
         PIND=PI(ND)*PAR(JDLOC2+2*NBK+3)
      ENDIF
C
CC      PRINT 120,ND,PI(ND), PAR(JDLOC2+2*NBK+6)
CC 120  FORMAT('ND, PI, PAR=', I5,2(1X,E10.4) )
C
C     COMPUTE PHASE MOBILITIES.
      AMG=0.
      IF(PAR(JDLOC2+3).NE.0.) AMG=PAR(JDLOC2+2)/PAR(JDLOC2+3)
      AMW=0.
      IF(NPH.GE.2.AND.PAR(JDLOC2+3+NBK).NE.0.) 
     &             AMW=PAR(JDLOC2+2+NBK)/PAR(JDLOC2+3+NBK)
      AMO=0.
      IF(NPH.GE.3.AND.PAR(JDLOC2+3+2*NBK).NE.0.) 
     &             AMO=PAR(JDLOC2+2+2*NBK)/PAR(JDLOC2+3+2*NBK)
C
CC      PRINT 110,ND,AMG,AMW,AMO,PIND
CC 110  FORMAT('ND, AMG, AMW, AMO, PIND=', I5,4(1X,E10.4) )
C
C
C*****COMPUTE VOLUMETRIC PRODUCTION RATES FOR INDIVIDUAL PHASES.
      QVG=PIND  *AMG*X(JDLOCP+1)
      QVW=PIND  *AMW*X(JDLOCP+1)
      QVO=PIND  *AMO*X(JDLOCP+1)
C*****ARRAYS FOR CUMULATIVE PHASE RATES.
      IF(ND.EQ.N) GOTO 52
      QVGC(ND)=QVGC(ND-1)+QVG
      QVWC(ND)=QVWC(ND-1)+QVW
      QVOC(ND)=QVOC(ND-1)+QVO
      GOTO 53
   52 QVGC(ND)=QVG
      QVWC(ND)=QVW
      QVOC(ND)=QVO
   53 CONTINUE
C*****ARRAY OF GRADIENTS.
C
       IF(EOSN(1).EQ.'TMVOC1    ') THEN
C+++++   SECTION FOR TMVOC
         CALL AMOLW(JLM2,AMMIX(1),AMMIX(2),AMMIX(3))
         AMMIX(1)=AMMIX(1)*1.E-3
         AMMIX(2)=AMMIX(2)*1.E-3
         AMMIX(3)=AMMIX(3)*1.E-3
       ELSE
         AMMIX(1)=1.
         AMMIX(2)=1.
         AMMIX(3)=1.
       END IF
C .... FOR THE GRADIENT USE PHASE DENSITIES IN KG/M3
      GRAD(ND)=( QVGC(ND)*PAR(JDLOC2+4)*AMMIX(1)
     &         +QVWC(ND)*PAR(JDLOC2+NBK+4)*AMMIX(2)
     &         +QVOC(ND)*PAR(JDLOC2+2*NBK+4)*AMMIX(3))*9.80665
     $         /( QVGC(ND)+QVWC(ND)+QVOC(ND) )
C
CC      PRINT 100,ND,QVGC(ND),QVWC(ND),QVOC(ND),GRAD(ND)
CC 100  FORMAT('ND, QG, QW, QO, GRAD=', I5,4(1X,E10.4) )
C
   50 CONTINUE
C
C-----NOW PERFORM LOOP GOING FROM TOP LAYER DOWN.
C
      N1=N+1
      DO51 ND=N1,NLTAB1
      NDD=NLTAB1+N-ND
C     NDD IS THE RUNNING SOURCE INDEX, GOING FROM THE LAYER JUST BELOW
C     THE TOP (INDEX N+LTABA-2) DOWN TO THE BOTTOM LAYER (INDEX N).
C
      PWB(NDD)=PWB(NDD+1)+(HG(NDD+1)*GRAD(NDD+1)+HG(NDD)*GRAD(NDD))/2.
   51 CONTINUE
C
      RETURN
      END
C
      SUBROUTINE MULTI
C
C-----SET UP THE COUPLED LINEAR EQUATIONS ARISING AT EACH ITERATION
C     STEP.
C
C***** N O T A T I O N ********************
C
C     NEQ IS THE NUMBER OF EQUATIONS, AND THE NUMBER OF PRIMARY
C         DEPENDENT VARIABLES (PER ELEMENT).
C
C     NPH IS THE NUMBER OF PHASES WHICH CAN BE PRESENT.
C
C     NK  IS THE NUMBER OF COMPONENTS (SPECIES).
C
C     NB  IS THE NUMBER OF PHASE-DEPENDENT SECONDARY VARIABLES OTHER
C         THAN COMPONENT MASS FRACTIONS.
C
C     NBK = NB+NK IS THE TOTAL NUMBER OF PHASE-DEPENDENT SECONDARY
C         VARIABLES.
C
C     NSEC = NPH*NBK+2 IS THE TOTAL NUMBER OF SECONDARY VARIABLES.
C
C******************************************
C
C$$$$$$$$$ COMMON BLOCKS FOR ELEMENTS $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
C
C     THESE BLOCKS HAVE A LENGTH OF NEL (= NUMBER OF ELEMENTS)
C
      COMMON/E1/ELEM(1)
      COMMON/E2/MATX(1)
      COMMON/E3/EVOL(1)
      COMMON/E4/PHI(1)
      COMMON/E5/P(1)
      COMMON/E6/T(1)
      COMMON/E7/PM(1)
C
C$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
C
C$$$$$$$$$ COMMON BLOCKS FOR PRIMARY VARIABLES $$$$$$$$$$$$$$$$$$$$$$$$$
C
C     THESE BLOCKS HAVE A LENGTH OF NEL*NK1
C
      COMMON/P1/X(1)
      COMMON/P2/DX(1)
      COMMON/P3/DELX(1)
C
C$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
C
C$$$$$$$$$ COMMON BLOCKS FOR RESIDUALS $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
C
C     THESE BLOCKS HAVE A LENGTH OF NEL*NEQ
C
      COMMON/P4/R(1)
      COMMON/P5/DOLD(1)
C
C$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
C
C
C$$$$$$$$$ COMMON BLOCKS FOR CONNECTIONS $$$$$$$$$$$$$$$$$$$$$$$$$$$C
C
      COMMON/C1/NEX1(1)
      COMMON/C2/NEX2(1)
      COMMON/C3/DEL1(1)
      COMMON/C4/DEL2(1)
      COMMON/C5/AREA(1)
      COMMON/C6/BETA(1)
      COMMON/C7/ISOX(1)
      COMMON/C8/GLO(1)
      COMMON/C9/ELEM1(1)
      COMMON/C10/ELEM2(1)
      COMMON/C11/FVD(1)
      COMMON/C12/SIG(1)
      COMMON/FMOLDIF/FDIF(1)
C
C$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
C
C$$$$$$$$$ COMMON BLOCK FOR SECONDARY VARIABLES $$$$$$$$$$$$$$$$$$$$
C
      COMMON/SECPAR/PAR(1)
C
C$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
C
C$$$$$$$$$ COMMON BLOCKS FOR FLOW RATES AND VELOCITIES $$$$$$$$$$$$$$$$$
C
      COMMON/COMPO/FLO(1)
      COMMON/PORVEL/VEL(1)
      COMMON/DARVELM/DVELM(1)
C
CRNT  BEGIN ADDITION FOR RNT
C
      COMMON/EOSEL/IE(16),FE(512)
      COMMON/VOLBC/VBC(1)
C
CRNT  END ADDITION FOR RNT
C$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
C
C$$$$$$$$$ COMMON BLOCKS FOR ROCK PROPERTIES $$$$$$$$$$$$$$$$$$$$$$$$$$
C
      COMMON/SOLID/NM,DM(27),POR(27),PER(3,27),CWET(27),SH(27)
      COMMON/SOCH/MAT(27)
      COMMON/SOLI/COM(27),EXPAN(27),CDRY(27),TORT(27),GK(27)
      COMMON/SOLII/XKD3(27),XKD4(27),SII3(27)
      COMMON/ADSORPTM/OCKM(20),FOXM,FOCM(27),ALAMM(20)
      COMMON /MOLWT/ AMWTM(20)
      COMMON/NUMBER/ NHC
      COMMON/BIODEG/TEG(20),TEG0(20)
C.....6-4-93: APPEND TWO COMMON BLOCKS USED IN THE T2VOC VERSION
C             OF TOUGH2.
      COMMON/FLOVP1/FLOVG(1)
      COMMON/FLOVP2/FLOVW(1) 
C
C$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
C
C$$$$$$$$$ COMMON BLOCKS FOR LINEAR EQUATIONS $$$$$$$$$$$$$$$$$$$$$$$$$
C
      COMMON/L1/IRN(1)
      COMMON/L2/ICN(1)
      COMMON/L3/CO(1)
      COMMON/L4/WKAREA(1)
      COMMON/L5/IKEEP(1)
      COMMON/L6/IW(1)
      COMMON/L7/JVECT(1)
      COMMON/AMMIS/MA,IPIV,U,IAB,NZ
C
C$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
C
      COMMON/CYC/KCYC,ITER,ITERC,TIMIN,SUMTIM,GF,TIMOUT
      COMMON/CONTST/RE1,RE2,RERM,NER,KER,DFAC
      COMMON/NN/NEL,NCON,NOGN,NK,NEQ,NPH,NB,NK1,NEQ1,NBK,NSEC,NFLUX
      COMMON/DM/DELTEN,DELTEX,FOR,FORD
      COMMON/KONIT/KON,DELT,IGOOD
      COMMON/DG/WUP,WNR
      COMMON/SVZ/NOITE,MOP(24)
      COMMON/PATCH/SING
      COMMON/BIND/DIFF0,TEXP,BE
      COMMON/GASLAW/RGAS,AMS,AMA,CVGAS
      COMMON/DFM/TIMAX,REDLT
      COMMON/BC/NELA
      COMMON/RANT2/IRANT2
      COMMON/DIPA/FDDIAG(3,20)
      COMMON/ECH/EOSN(20)
C.....FOR TMVOC
      COMMON/NCGN/NCG,NCGCODE(10)
      common/ncgl/alncg(10)
C
      CHARACTER EOSN*10
      CHARACTER*5 ELEM,ELEM1,ELEM2,MAT
C
C######### LOCAL ARRAYS ###############################################
C
      DIMENSION D(21,22),F(21,37)
CRNT  BEGIN ADDITION FOR RNT.
C
      DIMENSION XLAM(6),XKD(5,27)
      SAVE XLAM,XKD,XMW3,XMW4,XMW43,DEG3ZERO,DEG4ZERO
C
CRNT  END ADDITION FOR RNT.
C
C######################################################################
C
      SAVE ICALL,M11,STEB
C     STEB IS THE STEFAN-BOLTZMANN CONSTANT FOR RADIATIVE HEAT
C     EMISSION, IN UNITS OF J/(M^2 K^4 S)
      DATA ICALL,STEB/0,5.6687E-8/
      ICALL=ICALL+1
C
C===== ICALL = 1 SECTION
      IF(ICALL.EQ.1) THEN
         WRITE(11,899)
C 899 FORMAT(/6X,'MULTI    1.0       9 MAY       1991',6X,
C 899 FORMAT(/6X,'MULTI    1.01      7 MAY       1993',6X,
C 899 FORMAT(/6X,'MULTI    1.01 S   24 JUNE      1994',6X,
C 899 FORMAT(/6X,'MULTI    1.02 S    7 DECEMBER  1994',6X,
C 899 FORMAT(/6X,'MULTI    1.02 S   16 FEBRUARY  1996',6X,
C 899 FORMAT(/6X,'MULTI    1.1      26 SEPTEMBER 1997',6X,
C 899 FORMAT(/6X,'MULTI    1.1      23 MARCH     1998',6X,
C 899 FORMAT(/6X,'MULTI    1.1      27 MARCH     1998',6X,
C 899 FORMAT(/6X,'MULTI    1.1      30 SEPTEMBER 1998',6X,
C 899 FORMAT(/6X,'MULTI    1.1       8 DECEMBER  1998',6X,
C 899 FORMAT(/6X,'MULTI    1.1      11 JANUARY   1999',6X,
C 899 FORMAT(/6X,'MULTI    1.1      21 FEBRUARY  1999',6X,
C 899 FORMAT(/6X,'MULTI    1.1       1 MARCH     1999',6X,
C 899 FORMAT(/6X,'MULTI    1.1      30 MARCH     1999',6X,
C 899 FORMAT(/6X,'MULTI    1.1       8 JUNE      1999',6X,
C 899 FORMAT(/6X,'MULTI    1.1      16 JUNE      1999',6X,
C 899 FORMAT(/6X,'MULTI    1.1       8 OCTOBER   1999',6X,
C 899 FORMAT(/6X,'MULTI    1.1      15 NOVEMBER  2000',6X,
C 899 FORMAT(/6X,'MULTI    1.1       5 JANUARY   2001',6X,
C 899 FORMAT(/6X,'MULTI    1.1      10 JANUARY   2001',6X,
C 899 FORMAT(/6X,'MULTI    1.1      10 APRIL     2001',6X,
C 899 FORMAT(/6X,'MULTI    1.1      18 APRIL     2001',6X,
C 899 FORMAT(/6X,'MULTI    1.1      29 JUNE      2001',6X,
C 899 FORMAT(/6X,'MULTI    1.1      10 JULY      2001',6X,
C 899 FORMAT(/6X,'MULTI    2.0      23 AUGUST    2001',6X,
c 899 FORMAT(/6X,'MULTI    2.0      30 AUGUST    2001',6X,
c 899 FORMAT(/6X,'MULTI    2.0       3 January   2006',6X,
c 899 FORMAT(/6X,'MULTI    2.0      30 May       2007',6X,
c 899 FORMAT(/6X,'MULTI    2.0      17 August    2010',6X,
c 899 FORMAT(/6X,'MULTI    2.0       3 March     2011',6X,
  899 FORMAT(/6X,'MULTI    2.1       7 MARCH     2011',6X,
     X'ASSEMBLE ALL ACCUMULATION AND FLOW TERMS'/
     X47X,'INCLUDES CAPABILITIES FOR RADIATIVE HEAT TRANSFER',
     X' AND DIFFUSION IN ALL PHASES'/
     X47X,'WITH LOCAL EQUILIBRIUM PHASE PARTITIONING BETWEEN GAS',
     X' AND LIQUID'/
     X47X,'ALLOWS BLOCK-BY-BLOCK PERMEABILITY MODIFICATION'/
     X47X,'PERMEABILITY CHANGES FROM PRECIP. (EWASG) THROUGH',
     X' PAR(NLOC2S+3)'/
     X47X,'TMVOC-COMPATIBLE VERSION; NO DYNAMIC D=0 SWITCHING'/
     x47X,'NCGs CAN BIODEGRADE WITH A HALF-LIFE MODEL')
c---*----1----*----2----*----3----*----4----*----5----*----6----*----7----*----8
C
         M11=MOD(MOP(11),2)
C
         IF(EOSN(1).EQ.'EOS7R     ') THEN
C+++++ SECTION FOR EOS7R
           IF(FE(33).EQ.0.0.OR.FE(41).EQ.0.0) THEN
              PRINT 8891
 8891 FORMAT(/'EEEEEEE  INPUT ERROR.  RADIONUCLIDE TRANSPORT',
     &' REQUIRES NON-ZERO ',
     &'VALUES FOR THE HALF-LIVES (FE(33) AND FE(41)).   EEEEEE'/)
              STOP
           ENDIF
           XLAM(1)=0.
           XLAM(2)=0.
           XLAM(3)=0.693147/FE(33)
           XLAM(4)=0.693147/FE(41)
           XLAM(5)=0.
           XLAM(6)=0.
           DO N=1,NM
               XKD(1,N)=0.
               XKD(2,N)=0.
               XKD(3,N)=XKD3(N)
               XKD(4,N)=XKD4(N)
               XKD(5,N)=0.
           END DO
           XMW3=FE(34)
           XMW4=FE(42)
           XMW43=XMW4/XMW3
C
C     IF GRIDBLOCK VOLUME IS GREATER THAN 1.E50 M^3, ASSUME THE
C     RADIONUCLIDE COMPONENT DOES NOT DECAY WITH TIME IN THE
C     GRIDBLOCK.
C
         ENDIF
C+++++ END OF EOS7R SECTION
C
           DO N=1,NELA
               VBC(N)=1.
               IF(EVOL(N).GE.1.E50) THEN
                  VBC(N)=0.
               ENDIF
           END DO
c
c.....re-order array ALAMM; VOCs in slots NCG+2 through NCG+1+NHC
      if(EOSN(1).EQ.'TMVOC1    ') THEN
            do k=1,nhc
               k1=nhc+1-k
               k2=nhc+2+ncg-k
               alamm(k2)=alamm(k1)
            enddo
c.....no decay for water
               alamm(1)=0.
c.....store NCG decay constants in slots 2 through NCG+1
            ncg1=ncg+1
            do k=2,ncg1
               alamm(k)=alncg(k-1)
            enddo
      print 111,(alamm(k),k=1,nk)
  111 format(/' decay constants for components'/(10(1X,E12.6)))
C
      endif
      ENDIF
C===== END OF ICALL = 1 SECTION
C
C
C$$$$$FOR IAB=0 COLUMN INDICES WILL BE STORED IN ICN, OTHERWISE IN JVECT
C-----INITIALIZE COUNTER FOR MATRIX ELEMENTS.
      NZ=0
C
C***** LOOP OVER ELEMENTS ******************************
C
C-----COMPUTE ALL QUANTITIES WHICH DEPEND ONLY UPON VARIABLES PERTAINING
C     TO ONE VOLUME ELEMENT.
C
      IF(MOP(3).GE.2) PRINT 201,KCYC,ITER
  201 FORMAT(/39H $$$$$$$$$$ SUBROUTINE MULTI $$$$$$$$$$,24H   ---   [KC
     AYC,ITER] = [,I4,1H,,I3,1H]/)
C
      IF(MOP(3).GE.3) PRINT 204
  204 FORMAT(' ===== ACCUMULATION TERMS =====   MASS BALANCES FIRST,',
     X' ENERGY BALANCE LAST'/)
      DO 100 N=1,NELA
C     IDENTIFY MATRIX BLOCK.
      NLOC=(N-1)*NEQ
      NLOCP=(N-1)*NK1
C     IDENTIFY THE START OF SECONDARY VARIABLES FOR ELEMENT N.
      NLOC2=(N-1)*NSEC*NEQ1
C
C-----ZERO OUT RESIDUALS FOR ELEMENT N.
      DO 20 K=1,NEQ
   20 R(NLOC+K)=0.
C
C
C     ASSIGN QUANTITIES WHICH DEPEND ONLY UPON ELEMENT INDEX N.
      PHIN=PHI(N)
      NMAT=MATX(N)
      CD=SH(NMAT)*DM(NMAT)*(1.-POR(NMAT))
C
C
C     LOOP THROUGH INCREMENTS FOR DERIVATIVES.
      DO 101 M=1,NEQ1
C     IDENTIFY BEGINNING OF SECONDARY VARIABLES CORRESPONDING
C     TO INCREMENTING PRIMARY VARIABLE (M-1).
      NLM2=NLOC2+(M-1)*NSEC
C
C-----ZERO OUT BLOCK (N,N).
      DO 1011 K=1,NK1
 1011 D(K,M)=0.
C
      IF(EOSN(1).EQ.'TMVOC1    ') THEN
         DO L=1,NK
            TEG(L)=0.
         ENDDO
      ENDIF
C
C-----COMPUTE CHANGE IN POROSITY.
      DPRES=0.
      IF(M.EQ.2) DPRES=DELX(NLOCP+1)
      PRES=X(NLOCP+1)+DX(NLOCP+1)+DPRES
      DPHI=PHIN*(COM(NMAT)*(PRES-P(N))+EXPAN(NMAT)*(PAR(NLM2+NSEC-1)
     A-T(N)))
      PHINN=PHIN+DPHI
C
      DO 102 NP=1,NPH
C     IDENTIFY BEGINNING OF SECONDARY VARIABLES FOR PHASE NP.
      NL2NP=NLM2+(NP-1)*NBK
C     SATURATION.
      SNP=PAR(NL2NP+1)
      IF(SNP.EQ.0.) GOTO 102
C     DENSITY.
      RHONP=PAR(NL2NP+4)
      PHISRO=PHINN*SNP*RHONP
C
C-----SUM OVER COMPONENTS IN EACH PHASE.
      DO 103 K=1,NK
C     MASS FRACTION OF COMPONENT K IN PHASE NP.
      XNPKM=PAR(NL2NP+NB+K)
      D(K,M)=D(K,M)+XNPKM*PHISRO
      IF(EOSN(1).EQ.'EOS7R     ') THEN
C
C     INCLUDE ADSORPTION TERM FOR LIQUID PHASE.
C
      IF(NP.EQ.2) THEN
          D(K,M)=D(K,M)+(1.0-PHINN)*DM(NMAT)*
     &           RHONP*XNPKM*XKD(K,NMAT)
      ENDIF
C
C     CALCULATE CURRENT DECAY/DEGRADATION TERM.
C     VBC CONSTANT IS ZERO FOR GRIDBLOCKS WITH VOLUME GREATER
C     THAN 1.E50 M^3 MAKING DECAY NOT OCCUR IN THAT GRIDBLOCK.
C
          DEG=ABS(XLAM(K))*DELT*D(K,M)*VBC(N)
          IF(K.EQ.3) DEG3=DEG
          IF(K.EQ.4) DEG4=DEG-DEG3*XMW43
C
      ENDIF
C
      IF(EOSN(1).EQ.'TMVOC1    ') THEN
C---- ADD TERM FOR ADSORPTION OF VOCS ONTO SOIL
C     ADSORPTION OF VOC'S IS DESCRIBED ACCORDING TO A LINEAR
C     ADSORPTION ISOTERM FOLLOWING THE T2VOC FORMULATION.
C     CS = KOC * FOC * CW , WHERE KOC IS IN M3(AQ PHASE)/KG(ROCK).
C     FURTHER ADSORPTION ISOTERMS WILL BE ADDED LATER ON: A NEW 
C     DEDICATED SUBROUTINE COULD BE USED, WITH DIFFERENT ISOTHERMS
C     CHOOSEN FOR DIFFERENT VOC'S  !!
C
                IF (NP.EQ.2.AND.K.GT. 1) THEN
C
C.....4-10-01:    SIMPLE HALF-LIFE DEGRADATION MODEL
                  TEG(K)=ALAMM(K)*FORD*XNPKM*PHISRO
      if(k.gt.1+ncg) D(K,M) = D(K,M) + (1.E0-PHINN)*DM(NMAT)*
     &                     OCKM(K-1-NCG)*FOCM(NMAT)*RHONP*XNPKM
                ENDIF
      ENDIF
C
  103 CONTINUE
C
c     if(iter.eq.1) print 112,(teg(k),k=1,nk)
  112 format(/' decay terms TEG'/(10(1X,E12.6)))
C
C     INTERNAL ENERGY IN PHASE NP.
      ENNP=PHISRO*PAR(NL2NP+5)-PHINN*SNP*PRES
      D(NK1,M)=D(NK1,M)+ENNP
C
  102 CONTINUE
C
C     ADD ROCK ENERGY.
      ENR=CD*PAR(NLM2+NSEC-1)
      D(NK1,M)=D(NK1,M)+ENR
C
C     ASSIGN ACCUMULATION TERM AT BEGINNING OF TIME STEP.
C
      IF(ITER.EQ.1.AND.M.EQ.1) THEN
          DO 105 K=1,NEQ
             DOLD(NLOC+K)=D(K,1)
 105      CONTINUE
C     DOLD(NLOC+K) HOLDS THE QUANTITY OF COMPONENT K IN ELEMENT N
C     AT THE END OF THE LAST COMPLETED TIME STEP.
C
      IF(EOSN(1).EQ.'EOS7R     ') THEN
CRNT  BEGIN ADDITION FOR RNT.
C
C     STORE DECAY/DEGRADATION AT BEGINNING OF TIME STEP.
C
           DEG3ZERO=DEG3
           DEG4ZERO=DEG4
C
      ENDIF
      IF(EOSN(1).EQ.'TMVOC1    ') THEN
         DO I=1,NK
            TEG0(I)=TEG(I)
         ENDDO
      ENDIF
      ENDIF
C
      IF(EOSN(1).EQ.'EOS7R     ') THEN
      IF(M.EQ.1.AND.MOP(3).GE.3)
     & PRINT 205,ELEM(N),(D(KK,1),KK=1,NEQ),DEG
  205 FORMAT('       AT ELEMENT *',A5,'*   ',7(1X,E20.14))
C
C     ADD DECAY/DEGRADATION TERM TO ACCOUNT FOR RADIOACTIVE DECAY.
C     IF XLAM(3) OR XLAM(4) IS INPUT AS A NEGATIVE NUMBER, USE THE
C     MIDPOINT AVERAGE DECAY SCHEME.  IF POSITIVE, USE THE DEFAULT
C     IMPLICIT SCHEME FOR RADIOACTIVE DECAY.
C
      IF(XLAM(3).LT.0.OR.XLAM(4).LT.0) THEN
          D(3,M)=D(3,M) + 0.5*(DEG3 + DEG3ZERO)
          D(4,M)=D(4,M) + 0.5*(DEG4 + DEG4ZERO)
      ELSE
          D(3,M)=D(3,M) + DEG3
          D(4,M)=D(4,M) + DEG4
      ENDIF
C
      ENDIF
CRNT  END ADDITION FOR RNT.
C
      IF(EOSN(1).EQ.'TMVOC1    ') THEN
         DO I=1,NK
            D(I,M)=D(I,M)+0.5*(TEG(I)+TEG0(I))
         ENDDO
      ENDIF
  101 CONTINUE
C
      IF(MOP(3).GE.3.AND.EOSN(1).NE.'EOS7R     ') 
     XPRINT 200,ELEM(N),(D(K,1),K=1,NEQ)
  200 FORMAT('       AT ELEMENT *',A5,'*   ',8(1X,E12.6))
C
C
C+++++ASSIGN ALL ONE-ELEMENT TERMS++++++++++++++++++++++++++++++
C
      DO 106 K=1,NEQ
C     K IS THE ROW INDEX IN MATRIX BLOCK (N,N).
C-----INITIALIZE RESIDUALS.
      R(NLOC+K)=R(NLOC+K)+D(K,1)-DOLD(NLOC+K)
C
      DO 106 L=1,NEQ
C     L IS THE COLUMN INDEX IN MATRIX BLOCK (N,N).
C-----COMPUTE MATRIX ELEMENT ARISING FROM DEPENDENCE OF COMPONENT K
C     UPON PRIMARY VARIABLE L.
      IRN(NZ+1)=NLOC+K
      IF(IAB.EQ.0) ICN(NZ+1)=NLOC+L
      IF(IAB.NE.0) JVECT(NZ+1)=NLOC+L
      CO(NZ+1)=-(D(K,L+1)-D(K,1))/DELX(NLOCP+L)
      IF(PHINN.EQ.0..AND.K.EQ.L.AND.K.NE.NK1) CO(NZ+1)=1.
  106 NZ=NZ+1
C
C+++++END OF ASSIGNMENT OF ONE-ELEMENT TERMS++++++++++++++++++++
C
C
  100 CONTINUE
C
C***** SINKS AND SOURCES ***********************************************
C
      IF(NOGN.NE.0) CALL QU
      IF(IGOOD.EQ.3) RETURN
      IF(MOP(15).GE.1) CALL QLOSS
C
C+++++ END OF SINKS/SOURCES ++++++++++++++++++++++++++++++++++++++++++++
C
C
C.....FOR MODIFIED PERMEABILITY, SCALE PCAP
      IF(IRANT2.NE.0) THEN
         IF(ITER.EQ.1) CALL EOS
         N1=NEL
         IF(ICALL.GT.1) N1=NELA 
         DO107 N=1,N1
         NLOC2=(N-1)*NSEC*NEQ1
         DO107 M=1,NEQ1
         NLM2=NLOC2+(M-1)*NSEC
         DO107 NP=1,NPH
         NLM2P=NLM2+(NP-1)*NBK
  107    IF(PM(N).NE.0.) PAR(NLM2P+6)=PAR(NLM2P+6)/SQRT(PM(N))
      ELSE
      ENDIF
C
C***** LOOP OVER CONNECTIONS ***********************************
C
C
C-----COMPUTE ALL QUANTITIES WHICH DEPEND UPON VARIABLES FOR TWO VOLUME
C     ELEMENTS ("INTERFACE QUANTITIES").
C
C     AT EACH INTERFACE HAVE ONE FLUX FOR EACH COMPONENT, WHICH IS A SUM
C     OF CONTRIBUTIONS FROM ALL PHASES. FOR PURPOSES OF COMPUTING
C     DERIVATIVES, NEED ALSO THE FLUXES OBTAINED BY INCREMENTING ANY
C     OF THE NEQ VARIABLES FOR THE "FIRST" ELEMENT, AND ANY OF THE
C     NEQ VARIABLES FOR THE "SECOND" ELEMENT.
C
C-----INITIALIZE TOTAL NUMBER OF FLUX TERMS.
C
C
      DO1 N=1,NCON
C
      N1=NEX1(N)
      N2=NEX2(N)
      IF(N1.EQ.0.OR.N2.EQ.0) GOTO 1
      IF(N1.GT.NELA.AND.N2.GT.NELA) GOTO 1
CFIB
         FLOVG(N)=0. 
         FLOVW(N)=0. 
CFIE
C-----IDENTIFY LOCATION OF MATRIX BLOCKS.
      N1LOC=(N1-1)*NEQ
      N2LOC=(N2-1)*NEQ
      N1LOCP=(N1-1)*NK1
      N2LOCP=(N2-1)*NK1
C
C-----IDENTIFY START OF SECONDARY VARIABLES FOR ELEMENTS N1 AND N2.
      N1LOC2=(N1-1)*NSEC*NEQ1
      N2LOC2=(N2-1)*NSEC*NEQ1
C
C-----OBTAIN SOME QUANTITIES PERTAINING TO CONNECTION.
C
      D1=DEL1(N)
      D2=DEL2(N)
C     SPATIAL INTERPOLATION FACTORS.
      WT1=D2/(D1+D2)
      WT2=1.-WT1
C.....3-23-98: ASSIGN WM1, WM2 TO "SOMETHING" IN CASE ADVECTIVE
C              SECTION IS SKIPPED OVER (NEEDED IN DO 4 LOOP)
      WM1=0.5
      WM2=0.5
C
      NMAT1=MATX(N1)
      NMAT2=MATX(N2)
      PO1=POR(NMAT1)
      PO2=POR(NMAT2)
      PHI1=PHI(N1)
      PHI2=PHI(N2)
      ISO=ISOX(N)
      GX=BETA(N)*GF
      AX=AREA(N)
      PER1=PER(ISO,NMAT1)
      PER2=PER(ISO,NMAT2)
C.....
      IF(IRANT2.NE.0) THEN
      PER1=PER1*PM(N1)
      PER2=PER2*PM(N2)
      ELSE
C.....?????
      ENDIF
      DPERI=WT1*PER1+WT2*PER2
      PERI=0.
      IF(DPERI.NE.0.) PERI=PER1*PER2/DPERI
      IF(D1.EQ.0.) PERI=PER2
      IF(D2.EQ.0.) PERI=PER1
C
C-----ASSIGN FACTORS FOR FLUX TERMS.
      IF(N1.LE.NELA) FAC1=FORD/EVOL(N1)
      IF(N2.LE.NELA) FAC2=FORD/EVOL(N2)
C
      DPX0=X(N2LOCP+1)+DX(N2LOCP+1)-X(N1LOCP+1)-DX(N1LOCP+1)
      DPX0=DPX0/(D1+D2)
C
      IF(MOP(3).GE.4) PRINT 199,N,ELEM(N1),ELEM(N2),FAC1,FAC2
  199 FORMAT(/' ***** CONNECTION #',I5,'   ELEMENTS (',A5,',',A5,')',
     X'  ***  (DELT/V1) = ',E12.6,'   (DELT/V2) = ',E12.6)
C
C-----NOW GET ALL FLUX TERMS.
C     M=1 IS THE FLUX FOR LATEST UPDATED VARIABLES.
C     M=2, ... ,NEQ+1 ARE THE NEQ FLUXES CORRESPONDING TO INCREMENTING
C     THE VARIABLES FOR ELEMENT N1.
C
C     M=NEQ+2, ... ,2*NEQ+1 ARE THE NEQ FLUXES CORRESPONDING TO
C     INCREMENTING THAT VARIABLES FOR ELEMENT N2.
C
      DO 2 M=1,NFLUX
C
      DPRES1=0.
      DPRES2=0.
      IF(M.EQ.2) DPRES1=DELX(N1LOCP+1)
      IF(M.EQ.NEQ+2) DPRES2=DELX(N2LOCP+1)
      PRES1=X(N1LOCP+1)+DX(N1LOCP+1)+DPRES1
      PRES2=X(N2LOCP+1)+DX(N2LOCP+1)+DPRES2
      DPX=(PRES2-PRES1)/(D1+D2)
C
C-----IDENTIFY LOCATION OF SECONDARY VARIABLES.
C
C     DEFINE AUXILIARY QUANTITY M2 TO BE 0 FOR M=1, TO BE 1 FOR
C     SUBSEQUENT GROUP OF NEQ FLUXES INCREMENTED AT N1, TO BE 2 FOR
C     SUBSEQUENT GROUP OF NEQ FLUXES INCREMENTED AT N2.
C
      M2=2
      IF(M.EQ.1) M2=0
      IF(2.LE.M.AND.M.LE.NEQ1) M2=1
      N1LM2=N1LOC2+MOD(M2,2)*(M-1)*NSEC
      N2LM2=N2LOC2+(M2/2)*(M-NEQ1)*NSEC
C
C
      DO 21 K=1,NK1
   21 F(K,M)=0.
C
C.....6-8-95: CONDUCTIVE HEAT FLUX ONLY WHEN RADIATIVE HEAT TRANSFER
C             COEFFICIENT IS NON-NEGATIVE.
      IF(SIG(N).GE.0.) THEN
C
C-----HEAT CONDUCTIVITY.
C
C.....STANDARD TOUGH2 USED SECOND (= AQUEOUS) PHASE;
C     REPLACE THAT WITH TOTAL LIQUID
C         S1=PAR(N1LM2+NBK+1)
C         S2=PAR(N2LM2+NBK+1)
      IF(EOSN(1)(1:5).EQ.'ECO2M') THEN
         S1 = 1.E0 - PAR(N1LM2+2*NBK+1)
         S2 = 1.E0 - PAR(N2LM2+2*NBK+1)
      ELSE
         S1 = 1.E0 - PAR(N1LM2+1)
         S2 = 1.E0 - PAR(N2LM2+1)
      ENDIF
C**  THE THERMAL CONDUCTIVITY OF A GRID BLOCK AT A GIVEN TOTAL
C**  LIQUID SATURATION IS OBTAINED BY INTERPOLATING BETWEEN ITS
C**  DRY AND WET THERMAL CONDUCTIVITIES. NOTE THAT IN THE SCHEME
C**  USED HERE, NO DISTINCTION IS MADE BETWEEN NAPL AND WATER
C**  REGARDING THEIR RESPECTIVE THERMAL CONDUCTIVITIES.
C
      IF(MOP(10).NE.0) GOTO 22
      S1X=MAX(S1,0.)
      S2X=MAX(S2,0.)
          CON1=CDRY(NMAT1)+SQRT(S1X)*(CWET(NMAT1)-CDRY(NMAT1))
          CON2=CDRY(NMAT2)+SQRT(S2X)*(CWET(NMAT2)-CDRY(NMAT2))
      GOTO23
C
   22 CONTINUE
          CON1=CDRY(NMAT1)+S1*(CWET(NMAT1)-CDRY(NMAT1))
          CON2=CDRY(NMAT2)+S2*(CWET(NMAT2)-CDRY(NMAT2))
   23 CONTINUE
      DCONI=WT1*CON1+WT2*CON2
      CONI=0.
      IF(DCONI.NE.0.) CONI=CON1*CON2/DCONI
      IF(D1.EQ.0.) CONI=CON2
      IF(D2.EQ.0.) CONI=CON1
C
C-----TEMPERATURE GRADIENT.
      T1=PAR(N1LM2+NSEC-1)
      T2=PAR(N2LM2+NSEC-1)
      DTX=(T2-T1)/(D1+D2)
C
C-----CONDUCTIVE HEAT FLOW.
      F(NK1,M)=CONI*DTX*AX
      ENDIF
C.....6-8-95: ADD RADIATIVE HEAT FLUX
      IF(SIG(N).NE.0.) THEN
      T1K=PAR(N1LM2+NSEC-1)+273.15
      T2K=PAR(N2LM2+NSEC-1)+273.15
      F(NK1,M)=F(NK1,M)+ABS(SIG(N))*STEB*AX
     X*(T2K*T2K*T2K*T2K-T1K*T1K*T1K*T1K)
      ENDIF
C
      IF((MOP(3).GE.6.AND.M.EQ.1).OR.MOP(3).GE.8)
     XPRINT 198,M,N1LM2,N2LM2,PRES1,PRES2,DPX
  198 FORMAT(/'   *** FLUX NO.',I3,'  SECONDARY INDICES (',I5,',',I5,
     X')',15X,'      P1 = ',E12.6,'  P2 = ',E12.6,'    DPX = ',E12.6)
      IF((MOP(3).GE.6.AND.M.EQ.1).OR.MOP(3).GE.8)
     XPRINT 197,CONI,PAR(N1LM2+NSEC-1),
     APAR(N2LM2+NSEC-1),DTX,F(NK1,M)
  197 FORMAT(21X,'       CONI = ',E12.6,'    T1 = ',E12.6,
     A'    T2 = ',E12.6,' DTX = ',E12.6,'   FNK1 = ',E12.6)
C
C-----OBTAIN FLUX FOR EACH PHASE.
      DO 3 NP=1,NPH
C
      PER1=PER(ISO,NMAT1)
      PER2=PER(ISO,NMAT2)
C.....
      IF(IRANT2.NE.0) THEN
      PER1=PER1*PM(N1)
      PER2=PER2*PM(N2)
      ELSE
C.....???????
      ENDIF
      IF((EOSN(1)(1:5).NE.'ECO2M'.AND.NP.NE.1).OR.
     &   (EOSN(1)(1:5).EQ.'ECO2M'.AND.NP.NE.3)) GOTO 30
C-----COME HERE TO ASSIGN KLINKENBERG FACTORS
      FK1=GK(NMAT1)/PRES1
      FK2=GK(NMAT2)/PRES2
      PER1=PER1*(1.+FK1)
      PER2=PER2*(1.+FK2)
   30 CONTINUE
C---*----1----*----2----*----3----*----4----*----5----*----6----*----7----*----8
      IF(EOSN(1).EQ.'EWASG     '.or.eosn(1).eq.'ECO2N     '.or.
     &                              eosn(1).eq.'ECO2M     ') THEN
C........APPLY PERMEABILITY REDUCTION DIRECTLY TO ABSOLUTE PERMEABILITY
C.....11-15-00: CHANGE FROM PAR(...+6) TO PAR(...+3)
         PER1=PER1*PAR(N1LM2+(nph-1)*NBK+3)
         PER2=PER2*PAR(N2LM2+(nph-1)*NBK+3)
      ENDIF
C
      DPERI=WT1*PER1+WT2*PER2
      PERI=0.
      IF(DPERI.NE.0.) PERI=PER1*PER2/DPERI
C
C-----IDENTIFY BEGINNING OF SECONDARY VARIABLES FOR PHASE NP.
C
      N1L2NP=N1LM2+(NP-1)*NBK
      N2L2NP=N2LM2+(NP-1)*NBK
C
      IF(M.EQ.1) THEN
         VEL((N-1)*NPH+NP)=0.
         DVELM((M-1)*NCON*NPH+(N-1)*NPH+NP)=0.
      ENDIF
C
      REL1=PAR(N1L2NP+2)
      REL2=PAR(N2L2NP+2)
C
      FNPM=0.
C
      S1=PAR(N1L2NP+1)
      VIS1=PAR(N1L2NP+3)
      RHO1=PAR(N1L2NP+4)
      RHO10=PAR(N1LOC2+(NP-1)*NBK+4)
      H1=PAR(N1L2NP+5)
      PCAP1=PAR(N1L2NP+6)
      PCAP10=PAR(N1LOC2+(NP-1)*NBK+6)
C
      S2=PAR(N2L2NP+1)
      VIS2=PAR(N2L2NP+3)
      RHO2=PAR(N2L2NP+4)
      RHO20=PAR(N2LOC2+(NP-1)*NBK+4)
      H2=PAR(N2L2NP+5)
      PCAP2=PAR(N2L2NP+6)
      PCAP20=PAR(N2LOC2+(NP-1)*NBK+6)
C
C.....2-16-96: THE NEXT TWO STATEMENTS ('IF ... GOTO 31') WERE
C              MOVED FROM IN FRONT OF THE PAR-ASSIGNMENTS TO
C              BEHIND THEM; THIS WAY THERMOPHYSICAL PROPERTIES
C              ARE STILL ASSIGNED EVEN WHEN ADVECTIVE FLUXES ARE
C              SKIPPED OVER, BECAUSE DIFFUSIVE FLUXES MAY BE
C              PRESENT.
C-----CHECK WHETHER PHASE NP IS MOBILE.
      IF(REL1.EQ.0..AND.REL2.EQ.0.) GOTO31
C.....5-7-93: NO ADVECTIVE FLUX WHEN EITHER PERMEABILITY IS ZERO.
      IF(PER1.EQ.0..OR.PER2.EQ.0.) GOTO31
C
C-----OBTAIN WEIGHTED INTERFACE DENSITY.
      W1=0.5
      IF(RHO1.EQ.0.) W1=0.
      IF(RHO2.EQ.0.) W1=1.
      W2=1.-W1
      RHOX=W1*RHO1+W2*RHO2
      RHOX0=W1*RHO10+W2*RHO20
C
C.....ASSIGN SEPARATE VALUES FOR USE IN THE GRAVITY TERM
      RHOXM=RHOX
      RHOXM0=RHOX0
C
      IF(EOSN(1).EQ.'TMVOC1    ') THEN
C
C.... THE AQUEOUS PHASE DENSITY WAS TAKEN EQUAL TO WATER DENSITY IN M2NOTS;
C      NOW THE AQ PHASE DENSITY TAKES INTO ACCOUNT THE EFFECT OF DISSOLVED 
C      VOC'S: THUS, AQ PHASE MASS DENSITY MUST BE COMPUTED ACCORDINGLY BY
C      SUBROUTINE WMASSD, KEEPING IN MIND THAT THE NCG CONCENTRATION EFFECTS
C      ARE NEGLECTED. 
          CALL WMASSD(NP,N1L2NP,AMWM1)
          CALL WMASSD(NP,N2L2NP,AMWM2)
          RHO1M =RHO1 *AMWM1/1.E3
          RHO1M0=RHO10*AMWM1/1.E3
          RHO2M =RHO2 *AMWM2/1.E3
          RHO2M0=RHO20*AMWM2/1.E3
                  RHOXM = W1 * RHO1M + W2 * RHO2M
                  RHOXM0= W1 * RHO1M0+ W2 * RHO2M0
      ENDIF
C
C-----EFFECTIVE PRESSURE GRADIENT.
      DR=DPX+(PCAP2-PCAP1)/(D1+D2)-RHOXM*GX
      DR0=DPX0+(PCAP20-PCAP10)/(D1+D2)-RHOXM0*GX
C
C
C-----PERFORM APPROPRIATE UPSTREAM WEIGHTING FOR MOBILITIES.
C
      IF(DR0.GT.0..AND.S2.EQ.0.) GOTO 31
      IF(DR0.LT.0..AND.S1.EQ.0.) GOTO 31
      IF(M11.GE.1) WM1=W1
      IF(M11.EQ.0.AND.DR0.GT.0.) WM1=1.-WUP
      IF(M11.EQ.0.AND.DR0.LE.0.) WM1=WUP
C
      IF(RHO1.EQ.0.) WM1=0.
      IF(RHO2.EQ.0.) WM1=1.
C
      WM2=1.-WM1
C
C-----IF A NODAL POINT FALLS RIGHT ON THE INTERFACE (I.E., NODAL
C     DISTANCE = 0), USE RELATIVE PERMEABILITY OF THE OTHER BLOCK.
C.....2-21-99: NEXT TWO LINES COMMENTED OUT; CODING REVISED
C     IF(D1.EQ.0..AND.REL1.EQ.0.) REL1=REL2
C     IF(D2.EQ.0..AND.REL2.EQ.0.) REL2=REL1
         REL10=PAR(N1LOC2+(NP-1)*NBK+2)
         REL20=PAR(N2LOC2+(NP-1)*NBK+2)
c.....1-3-06: remove rel.perm.dependent interface switch for D=0
c     IF(D1.EQ.0..AND.REL20.NE.0.) REL1=REL2
c     IF(D2.EQ.0..AND.REL10.NE.0.) REL2=REL1
C......
C
C-----USE UPSTREAM WEIGHTING FOR ABSOLUTE PERMEABILITY FOR
C     MOP(11) = 0 OR 1; KEEP HARMONIC WEIGHTING FOR MOP(11) .GT. 1.
      IF(MOP(11).LE.1.AND.DR0.GT.0.) PERI=PER2
      IF(MOP(11).LE.1.AND.DR0.LE.0.) PERI=PER1
C     EXCEPTION: WHEN A NODAL DISTANCE IS ZERO, USE ABSOLUTE
C     PERMEABILITY OF THE OTHER BLOCK.
      IF(D1.EQ.0.) PERI=PER2
      IF(D2.EQ.0.) PERI=PER1
C
C-----INTERFACE MOBILITY.
      IF(MOP(11).EQ.4) THEN
      XM1=PER1*REL1/VIS1
      XM2=PER2*REL2/VIS2
      DEN=WT1*XM1+WT2*XM2
      DMOBI=0.
      IF(DEN.NE.0.) DMOBI=XM1*XM2/DEN
C
      ELSE
      DMOBI=(WM1*REL1/VIS1+WM2*REL2/VIS2)*PERI
C.....2-21-99: COMMENTED OUT
C.....2-20-99
C     IF(D1.EQ.0.) DMOBI=WM2*REL2/VIS2*PERI
C     IF(D2.EQ.0.) DMOBI=WM1*REL1/VIS1*PERI
      ENDIF
C
C-----COME HERE TO COMPUTE PORE VELOCITIES FOR GASEOUS AND LIQUID
C     PHASES, RESPECTIVELY.
C
      IF(M.EQ.1) THEN
        PHIS=WM1*PHI(N1)*S1+WM2*PHI(N2)*S2
        IF(PHIS.NE.0.)
     A  VEL((N-1)*NPH+NP)=DMOBI*DR/PHIS
      ENDIF
C
C***  BEGIN ADDITION FOR T2DM
C
      DVELM((M-1)*NCON*NPH+(N-1)*NPH+NP)=-DMOBI*DR
C.....STORE DARCY VELOCITIES FOR USE IN DISF.
C     SIGN CONVENTION IS OPPOSITE TO COMMON TOUGH2 USAGE.
C     IN TOUGH2, ALL VECTORIAL QUANTITIES ARE REFERRED TO LOCAL
C     COORDINATES, IN WHICH BY CONVENTION THE COORDINATE AXIS
C     POINTS FROM THE SECOND TO THE FIRST GRID BLOCK IN EACH
C     FLOW CONNECTION.
C     IN MESHMAKER Y-Z GRIDS, THE FIRST GRID BLOCK IN A CONNECTION
C     IS ALWAYS "LEFT" OR "ABOVE," SO THAT LOCAL COORDINATES POINT
C     LEFT OR UPWARD, AND FLUXES AND VELOCITY COMPONENTS ARE
C     POSITIVE WHEN THEY POINT "LEFT" OR "UP."
C     THE GLOBAL COORDINATES USED IN DISF POINT IN DIRECTION OF
C     INCREASING Y- AND Z-INDEX, I.E., TO THE RIGHT AND DOWNWARD.
C     THEREFORE, VECTOR COMPONENTS IN GLOBAL DISF COORDINATES
C     HAVE THE OPPOSITE SIGN AS IN LOCAL COORDINATES.
C     8-10-92 (KP)
C
C***  END OF ADDITION FOR T2DM
C
    7 CONTINUE
C
C
C-----REDEFINE INTERFACE DENSITY WITH UPSTREAM WEIGHTING.
      IF(MOP(18).EQ.0.OR.S1.EQ.0..OR.S2.EQ.0.)
     XRHOX=WM1*RHO1+WM2*RHO2
C-----FLUX IN PHASE NP.
      FNPM=DMOBI*RHOX*DR*AX
C
   31 CONTINUE
C
C-----OBTAIN FLUX FOR COMPONENT K IN PHASE NP.
      DO 4 K=1,NK
C
      XNPMK=WM1*PAR(N1L2NP+NB+K)+WM2*PAR(N2L2NP+NB+K)
C
CFIB .. COMPUTE VOCS FLUX IN THE GAS AND AQ. PHASES THROUGH CONNECTIONS
C    .. IT IS PRINTED IN THE OUTPUT FILE, BUT DIFFUSIVE FLUXES HAVE NOT 
C       YET BEEN ADDED !!!
        IF(M.NE.1) GO TO 401 
          IF(K.GT.1+NCG.AND.NP.EQ.1) FLOVG(N)=FLOVG(N)+XNPMK*FNPM 
          IF(K.GT.1+NCG.AND.NP.EQ.2) FLOVW(N)=FLOVW(N)+XNPMK*FNPM 
  401   CONTINUE 
CFIE
      F(K,M)=F(K,M)+XNPMK*FNPM
      IF(NK.EQ.1.OR.NB.NE.8) GOTO 4
C
C-----DIFFUSION OCCURS ONLY WHEN
C        - THERE ARE AT LEAST 2 COMPONENTS (NK > 1)
C        - PARAMETER NB >= 8 (INGREDIENTS OF DIFFUSIVITY STORED
C                           IN PAR(...+7) AND PAR(...+8)
C
      IF(MOP(24).EQ.0) THEN
C-----COME HERE TO COMPUTE BINARY DIFFUSION, USING HARMONIC WEIGHTING
C     FOR FULL MULTIPHASE DIFFUSIVE STRENGTH.
C     CODING IS FOR A 2-PHASE SYSTEM WITH PHASE 1 - GAS, PHASE 2-
C     LIQUID, AND ASSUMING EQUILIBRIUM PHASE PARTITIONING.
C
      IF(NP.NE.2) GOTO 4
C     ASSEMBLE TERMS ONLY WHEN "SECOND PHASE" (AQUEOUS) IS BEING
C     WORKED THROUGH.
      IF(FDDIAG(NP,K).EQ.0..AND.FDDIAG(NP-1,K).EQ.0.) GOTO 4
C     SKIP SECTION WHEN DIFFUSIVITIES ARE ZERO
C
      X1L=PAR(N1L2NP+NB+K)
      X2L=PAR(N2L2NP+NB+K)
      X1G=PAR(N1L2NP+NB+K-NBK)
      X2G=PAR(N2L2NP+NB+K-NBK)
      IF((X2L-X1L).NE.0.) THEN
         HGL=(X2G-X1G)/(X2L-X1L)
      ELSE
         HGL=0.
      ENDIF
      DXR=(X2L-X1L)/(D1+D2)
C
C.....CALCULATE ENTIRE COEFFICIENT TO MULTIPLY LIQUID MASS
C     FRACTION GRADIENT; FOR BOTH GRID BLOCKS
C.....LIQUID PHASE
         FDD=FDDIAG(NP,K)
         IF(FDD.LT.0.) THEN
            DICO1L=-PHI1*S1*RHO1*FDD
            DICO2L=-PHI2*S2*RHO2*FDD
         ELSE
            DICO1L=PAR(N1L2NP+7)*PAR(N1L2NP+8)*FDD
            DICO2L=PAR(N2L2NP+7)*PAR(N2L2NP+8)*FDD
         ENDIF
C.....GAS PHASE
         FDD=FDDIAG(NP-1,K)
            S1G=PAR(N1L2NP+1-NBK)
            S2G=PAR(N2L2NP+1-NBK)
         IF(FDD.LT.0.) THEN
            RHO1G=PAR(N1L2NP+4-NBK)
            RHO2G=PAR(N2L2NP+4-NBK)
            DICO1G=-PHI1*S1G*RHO1G*FDD
            DICO2G=-PHI2*S2G*RHO2G*FDD
         ELSE
            DICO1G=PAR(N1L2NP+7-NBK)*PAR(N1L2NP+8-NBK)*FDD
            DICO2G=PAR(N2L2NP+7-NBK)*PAR(N2L2NP+8-NBK)*FDD
            IF(BE.NE.0..AND.K.EQ.1) THEN
C.....COME HERE TO ASSIGN ENHANCEMENT FACTOR FOR VAPOR DIFFUSION.
               DICO1G=BE*PAR(N1L2NP+4-NBK)*PAR(N1L2NP+8-NBK)*FDD
               DICO2G=BE*PAR(N2L2NP+4-NBK)*PAR(N2L2NP+8-NBK)*FDD
C     VAPOR ENTHALPIES
               HV1=PAR(N1LM2+NSEC)
               HV2=PAR(N2LM2+NSEC)
            ENDIF
         ENDIF
      DICO1=DICO1L+HGL*DICO1G
      DICO2=DICO2L+HGL*DICO2G
C
C     1-10-01: ADD CONTRIBUTIONS OF A THIRD PHASE ("N" FOR NAPL)
      IF(NPH.EQ.3) THEN
      X1N=PAR(N1L2NP+NB+K+NBK)
      X2N=PAR(N2L2NP+NB+K+NBK)
         IF((X2L-X1L).NE.0.) THEN
            HNL=(X2N-X1N)/(X2L-X1L)
         ELSE
            HNL=0.
         ENDIF
C.....NAPL PHASE
         FDD=FDDIAG(NP+1,K)
            S1N=PAR(N1L2NP+1+NBK)
            S2N=PAR(N2L2NP+1+NBK)
         IF(FDD.LT.0.) THEN
            RHO1N=PAR(N1L2NP+4+NBK)
            RHO2N=PAR(N2L2NP+4+NBK)
            DICO1N=-PHI1*S1N*RHO1N*FDD
            DICO2N=-PHI2*S2N*RHO2N*FDD
         ELSE
            DICO1N=PAR(N1L2NP+7+NBK)*PAR(N1L2NP+8+NBK)*FDD
            DICO2N=PAR(N2L2NP+7+NBK)*PAR(N2L2NP+8+NBK)*FDD
         ENDIF
       DICO1=DICO1+HNL*DICO1N
       DICO2=DICO2+HNL*DICO2N
      ENDIF
C     END OF THIRD PHASE
C
C     HARMONICALLY WEIGHT THE INTERFACE DIFFUSIVITY.
      DEN=DICO1*D2 + DICO2*D1
      IF(DEN.EQ.0) THEN
          DICO=0.
      ELSE
          DICO=DICO1*DICO2*(D1+D2)/DEN
      ENDIF
C
      FDIFF=AX*DICO*DXR
      F(K,M)=F(K,M) + FDIFF
C
         IF(BE.NE.0..AND.K.EQ.1) THEN
C...........COME HERE FOR HEAT TRANSFER DUE TO ENHANCED
C           VAPOR DIFFUSION
            IF(FDIFF.GT.0..AND.S2G.NE.0.) GDIFF=FDIFF*HV2
            IF(FDIFF.LT.0..AND.S1G.NE.0.) GDIFF=FDIFF*HV1
            F(NK1,M)=F(NK1,M)+GDIFF
         ENDIF
C
C     OTHER HEAT TRANSFER DUE TO MASS DIFFUSION IS IGNORED.
            IF(M.EQ.1) THEN
C.....STORE AWAY DIFFUSIVE FLUXES "AT STATE POINT" FOR PRINTING.
                II=(N-1)*NPH*NK+(NP-1)*NK+K
                FDIF(II)=FDIFF
                IF(K.EQ.1) FVD(N)=FDIFF
            ENDIF
C
      IF(M.EQ.1.AND.MOP(3).GE.7) 
     XPRINT 189,K,DICO1L,DICO2L,DICO1G,DICO2G,HGL,DICO1,DICO2,
     XDICO,DXR
  189 FORMAT(/' K=',I1,'  DICO1L     DICO2L       DICO1G       DICO2G',
     X'        HGL         DICO1        DICO2         ',
     X'DICO         DXR'/(10(1X,E12.6)))
      IF(M.EQ.1.AND.MOP(3).GE.7) 
     XPRINT 187,DICO1N,DICO2N,HNL
  187 FORMAT(4X,'  DICO1N     DICO2N        HNL'/(10(1X,E12.6)))
C
C-----END OF MOP(24)=0 SECTION FOR HARMONIC WEIGHTING OF COUPLED MULTIPHASE
C     DIFFUSION STRENGTH
      ENDIF
C
      IF(MOP(24).EQ.1) THEN
C-----NOW A SECTION FOR COMPUTING BINARY DIFFUSION,
C     WITH SEPARATE HARMONIC WEIGHTING OF PHASE FLUXES
C
      IF(FDDIAG(NP,K).EQ.0.) GOTO 4
C
      X1=PAR(N1L2NP+NB+K)
      X2=PAR(N2L2NP+NB+K)
      DXR=(X2-X1)/(D1+D2)
C
C.....CALCULATE ENTIRE COEFFICIENT TO MULTIPLY MASS
C     FRACTION GRADIENT; FOR BOTH GRID BLOCKS
         FDD=FDDIAG(NP,K)
         IF(FDD.LT.0.) THEN
            DICO1=-PHI1*S1*RHO1*FDD
            DICO2=-PHI2*S2*RHO2*FDD
         ELSE
            DICO1=PAR(N1L2NP+7)*PAR(N1L2NP+8)*FDD
            DICO2=PAR(N2L2NP+7)*PAR(N2L2NP+8)*FDD
            IF(BE.NE.0..AND.K.EQ.1.AND.NP.EQ.1) THEN
C.....COME HERE TO ASSIGN ENHANCEMENT FACTOR FOR VAPOR DIFFUSION.
               DICO1=BE*PAR(N1L2NP+4)*PAR(N1L2NP+8)*FDD
               DICO2=BE*PAR(N2L2NP+4)*PAR(N2L2NP+8)*FDD
C     VAPOR ENTHALPIES
               HV1=PAR(N1LM2+NSEC)
               HV2=PAR(N2LM2+NSEC)
            ENDIF
         ENDIF
C     HARMONICALLY WEIGHT THE INTERFACE DIFFUSIVITY.
      DEN=DICO1*D2 + DICO2*D1
      IF(DEN.EQ.0) THEN
          DICO=0.
      ELSE
          DICO=DICO1*DICO2*(D1+D2)/DEN
      ENDIF
C
      FDIFF=AX*DICO*DXR
      F(K,M)=F(K,M) + FDIFF
C
         IF(BE.NE.0..AND.K.EQ.1.AND.NP.EQ.1) THEN
C...........COME HERE FOR HEAT TRANSFER DUE TO ENHANCED
C           VAPOR DIFFUSION
            IF(FDIFF.GT.0..AND.S2.NE.0.) GDIFF=FDIFF*HV2
            IF(FDIFF.LT.0..AND.S1.NE.0.) GDIFF=FDIFF*HV1
            F(NK1,M)=F(NK1,M)+GDIFF
         ENDIF
C
C     OTHER HEAT TRANSFER DUE TO MASS DIFFUSION IS IGNORED.
            IF(M.EQ.1) THEN
C.....STORE AWAY DIFFUSIVE FLUXES "AT STATE POINT" FOR PRINTING.
                II=(N-1)*NPH*NK+(NP-1)*NK+K
                FDIF(II)=FDIFF
                IF(K.EQ.1) FVD(N)=FDIFF
            ENDIF
C
      IF(M.EQ.1.AND.MOP(3).GE.7) PRINT 188,NP,K,DICO1,DICO2,DICO,DXR
  188 FORMAT(/' PHASE #',I2,'   COMPONENT #',I2,'   DICO1=',E12.6,
     X'   DICO2=',E12.6,'   DICO=',E12.6,'   DXR=',E12.6)
C
C-----END OF MOP(24)=1 SECTION WITH SEPARATE HARMONIC WEIGHTING OF
C     PHASE FLUXES
      ENDIF
C
    4 CONTINUE
C
      HNPM=WM1*H1+WM2*H2
C
C-----HEAT FLUX IN PHASE NP.
      F(NK1,M)=F(NK1,M)+HNPM*FNPM
C
C
      IF(MOP(3).LT.7) GOTO 190
      IF(MOP(3).LT.9.AND.M.GT.1) GOTO 190
      PRINT 999,N,NP,RHO1M,RHO2M
  999 FORMAT(' AT CONNECTION #',I3,'  NP=',I2,' MASS DENSITIES ARE',
     X'   RHO1M = ',E12.6,'   RHO2M = ',E12.6)
      PRINT 196,NP,N1L2NP,N2L2NP,PER1,PER2,PERI
  196 FORMAT(9X,'NP =',I2,'     SECONDARY INDICES (',I5,1H,,I5,1H),
     X15X,'    PER1 = ',E12.6,' PER2= ',E12.6,'   PERI = ',E12.6)
      PRINT 195,S1,REL1,VIS1,RHO1,H1,PCAP1
  195 FORMAT('       S1   = ',E12.6,'  REL1 = ',E12.6,'  VIS1 = ',E12.6,
     A'  RHO1 = ',E12.6,'  H1 = ',E12.6,'  PCAP1 = ',E12.6)
      PRINT 194,S2,REL2,VIS2,RHO2,H2,PCAP2
  194 FORMAT('       S2   = ',E12.6,'  REL2 = ',E12.6,'  VIS2 = ',E12.6,
     A'  RHO2 = ',E12.6,'  H2 = ',E12.6,'  PCAP2 = ',E12.6)
      PRINT 193,DMOBI,RHOX,DR,FNPM,HNPM
  193 FORMAT('       DMOBI= ',E12.6,'  RHOX = ',E12.6,'    DR = ',E12.6,
     X'  FNPM = ',E12.6,' HNPM= ',E12.6)
C
      PRINT 192,(F(K,M),K=1,NK1)
  192 FORMAT('     * FLOW TERMS',8(1X,E12.6))
  190 CONTINUE
C
C
      IF(M.NE.1) GOTO 3
C-----STORE FLUXES IN EACH PHASE AND HEAT FLUX.
      FLO((N-1)*NPH+NP)=FNPM
C
    3 CONTINUE
C
    2 CONTINUE
C
      GLO(N)=F(NK1,1)
C
C
C+++++ASSIGN ALL INTERFACE TERMS+++++++++++++++++++++++++++++++++
C
      DO 5 K=1,NEQ
C     K IS THE ROW INDEX WITHIN A BLOCK PERTAINING TO ELEMENT N1 OR N2.
C
C-----COMPUTE FLUX CONTRIBUTIONS TO RESIDUALS.
      IF(N1.LE.NELA)
     XR(N1LOC+K)=R(N1LOC+K)-FAC1*F(K,1)
      IF(N2.LE.NELA)
     XR(N2LOC+K)=R(N2LOC+K)+FAC2*F(K,1)
C
      DO 6 L=1,NEQ
C     L IS THE COLUMN INDEX WITHIN A BLOCK.
      IF(N1.GT.NELA .OR. N2.GT.NELA) GOTO 61
C-----OFF-DIAGONAL TERM IN EQUATION FOR ELEMENT N1, ARISING FROM
C     DEPENDENCE OF COMPONENT K-FLUX UPON VARIABLE L IN ELEMENT N2.
      IRN(NZ+1)=N1LOC+K
      IF(IAB.EQ.0) ICN(NZ+1)=N2LOC+L
      IF(IAB.NE.0) JVECT(NZ+1)=N2LOC+L
      CO(NZ+1)=FAC1*(F(K,L+1+NEQ)-F(K,1))/DELX(N2LOCP+L)
      NZ=NZ+1
C
C-----OFF-DIAGONAL TERM IN EQUATION FOR ELEMENT N2, ARISING FROM
C     DEPENDENCE OF COMPONENT K-FLUX UPON VARIABLE L IN ELEMENT N1.
      IRN(NZ+1)=N2LOC+K
      IF(IAB.EQ.0) ICN(NZ+1)=N1LOC+L
      IF(IAB.NE.0) JVECT(NZ+1)=N1LOC+L
      CO(NZ+1)=-FAC2*(F(K,L+1)-F(K,1))/DELX(N1LOCP+L)
      NZ=NZ+1
   61 CONTINUE
C
C
C-----DIAGONAL TERM IN EQUATION FOR ELEMENT N1, ARISING FROM
C     DEPENDENCE OF COMPONENT K-FLUX UPON VARIABLE L IN ELEMENT N1.
C
C     NOTE THAT EACH CONNECTION INVOLVING N1 WILL GENERATE A TERM
C     AT THE SAME MATRIX LOCATION.
      N1KL=(N1-1)*NEQ*NEQ+(K-1)*NEQ+L
      IF(N1.LE.NELA)
     XCO(N1KL)=CO(N1KL)+FAC1*(F(K,L+1)-F(K,1))/DELX(N1LOCP+L)
C
C-----DIAGONAL TERM IN EQUATION FOR ELEMENT N2, ARISING FROM
C     DEPENDENCE OF COMPONENT K-FLUX UPON VARIABLE L IN ELEMENT N2.
      N2KL=(N2-1)*NEQ*NEQ+(K-1)*NEQ+L
      IF(N2.LE.NELA)
     XCO(N2KL)=CO(N2KL)-FAC2*(F(K,L+1+NEQ)-F(K,1))/DELX(N2LOCP+L)
C
C
    6 CONTINUE
    5 CONTINUE
C
C+++++END OF ASSIGNMENT OF INTERFACE TERMS+++++++++++++++++++++++
C
    1 CONTINUE
C
C
C-----TEST FOR CONVERGENCE----------------------------------------------
C
      RERM=0.
      IF(MOP(3).GE.2) PRINT 203
  203 FORMAT(/' ===== RESIDUALS =====   MASS BALANCES FIRST,',
     X' ENERGY BALANCE LAST'/)
      NER=1
      KER=1
      DO10 N=1,NELA
      NLOC=(N-1)*NEQ
C
      IF(MOP(3).GE.2) PRINT 202,ELEM(N),(R(NLOC+K),K=1,NEQ)
  202 FORMAT('       AT ELEMENT *',A5,'*   ',8(1X,E12.6))
C
      DO10 K=1,NEQ
      NLM=NLOC+K
      DOA=ABS(DOLD(NLM))
      IF(DOA.LT.RE2) RER=R(NLM)/RE2
      IF(DOA.GE.RE2) RER=R(NLM)/DOLD(NLM)
      IF(ABS(RER).LE.RERM) GOTO 10
      RERM=ABS(RER)
      NER=N
      KER=K
   10 CONTINUE
C
C-----------------------------------------------------------------------
C
      RETURN
      END
C
      SUBROUTINE WMASSD(NP,N1L2NP,AMWM)
C----------------------------------------------------------------------------
C**   PARAMETERS
C
C       NP     - PHASE INDEX (1: GAS, 2: WATER, 3: NAPL)
C       N1L2NP - STARTING INDEX IN PAR ARRAY (INCL. INCREMENTING AND PHASE)
C
C**   ASSUMPTIONS/APPROXIMATIONS
C
C       GAS PHASE CONSISTS OF ALL COMPONENTS WITH THEIR PROPER MOLAR WEIGHT
C       LIQUID (AQUEOUS) PHASE HAS WATER AND NCG LUMPED AT WATER MOLAR WEIGHT
C       FOR NAPL PHASE THE WATER AND NCG CONTRIBUTIONS ARE NEGLECTED     
C----------------------------------------------------------------------------
      COMMON /GASLAW/ RGAS,AMS,AMA,CVGAS
      COMMON /MOLWT/ AMW(20)
      COMMON /SECPAR/ PAR(1)
      COMMON/NN/NEL,NCON,NOGN,NK,NEQ,NPH,NB,NK1,NEQ1,NBK,NSEC,NFLUX
      COMMON/NUMBER/ NHC
      COMMON/NCGN/NCG,NCGCODE(10)
C
      SAVE ICALL
      DATA ICALL/0/
      ICALL=ICALL+1
      IF(ICALL.EQ.1) WRITE(11,899)
C 899 FORMAT(6X,'WMASSD   1.00 AA   6 AUGUST    1997',6X,
C 899 FORMAT(6X,'WMASSD   1.00      5 JULY      2001',6X,
  899 FORMAT(6X,'WMASSD   2.00     23 AUGUST    2001',6X,
     X'CONVERT MOLAR DENSITIES TO MASS DENSITIES')
C
C**   CALCULATE THE MOLECULAR WEIGHT OF A MIXTURE
C
      AMWM=0.
      IF(NP.EQ.1) THEN
        DO 20 L=1,NCG
   20      AMWM = AMWM+ AMW(NHC+L)*PAR(N1L2NP+NB+1+L)
        AMWM = AMS*PAR(N1L2NP+NB+1) + AMWM
      ENDIF
C
      IF(NP.EQ.2) THEN
        DO 30 L=1,NCG
   30      AMWM = AMWM+ PAR(N1L2NP+NB+1+L)
        AMWM = AMS*(PAR(N1L2NP+NB+1) + AMWM )
      ENDIF
C
      DO 10 L = 1, NHC
        AMWM = AMWM + AMW(L)*PAR(N1L2NP+NB+1+NCG+L)
   10 CONTINUE
C
      IF(NP.EQ.3) THEN
         SUMINO=0.
         DO11 L=1,NHC
   11    SUMINO=SUMINO+PAR(N1L2NP+NB+1+NCG+L)
         IF(SUMINO.NE.0.) AMWM=AMWM/SUMINO
      ENDIF
C
C     PRINT 998,NP,N1L2NP,AMWM,(PAR(N1L2NP+NB+L),L=1,NK)
  998 FORMAT(' WMASSD ##### NP =',I2,'   N1L2NP =',I5,'   AMWM = ',
     XE12.6/' PAR = ',9(1X,E12.6))
C
      RETURN
      END
C
C
      SUBROUTINE AMOLW(JDLOC2,AMGAS,AMAQ,AMOIL)
C----------------------------------------------------------------------------
C---- CALCULATES THE MOLECULAR WEIGHT OF GAS, AQUEOUS
C     AND OIL PHASES BASED ON MOLAR COMPOSITION. 
C---- DEFINITIONS:
C---- JDLOC2: LOCATES BEGINNING OF THERMOD. VARIABLES FOR CURRENT ELEME.
C---- AMGAS : MOLECULAR WEIGHT OF THE GAS PHASE
C---- AMAQ  : MOLECULAR WEIGHT OF THE AQUEOUS PHASE
C---- AMOIL : MOLECULAR WEIGHT OF THE OIL PHASE
C-------------------------------------------------------------------------
      COMMON /GASLAW/ RGAS,AMS,AMA,CVGAS
      COMMON/MOLWT/AMW(20)
      COMMON/NUMBER/ NHC
      COMMON/NCGN/NCG,NCGCODE(10)
      COMMON/SECPAR/PAR(1)
      COMMON/NN/NEL,NCON,NOGN,NK,NEQ,NPH,NB,NK1,NEQ1,NBK,NSEC,NFLUX
C
      SAVE ICALL
      DATA ICALL/0/
      ICALL=ICALL+1
      IF(ICALL.EQ.1) WRITE(11,899)
C 899 FORMAT(6X,'AMOLW    1.00 AA   7 AUGUST    1997',6X,
C 899 FORMAT(6X,'AMOLW    1.10      9 AUGUST    2001',6X,
  899 FORMAT(6X,'AMOLW    2.0      23 AUGUST    2001',6X,
     X'CALCULATE MOLECULAR WEIGHT OF PHASE MIXTURES')
C
C ... IF A PHASE DOES NOT EXIST, AMPHASE= 0. !!
C
      NBK = NB + NK
C     GAS PHASE: CHECK IF SG>0
      AMGAS=0.
      IDG=JDLOC2+NB
      IF(PAR(JDLOC2+1).GT.0) THEN
        AMGAS=AMS*PAR(IDG+1)
        DO 15 I=1,NCG
           AMGAS=AMGAS+AMW(NHC+I)*PAR(IDG+1+I)
 15     CONTINUE
        DO 10 I=1,NHC
           AMGAS=AMGAS+AMW(I)*PAR(IDG+1+NCG+I)
 10     CONTINUE
      ENDIF
C
C     AQUEOUS PHASE: CHECK IF SW>0
      AMAQ=0.
      IDL=IDG + NBK
      IF(PAR(JDLOC2+NBK+1).GT.0) THEN
C     AQ. DENSITY IS COMPUTED NEGLECTING AIR CONC. !!!
        AMAQ = PAR(IDL+1)
        DO 25 I=1,NCG
           AMAQ =AMAQ + PAR(IDL+1+I)
 25     CONTINUE
        AMAQ =AMS*AMAQ
        DO 20 I=1,NHC
           AMAQ =AMAQ +AMW(I)*PAR(IDL+1+NCG+I)
 20     CONTINUE
      ENDIF
C
C     OIL PHASE: CHECK IF SO>0
C     NAPL DENSITY IS COMPUTED NEGLECTING NCG AND WATER!
      AMOIL=0.
      IDO=IDL + NBK
      IF(PAR(JDLOC2+2*NBK+1).GT.0) THEN
        SUMINO=0.
        DO 30 I=1,NHC
           AMOIL=AMOIL+AMW(I)*PAR(IDO+1+NCG+I)
           SUMINO=SUMINO+PAR(IDO+1+NCG+I)
 30     CONTINUE
        IF(SUMINO.NE.0)THEN
           AMOIL=AMOIL/SUMINO
        ELSE
           AMOIL=0.
        ENDIF
      ENDIF
C
      RETURN
      END
C
      SUBROUTINE QU
C
C-----THIS SUBROUTINE COMPUTES ALL TERMS ARISING FROM SINKS AND
C     SOURCES.
C
C          *****************************************************
C          *              MODULAR VERSION OF 1998              *
C          *****************************************************
C
C     GENERATION TYPE IS PROCESSED IN RFILE FOR CHECKING AND
C     ASSIGNMENT OF MAJOR GENERATION OPTIONS THROUGH ARRAY LCOM
C
C     TYPE IS CONVEYED TO QU FOR SUB-CHOICES, ESPECIALLY THE
C     ASSIGNMENT OF FILE NAMES TO TABLES OF FLOWING BOTTOMHOLE
C     PRESSURES, FOR GEOTHERMAL PRODUCTION AT SPECIFIED WELLHEAD
C     CONDITIONS
C
C     A NEW SUBROUTINE TTAB WAS WRITTEN TO PERFORM INTERPOLATION
C     ON TIME-DEPENDENT TABLES
C
C     NEW SUBROUTINES PHAS AND PHASD WERE WRITTEN TO DETERMINE
C     COMPOSITION AND FLOWING ENTHALPY OF MULTIPHASE FLUIDS
C
      COMMON/L3/CO(1)
      COMMON/DM/DELTEN,DELTEX,FOR,FORD
      COMMON/SECPAR/PAR(1)
      COMMON/SVZ/NOITE,MOP(24)
      COMMON/BCIJT/IHD,MAX,MID,SET
      COMMON/KONIT/KON,DELT,IGOOD
C
C$$$$$ COMMON BLOCKS FOR SINKS/SOURCES $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
C
      COMMON/G1/F1(1)
      COMMON/G2/F2(1)
      COMMON/G3/F3(1)
      COMMON/G4/ELEG(1)
      COMMON/G5/SOURCE(1)
      COMMON/G6/LTABG(1)
      COMMON/G7/G(1)
      COMMON/G8/EG(1)
      COMMON/G9/NEXG(1)
      COMMON/G10/TYPE(1)
      COMMON/G11/NGIND(1)
      COMMON/G12/LCOM(1)
      COMMON/G13/PI(1)
      COMMON/G14/PWB(1)
      COMMON/G15/HG(1)
      COMMON/G16/GPO(1)
      COMMON/G17/SDENS(1)
      COMMON/G18/SSAT(1)
      COMMON/G19/GVOL(1)
      COMMON/G20/HL(1)
      COMMON/G21/HS(1)
      COMMON/G22/QVGC(1)
      COMMON/G23/QVWC(1)
      COMMON/G24/QVOC(1)
      COMMON/G25/GRAD(1)
      COMMON/G26/FF(1)
C
C$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
C
      COMMON/E3/EVOL(1)
      COMMON/P1/X(1)
      COMMON/P2/DX(1)
      COMMON/P3/DELX(1)
      COMMON/P4/R(1)
      COMMON/NN/NEL,NCON,NOGN,NK,NEQ,NPH,NB,NK1,NEQ1,NBK,NSEC,NFLUX
      COMMON/CYC/KCYC,ITER,ITERC,TIMIN,SUMTIM,GF,TIMOUT
      COMMON/DFM/TIMAX,REDLT
      COMMON/TIM/ITI,DELAF,ITPR,TIS(100),ITCO,NOWTIM,DELTMX
      COMMON/BC/NELA
      COMMON/DKM/D(21,22)
      COMMON /G32AB/ EGP(1)
      COMMON /NUMBER/ NHC
      COMMON /NCGN/NCG,NCGCODE(10)
      COMMON /MOLWT/ AMW(20)
      COMMON /GASLAW/ RGAS,AMS,AMA,CVAIR
      COMMON/SOLU/HCWATNAPL,HCAIRNAPL,SOLWAT_OIL,SOLAIR_OIL
      COMMON/ECH/EOSN(20)
      CHARACTER EOSN*10
      CHARACTER ELEG*5,SOURCE*5,ITABA*1,TYPE*5,TYPEN*5
C
      SAVE ICALL
      DATA ICALL/0/
      ICALL=ICALL+1
      IF(ICALL.EQ.1) WRITE(11,899)
C 899 FORMAT(6X,'QU       1.0      22 JANUARY   1990',6X,
C 899 FORMAT(6X,'QU       1.02     18 FEBRUARY  1993',6X,
C 899 FORMAT(6X,'QU       1.1       5 NOVEMBER  1997',6X,
C 899 FORMAT(/6X,'QU       1.1      23 JANUARY   1998',6X,
C 899 FORMAT(/6X,'QU       1.1      26 OCTOBER   2000',6X,
C 899 FORMAT(/6X,'QU       1.1      10 NOVEMBER  2000',6X,
C 899 FORMAT(/6X,'QU       1.1      23 NOVEMBER  2000',6X,
C 899 FORMAT(/6X,'QU       1.1      21 DECEMBER  2000',6X,
C 899 FORMAT(/6X,'QU       1.1      12 APRIL     2001',6X,
C 899 FORMAT(/6X,'QU       2.0      23 AUGUST    2001',6X,
C 899 FORMAT(/6X,'QU       2.0      30 AUGUST    2001',6X,
  899 FORMAT(/6X,'QU       2.0       2 APRIL     2002',6X,
     X'ASSEMBLE ALL SOURCE AND SINK TERMS'/
     X47X,'"RIGOROUS" STEP RATE CAPABILITY FOR MOP(12) = 2,'/
     X47X,'AND CAPABILITY FOR FLOWING WELLBORE PRESSURE CORRECTIONS')
C
      IF(MOP(4).GE.1) PRINT 101,KCYC,ITER
  101 FORMAT(/' QQQQQQQQQQ SUBROUTINE QU QQQQQQQQQQQQQQQQQQQQ',
     X' --- [KCYC,ITER] = [',I4,',',I3,']')
C
      DO3901 N=1,NOGN
C
      J=NEXG(N)
      IF(J.EQ.0.OR.J.GT.NELA) GOTO3900
      FAC=FORD/EVOL(J)
C
      JLOC=(J-1)*NEQ
      JLOCP=(J-1)*NK1
C
      MN=LCOM(N)
C
      JMN=JLOC+MN
      JNK1=JLOC+NK1
      J2LOC=(J-1)*NSEC*NEQ1
C
      LTABA=IABS(LTABG(N))
      TYPEN=TYPE(N)
      ITABA=TYPEN(5:5)
C
C----- NOW JUMP TO PROPER SUB-SECTION
C
      IF(MN.LE.NK) GOTO 1
      IF(MN.EQ.NK1) GOTO 2
      LNK=MN-NK1
C
C***** CHOOSE GENERATION OPTION
      GOTO(3,4,5,6),LNK
C
    1 CONTINUE
C     ************************************************************
C     *  COME HERE WHEN PRODUCTION OR INJECTION OF A MASS        *
C     *  COMPONENT AT PRESCRIBED RATE IS SPECIFIED (LTABA = 1)   *
C     ************************************************************
C
      IF(LTABA.LE.1) THEN
         GN=G(N)
         GPO(N)=G(N)
C
         EGN=EG(N)
         IF(EOSN(1).EQ.'TMVOC1    '.AND.GN.GE.0.) THEN
C+++++     SECTION FOR TMVOC
C...          WATER IS INJECTED
           IF (MN.EQ.1) AMWGN=AMS*1.E-3
C...          VOC IS INJECTED  
           IF (MN.GT.NCG+1) AMWGN=AMW(MN-1-NCG)*1.E-3
C...          NCG IS INJECTED
           IF (MN.GT.1.AND.MN.LE.NCG+1)AMWGN=AMW(NHC+MN-1)*1.E-3
           GN = G(N) / AMWGN
           GPO(N)=GN
           EGN=EG(N)*AMWGN
           EGP(N)=EGN
         ENDIF
C
         IF(GN.GE.0..OR.(GN.LT.0..AND.ITABA.NE.' ')) THEN
C...........MASS INJECTION (GN > 0), OR PRODUCTION (GN < 0)
C           WITH PRESCRIBED ENTHALPY
            if(source(n)(:3) .eq. 'CIR') call circulate(n,mn,gn,egn)  ! added for circulation wells 05-09-2014
            R(JMN)=R(JMN)-FAC*GN
            IF(NEQ.EQ.NK1) R(JNK1)=R(JNK1)-FAC*GN*EGN
            GOTO 3900
         ELSE
C...........MASS PRODUCTION, WITH ENTHALPY TO BE DETERMINED
C           FROM CONDITIONS IN PRODUCING BLOCK
            CALL PHAS(N,J2LOC,GN,FAC)
            GOTO 3898
C
         ENDIF
      ELSE
C     ************************************************************
C     *  COME HERE WHEN A TABLE OF TIME-DEPENDENT GENERATION     *
C     *  DATA IS SPECIFIED (LTABA > 1)                           *
C     ************************************************************
C
      CALL TTAB(N,LTABA,ITABA,GN,EGN)
      IF(IGOOD.NE.0) RETURN
C
      G(N)=GN
      GPO(N)=GN
C
       IF(EOSN(1).EQ.'TMVOC1    '.AND.GN.GE.0.) THEN
C+++++   SECTION FOR TMVOC
C...          WATER IS INJECTED
           IF (MN.EQ.1) AMWGN=AMS*1.E-3
C...          VOC IS INJECTED
           IF (MN.GT.NCG+1) AMWGN=AMW(MN-1-NCG)*1.E-3
C...          NCG IS INJECTED
           IF (MN.GT.1.AND.MN.LE.NCG+1)AMWGN=AMW(NHC+MN-1)*1.E-3
C
           GN = GN / AMWGN
           GPO(N)=GN
           EGN=EGN*AMWGN
           EGP(N)=EGN
       ENDIF
C
Cbug!!IF(ITABA.NE.' ') THEN
      IF(GN.GE.0..OR.(GN.LT.0..AND.ITABA.NE.' ')) THEN
         EG(N)=EGN
C...........MASS INJECTION (GN > 0), OR PRODUCTION (GN < 0)
C           WITH PRESCRIBED ENTHALPY
            if(source(n)(:3) .eq. 'CIR') call circulate(n,mn,gn,egn)  ! added for circulation wells 05-09-2014
            R(JMN)=R(JMN)-FAC*GN
            IF(NEQ.EQ.NK1) R(JNK1)=R(JNK1)-FAC*GN*EGN
            GOTO 3900
      ELSE
         CALL PHAS(N,J2LOC,GN,FAC)
         GOTO 3898
      ENDIF
C
C
      ENDIF
C
    2 CONTINUE
C     ************************************************************
C     *  COME HERE WHEN WITHDRAWAL OR INJECTION OF HEAT          *
C     *  AT PRESCRIBED RATE IS SPECIFIED                         *
C     ************************************************************
C
C
C-----IGNORE HEAT SINKS/SOURCES WHEN NOT SOLVING ENERGY EQUATION.
      IF(NEQ.LT.NK1) GOTO 3900
C
      IF(LTABA.GT.1) THEN
C     ************************************************************
C     * COME HERE WHEN A TABLE OF TIME-DEPENDENT GENERATION      *
C     * DATA IS SPECIFIED (LTABA > 1)                            *
C     ************************************************************
C
         CALL TTAB(N,LTABA,ITABA,GN,EGN)
         IF(IGOOD.NE.0) RETURN
         G(N)=GN
      ENDIF
C
      GPO(N)=G(N)
      R(JMN)=R(JMN)-FAC*G(N)
C
      GOTO 3900
C
    3 CONTINUE
C     ************************************************************
C     *  DELIVERABILITY OPTION: PRODUCTION WELL WITH SPECIFIED   *
C     *  DOWNHOLE PRESSURE; A SIMPLE GRAVITY CORRECTION FOR      *
C     *  FLOWING WELLBORE PRESSURE MAY BE PERFORMED*             *
C     ************************************************************
C
C.....OPTIONALLY PERFORM SIMPLE GRAVITY CORRECTION FOR FLOWING
C     DOWNHOLE PRESSURE OF MULTI-FEEDZONE WELLS.
      IF(ITER.EQ.1.AND.LTABA.GT.1) CALL GCOR(N,LTABA)
C
C.....OBTAIN SOURCE RATE, PHASE COMPOSITION, FLOWING ENTHALPY
      CALL PHASD(N,JLOCP,J2LOC,FAC)
C
      IF(EOSN(1).NE.'TMVOC1    ') THEN
C+++++   SECTION FOR TMVOC
         GPO(N)=G(N)
      ENDIF 
      GOTO 3898
C
    4 CONTINUE
C     ************************************************************
C     *  PRODUCTION WELL WITH SPECIFIED WELLHEAD PRESSURE, AND   *
C     *  FLOWING WELLBORE PRESSURE CORRECTION                    *
C     ************************************************************
C
C.....ON FIRST CALL, INITIALIZE STARTING GUESS FOR WELL RATE
      IF(ICALL.EQ.1) SDENS(N)=10.
C
C.....OBTAIN SOURCE RATE, PHASE COMPOSITION, FLOWING ENTHALPY
      CALL PHASF(N,JLOCP,J2LOC,FAC)
C
      GPO(N)=G(N)
      GOTO 3898
C
    5 CONTINUE
C-----COME HERE IF VOLUMETRIC PRODUCTION RATE IS SPECIFIED.
C
    6 CONTINUE
C
C
 3898 CONTINUE
C
C-----ASSIGN TERMS FOR LINEAR EQUATIONS.
C
      DO 105 K=1,NEQ
      R(JLOC+K)=R(JLOC+K)+D(K,1)
C
      DO 106 L=1,NEQ
      JKL=(J-1)*NEQ*NEQ+(K-1)*NEQ+L
      CO(JKL)=CO(JKL)-(D(K,L+1)-D(K,1))/DELX(JLOCP+L)
  106 CONTINUE
C
  105 CONTINUE
C
 3900 CONTINUE
C
      IF(MOP(4).GE.2) PRINT 102,ELEG(N),SOURCE(N),G(N),EG(N),
     A(FF((N-1)*NPH+NP),NP=1,NPH)
  102 FORMAT(9H ELEMENT ,A5,10H   SOURCE ,A5,21H   ---   FLOW RATE = ,E1
     A2.6,23H   SPECIFIC ENTHALPY = ,E12.6/8H FNP1 = ,E12.6,10H   FNP2 =
     B ,E12.6,10H   FNP3 = ,E12.6,10H   FNP4 = ,E12.6)
        IF(MOP(4).GE.3.) PRINT 104, NEXG(N),FAC,LCOM(N),AMWGN,R(JMN)
  104 FORMAT(8H NEXG = ,I5,7H FAC = ,E12.6,8H LCOM = ,I5,9H AMWGN = ,
     AE12.6,5H R = ,E12.6)

C
C
 3901 CONTINUE
      RETURN
      END
C
      SUBROUTINE CONVER
C
C-----THIS SUBROUTINE IS CALLED AFTER SUCCESFULL COMPLETION OF
C     A TIME STEP.
C     IT UPDATES PRIMARY VARIABLES, AND DEFINES THE NEXT TIME STEP.
C
C
      COMMON/SECPAR/PAR(1)
      COMMON/E2/MATX(1)
      COMMON/E4/PHI(1)
      COMMON/E5/P(1)
      COMMON/E6/T(1)
      COMMON/P1/X(1)
      COMMON/P2/DX(1)
C
      COMMON/G7/G(1)
      COMMON/G12/LCOM(1)
      COMMON/G15/HG(1)
C
      COMMON/SOLI/COM(27),EXPAN(27),CDRY(27),TORT(27),GK(27)
      COMMON/KC/KC
      COMMON/DFM/TIMAX,REDLT
      COMMON/SVZ/NOITE,MOP(24)
      COMMON/DM/DELTEN,DELTEX,FOR,FORD
      COMMON/NN/NEL,NCON,NOGN,NK,NEQ,NPH,NB,NK1,NEQ1,NBK,NSEC,NFLUX
      COMMON/DLT/NDLT,DLT(100)
      COMMON/KONIT/KON,DELT,IGOOD
      COMMON/CYC/KCYC,ITER,ITERC,TIMIN,SUMTIM,GF,TIMOUT
      COMMON/TIM/ITI,DELAF,ITPR,TIS(100),ITCO,NOWTIM,DELTMX
      COMMON/BC/NELA
C....FOR TMVOC
      COMMON/ECH/EOSN(20)
      COMMON /NEW/ INDICAT(1)
      COMMON /NEW0/ INDCAT0(1)
      common/intype/inre
      CHARACTER EOSN*10
C
      DIMENSION DXM(21)
      SAVE ICALL
      DATA ICALL/0/
      ICALL=ICALL+1
      IF(ICALL.EQ.1) WRITE(11,899)
C 899 FORMAT(/6X,'CONVER   1.0       4 MARCH     1991',6X,
C 899 FORMAT(/6X,'CONVER   1.0      25 OCTOBER   2000',6X,
  899 FORMAT(/6X,'CONVER   1.0       2 MAY       2002',6X,
     X'UPDATE PRIMARY VARIABLES AFTER CONVERGENCE IS ACHIEVED')
C
      DO10 K=1,NK1
   10 DXM(K)=0.
C
      DO3 N=1,NELA
      NLOC=(N-1)*NK1
      NLOC2=(N-1)*NSEC*NEQ1
C-----COMPUTE CHANGES IN POROSITY.
      PHIN=PHI(N)
      NMAT=MATX(N)
      DPHI=PHIN*(COM(NMAT)*DX(NLOC+1)+EXPAN(NMAT)*(PAR(NLOC2+NSEC-1)
     A-T(N)))
      PHI(N)=PHIN+DPHI
C
C-----UPDATE ELEMENT PRESSURES AND TEMPERATURES.
      P(N)=X(NLOC+1)+DX(NLOC+1)
      T(N)=PAR(NLOC2+NSEC-1)
C
c     IF(EOSN(1).EQ.'TMVOC1    ') THEN
      if(inre.ne.0) then
C+++++ SECTION FOR TMVOC
C..... 8-12-97: STORE LATEST CONVERGED PHASE INDICES IN INDCAT0
       INDCAT0(N)=INDICAT(N)
      END IF
C
C-----INCREMENT PRIMARY VARIABLES.
      DO3 M=1,NEQ
      NLM=NLOC+M
      X(NLM)=X(NLM)+DX(NLM)
      DXM(M)=MAX(DXM(M),ABS(DX(NLM)))
    3 CONTINUE
C
C-----FOR PERCENTAGE INJECTION, ASSIGN INJECTION RATE FOR NEXT
C     TIME STEP.
C%    DO 30 N=1,NOGN
C%    IF(LCOM(N).NE.NEQ+4) GOTO 30
C%    G(N)=-HG(N)*G(N-1)
C% 30 CONTINUE
C
      SUMTIM=SUMTIM+DELTEX
      IF(TIMAX.NE.0..AND.TIMAX.EQ.SUMTIM) NOWTIM=1
C-----AFTER CONVERGENCE UPDATE TOTAL TIME AND ASSIGN NEW TIME STEP.
      IF(NDLT.EQ.0) GOTO20
      IF(KC+1.GT.8*NDLT.OR.DLT(KC+1).EQ.0.) GOTO20
C-----IF NO FURTHER TIME STEP INSTRUCTIONS ARE PROVIDED, KEEP
C     GOING WITH LAST TIME STEP.
C-----COME HERE FOR NEW TIME STEP ASSIGNMENT.
      DELT=DLT(KC+1)
      GOTO 22
   20 DELT=DELTEX
      IF(ITER.LE.MOP(16)) DELT=2.*DELTEX
   22 IF(TIMAX.NE.0.) DELT=MIN(DELT,TIMAX-SUMTIM)
      IF(DELTMX.NE.0.) DELT=MIN(DELT,DELTMX)
C
      RETURN
      END
C
C
      SUBROUTINE WRIFI
C
C-----THIS SUBROUTINE IS CALLED AT THE COMPLETION OF A TOUGH2-RUN.
C     IT GENERATES A FILE *SAVE* TO BE USED FOR RESTARTING.
C
      COMMON/E1/ELEM(1)
      COMMON/E4/PHI(1)
      COMMON/P1/X(1)
      COMMON/NN/NEL,NCON,NOGN,NK,NEQ,NPH,NB,NK1,NEQ1,NBK,NSEC,NFLUX
      COMMON/CYC/KCYC,ITER,ITERC,TIMIN,SUMTIM,GF,TIMOUT
      COMMON/SOLID/NM,DM(27),POR(27),PER(3,27),CWET(27),SH(27)
      COMMON/SOCH/MAT(27)
      COMMON/BALA/TFM0(27),TVM0(27),TFE0(27),TVE0(27),TSE0(27)
      COMMON/POV6/TSTART
C.... FOR TMVOC1
      COMMON /NEW/ INDICAT(1)
      COMMON /NEW0/ INDCAT0(1)
C
C$$$$$$$$$ COMMON BLOCKS FOR HYSTERESIS MODEL $$$$$$$$$$$$$$$$$$$$$$$$$$
C
      COMMON/KHYST1/IHYSTER,KCYCX,IEHYS(16)
      COMMON/KHYST2/ICURV(1)
      COMMON/KHYST3/KTMPO(1)
      COMMON/KHYST4/irun(1)
      COMMON/KHYST5/IFLG1(1)
      COMMON/KHYST6/KFLAG(1)
      COMMON/SAVHYS1/SOLDH(1)
      COMMON/SAVHYS2/HOLD(1)
      COMMON/TMPHYS1/STMPO(1)
      COMMON/TMPHYS2/HTMPO(1)
      COMMON/HYS11/STRN1(1)
      COMMON/HYS12/STRN2(1)
      COMMON/HYS13/STRN3(1)
      COMMON/HYS21/HTRN1(1)
      COMMON/HYS22/HTRN2(1)
      COMMON/HYS23/HTRN3(1)
      COMMON/HYS31/SCMP1(1)
      COMMON/HYS32/SCMP2(1)
      COMMON/HYS33/BB2(1)
      COMMON/TORBENR1/SEOT(1)
      COMMON/TORBENR2/SETRN1(1)
      COMMON/TORBENR3/SOR(1)
      COMMON/TORBENR4/SOROLD(1)
      COMMON/TORBENR5/SORTMPO(1)
      common/hyssav1/strn1old(1)
      common/hyssav2/strn1tmpo(1)
      common/hyssav3/strn2old(1)
      common/hyssav4/strn2tmpo(1)
      common/hyssav5/strn3old(1)
      common/hyssav6/strn3tmpo(1)
      common/hyssav7/htrn1old(1)
      common/hyssav8/htrn1tmpo(1)
      common/hyssav9/htrn2old(1)
      common/hyssav10/htrn2tmpo(1)
      common/hyssav11/htrn3old(1)
      common/hyssav12/htrn3tmpo(1)
      common/hyssav13/scmp1old(1)
      common/hyssav14/scmp1tmpo(1)
      common/hyssav15/scmp2old(1)
      common/hyssav16/scmp2tmpo(1)
      common/hyssav17/bb2old(1)
      common/hyssav18/bb2tmpo(1)
      common/hyssa21/ hmin(1)
      common/hyssa22/ hminold(1)
      common/hyssa23/ hmintmpo(1)
      common/hyssa31/ shmin(1)
      common/hyssa32/ shminold(1)
      common/hyssa33/ shmintmpo(1)    
      common/hyssa41/ hminp(1)
      common/hyssa42/ hminpold(1)
      common/hyssa43/ hminptmpo(1)

      common/intype/inre
      COMMON/ECH/EOSN(20)
      CHARACTER EOSN*10
C
      CHARACTER*5 ELEM,MAT
C
      SAVE ICALL
      DATA ICALL/0/
      ICALL=ICALL+1
      IF(ICALL.EQ.1) WRITE(11,899)
C 899 FORMAT(6X,'WRIFI    1.0      22 JANUARY   1990',6X,
c 899 FORMAT(6X,'WRIFI    1.0      12 APRIL     2001',6X,
c 899 FORMAT(6X,'WRIFI    1.0       2 MAY       2002',6X,
  899 FORMAT(6X,'WRIFI    2.1       1 JULY      2011',6X,
     X'AT THE COMPLETION OF A TOUGH2 RUN, WRITE PRIMARY VARIABLES',
     X' ON FILE *SAVE*; SAVE HISTORY IF HYSTERETIC CURVES')
C
      KCYC=KCYC-1
C
      REWIND 7
      PRINT 5,KCYC,SUMTIM
    5 FORMAT(/' WRITE FILE *SAVE* AFTER',I5,' TIME STEPS  ---  THE',
     X' TIME IS ',E12.6,' SECONDS'/)
      WRITE(7,1) NEL,SUMTIM
    1 FORMAT(31HINCON -- INITIAL CONDITIONS FOR,I5,17H ELEMENTS AT TIME,
     AE14.6)
      DO 10 N=1,NEL
      NLOC=(N-1)*NK1
C
c     IF(EOSN(1).EQ.'TMVOC1    ') THEN
      if(inre.ne.0) then
C+++++ SECTION FOR TMVOC1
       WRITE(7,15) ELEM(N),PHI(N),INDICAT(N),(X(NLOC+I),I=1,NK1)
      ELSE
       WRITE(7,2) ELEM(N),PHI(N),(X(NLOC+I),I=1,NK1)
      END IF
   15 FORMAT (A5,10X,E15.8,I2/(4E20.13))
    2 FORMAT(A5,10X,E15.8/(4E20.13))
CFiHysteresisB
      IF (IHYSTER.LT.0)
     &   WRITE(7,7003) SOLDH(N),HOLD(N),ICURV(N),sorold(n),STRN1old(N),
     &   STRN2old(N),STRN3old(N),HTRN1old(N),HTRN2old(N),HTRN3old(N),
     &   SCMP1old(N),SCMP2old(N),BB2old(N),hminold(n),shminold(n),
     &   hminpold(n)
 7003 FORMAT(2E20.14,I5,15x,e20.14,4(/,3E20.14))
CFiHysteresisE

   10 CONTINUE
C
      WRITE(7,3)
    3 FORMAT('+++  ')
C
      WRITE(7,4) KCYC,ITERC,NM,TSTART,SUMTIM
    4 FORMAT(3I5,2E15.8)
C
      ENDFILE 7
C
      RETURN
      END
C
      SUBROUTINE OUTDF
C
C-----THIS SUBROUTINE GENERATES A PRINTOUT OF DIFFUSIVE FLUXES.
C
      COMMON/C1/NEX1(1)
      COMMON/C2/NEX2(1)
      COMMON/C9/ELEM1(1)
      COMMON/C10/ELEM2(1)
      COMMON/FMOLDIF/FDIF(1)
      COMMON/TITLE/ TITLE
      COMMON/SVZ/NOITE,MOP(24)
      COMMON/KONIT/KON,DELT,IGOOD
      COMMON/NN/NEL,NCON,NOGN,NK,NEQ,NPH,NB,NK1,NEQ1,NBK,NSEC,NFLUX
      COMMON/CYC/KCYC,ITER,ITERC,TIMIN,SUMTIM,GF,TIMOUT
      COMMON/ECH/EOSN(20)
      CHARACTER*1 H0
      CHARACTER*80 TITLE
      CHARACTER PHAC*12
      CHARACTER*5 ELEM1,ELEM2
      COMMON/FF/H1
      CHARACTER*1 H1
      CHARACTER*3 IJ(20),IJNPH
      CHARACTER EOSN*10
C
      SAVE ICALL,H0,PHAC,IJ,NPHD
      DATA H0/' '/
      DATA ICALL/0/
      DATA PHAC/'  PHASE COMP'/
      DATA IJ/'-1-','-2-','-3-','-4-','-5-','-6-','-7-','-8-',
     &        '-9-','-10','-11','-12','-13','-14','-15','-16',
     &        '-17','-18','-19','-20'/
      DATA IJNPH/'ALL'/
C
C     DEFINE ARITHMETIC FUNCTION II FOR CONVENIENCE.
C
      II(IKK,INP,IN)=(IN-1)*NPHD*NK+(INP-1)*NK+IKK
C
      ICALL=ICALL+1
      IF(ICALL.EQ.1) WRITE(11,899)
C 899 FORMAT(/6X,'OUTDF    0.5 !     6 SEPTEMBER 1995',6X,
C 899 FORMAT(/6X,'OUTDF    0.5 !     7 JANUARY   1999',6X,
C 899 FORMAT(/6X,'OUTDF    1.0      16 JUNE      1999',6X,
C 899 FORMAT(/6X,'OUTDF    1.0      12 NOVEMBER  1999',6X,
C 899 FORMAT(/6X,'OUTDF    1.0       8 JANUARY   2001',6X,
C 899 FORMAT(/6X,'OUTDF    1.0       5 APRIL     2001',6X,
  899 FORMAT(/6X,'OUTDF    1.0      11 OCTOBER   2001',6X,
     X'PRINT OUT INTERBLOCK DIFFUSIVE FLOW RATES.')
C
      NPHD=NPH
      IF(EOSN(1).EQ.'EWASG     ') THEN
C....    AVOID PRINTING OF DIFFUSION FLUXES IN THE SOLID SALT PHASE
         NPHD=NPH-1
      ENDIF
C-----PRINT A SHORT HEADER.
C
      IF(EOSN(1).EQ.'TMVOC1    ') THEN
         PRINT 5000,H1,TITLE,KCYC,ITER,SUMTIM,'MOLES'
      ELSE
         PRINT 5000,H1,TITLE,KCYC,ITER,SUMTIM,'KG'
      ENDIF
C
      IF(MOP(24).EQ.1) THEN
      PRINT 5060,H0,((PHAC,I=1,NK),J=1,NPHD)
      PRINT 5061,((IJ(J),IJ(I),I=1,NK),J=1,NPHD)
C
      DO 3030 N=1,NCON
        IF (NEX1(N).EQ.0.OR.NEX2(N).EQ.0) GO TO 3030
C
        IF (MOD(N,57).EQ.53) PRINT 5062,H1,((PHAC,I=1,NK),J=1,NPHD)
        IF (MOD(N,57).EQ.53) PRINT 5061,((IJ(J),IJ(I),I=1,NK),J=1,NPHD)
C
        PRINT 5071,ELEM1(N),ELEM2(N),((FDIF(II(KK,NP,N)),KK=1,NK),
     &    NP=1,NPHD)
C
C
 3030   CONTINUE
C
      PRINT 5140
      ENDIF
C
      IF(MOP(24).EQ.0) THEN
      PRINT 5060,H0,(PHAC,I=1,NK)
      PRINT 5061,(IJNPH,IJ(I),I=1,NK)
C
      DO 3031 N=1,NCON
        IF (NEX1(N).EQ.0.OR.NEX2(N).EQ.0) GO TO 3031
C
        IF (MOD(N,57).EQ.53) PRINT 5062,H1,(PHAC,I=1,NK)
        IF (MOD(N,57).EQ.53) PRINT 5061,(IJNPH,IJ(I),I=1,NK)
C
CFIX    PRINT 5071,ELEM1(N),ELEM2(N),(FDIF(II(KK,NPHD,N)),KK=1,NK)
        PRINT 5071,ELEM1(N),ELEM2(N),(FDIF(II(KK,2,N)),KK=1,NK)
C
C
 3031   CONTINUE
C
      PRINT 5140
      ENDIF
C
C---*----1----*----2----*----3----*----4----*----5----*----6----*----7----*----8
 5000 FORMAT(A1/10X,A80//80X,'KCYC =',I5,'  -  ITER =',I5,
     1  '  -  TIME =',E12.6/
     &' MASS FLOW RATES (',A,'/S) FROM DIFFUSION'/)
 5060 FORMAT(A1,'ELEM1 ELEM2',10A12)
 5061 FORMAT(12X,10('   ',A3,2X,A3,' ')/)
 5062 FORMAT(A1/' ELEM1 ELEM2',10A12)
 5071 FORMAT(1X,A5,1X,A5,(10(1X,E11.5)))
 5140 FORMAT(' ',131('@'))
C
      RETURN
      END

c************************************************************************
      SUBROUTINE CIRCULATE(N,MN,GN,EGN)

C --- Recirculation: Inject fluid extracted from a production well
C     Production well (sink) must be prescribed first in GENER block
C     Use source code name "CIRxx" for circulation injection well,
C     where "xx" is the order number of the linked production well
C     in the GENER block
C     For the injection well, specify component that shall be reinjected,
C     an arbitrary positive injection rate (to be replaced by the production rate
C     of the corresponding component), and an enthalpy if different from that
C     of the produced fluid.
C     Repeat for all components that shall be reinjected, using the same
C     element and source code names.
C$$$$$ COMMON BLOCKS FOR SINKS/SOURCES $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
C
      COMMON/G5/SOURCE(1)
      COMMON/G7/G(1)
      COMMON/G8/EG(1)
      COMMON/G9/NEXG(1)
      COMMON/G16/GPO(1)
      COMMON/G26/FF(1)
      COMMON/G32AB/EGP(1)
      COMMON/G25/GRAD(1)
C
C$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
C
      COMMON/SECPAR/PAR(1)
      COMMON/NN/NEL,NCON,NOGN,NK,NEQ,NPH,NB,NK1,NEQ1,NBK,NSEC,NFLUX
      CHARACTER SOURCE*5
c
      SAVE ICALL
      DATA ICALL/0/
      ICALL=ICALL+1
      IF(ICALL.EQ.1) THEN
        WRITE(11,899)
          DO 1001 I=1,NOGN
          EGP(I)= EG(I)
          GRAD(I)= G(I)
 1001     CONTINUE
  899 FORMAT(6X,'Circulate     1.0       9 May  2014',6X,
     x'calculate composition and enthalpy of circulation fluid')
      ENDIF

      READ(SOURCE(N)(4:5),'(I2)',IOSTAT=IOS) NPRODWELL
      NNP=(NPRODWELL-1)*NPH
      J=NEXG(NPRODWELL)
      JLOC2=(J-1)*NSEC*NEQ1
      FFT=0.0D0
      XMP=0.0D0
      DO 1000 I=1,NPH
         JLOC2NP=JLOC2+(I-1)*NBK+NB
         FFT=FFT+FF(NNP+I)
         XMP=XMP+FF(NNP+I)*PAR(JLOC2NP+MN)
 1000 CONTINUE
      GN=-GPO(NPRODWELL)*XMP/FFT*GRAD(N)
      IF (ABS(EGP(N)).LT.1.0E-10) EGN=EG(NPRODWELL)
      GPO(N)=GN
      G(N)=GN
      EG(N)=EGN
      RETURN
      END

